@@ -10,7 +10,8 @@ use crate::{
 	trace,
 	util::{
 		errors::{
-			wrap, AnyError, OAuthError, RefreshTokenNotAvailableError, StatusError, WrappedError,
+			wrap, AnyError, CodeError, OAuthError, RefreshTokenNotAvailableError, StatusError,
+			WrappedError,
 		},
 		input::prompt_options,
 	},
@@ -20,7 +21,7 @@ use async_trait::async_trait;
 use chrono::{DateTime, Duration, Utc};
 use gethostname::gethostname;
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
-use std::{cell::Cell, fmt::Display, path::PathBuf, sync::Arc};
+use std::{cell::Cell, fmt::Display, path::PathBuf, sync::Arc, thread};
 use tokio::time::sleep;
 use tunnels::{
 	contracts::PROD_FIRST_PARTY_APP_ID,
@@ -49,7 +50,7 @@ struct AuthenticationError {
 	error_description: Option<String>,
 }
 
-#[derive(clap::ArgEnum, Serialize, Deserialize, Debug, Clone, Copy)]
+#[derive(clap::ValueEnum, Serialize, Deserialize, Debug, Clone, Copy)]
 pub enum AuthProvider {
 	Microsoft,
 	Github,
@@ -172,9 +173,9 @@ pub struct Auth {
 }
 
 trait StorageImplementation: Send + Sync {
-	fn read(&mut self) -> Result<Option<StoredCredential>, WrappedError>;
-	fn store(&mut self, value: StoredCredential) -> Result<(), WrappedError>;
-	fn clear(&mut self) -> Result<(), WrappedError>;
+	fn read(&mut self) -> Result<Option<StoredCredential>, AnyError>;
+	fn store(&mut self, value: StoredCredential) -> Result<(), AnyError>;
+	fn clear(&mut self) -> Result<(), AnyError>;
 }
 
 // unseal decrypts and deserializes the value
@@ -210,6 +211,66 @@ const KEYCHAIN_ENTRY_LIMIT: usize = 128 * 1024;
 
 const CONTINUE_MARKER: &str = "<MORE>";
 
+/// Implementation that wraps the KeyringStorage on Linux to avoid
+/// https://github.com/hwchen/keyring-rs/issues/132
+struct ThreadKeyringStorage {
+	s: Option<KeyringStorage>,
+}
+
+impl ThreadKeyringStorage {
+	fn thread_op<R, Fn>(&mut self, f: Fn) -> Result<R, AnyError>
+	where
+		Fn: 'static + Send + FnOnce(&mut KeyringStorage) -> Result<R, AnyError>,
+		R: 'static + Send,
+	{
+		let mut s = match self.s.take() {
+			Some(s) => s,
+			None => return Err(CodeError::KeyringTimeout.into()),
+		};
+
+		// It seems like on Linux communication to the keyring can block indefinitely.
+		// Fall back after a 5 second timeout.
+		let (sender, receiver) = std::sync::mpsc::channel();
+		let tsender = sender.clone();
+
+		thread::spawn(move || sender.send(Some((f(&mut s), s))));
+		thread::spawn(move || {
+			thread::sleep(std::time::Duration::from_secs(5));
+			let _ = tsender.send(None);
+		});
+
+		match receiver.recv().unwrap() {
+			Some((r, s)) => {
+				self.s = Some(s);
+				r
+			}
+			None => Err(CodeError::KeyringTimeout.into()),
+		}
+	}
+}
+
+impl Default for ThreadKeyringStorage {
+	fn default() -> Self {
+		Self {
+			s: Some(KeyringStorage::default()),
+		}
+	}
+}
+
+impl StorageImplementation for ThreadKeyringStorage {
+	fn read(&mut self) -> Result<Option<StoredCredential>, AnyError> {
+		self.thread_op(|s| s.read())
+	}
+
+	fn store(&mut self, value: StoredCredential) -> Result<(), AnyError> {
+		self.thread_op(move |s| s.store(value))
+	}
+
+	fn clear(&mut self) -> Result<(), AnyError> {
+		self.thread_op(|s| s.clear())
+	}
+}
+
 #[derive(Default)]
 struct KeyringStorage {
 	// keywring storage can be split into multiple entries due to entry length limits
@@ -222,7 +283,7 @@ macro_rules! get_next_entry {
 		match $self.entries.get($i) {
 			Some(e) => e,
 			None => {
-				let e = keyring::Entry::new("vscode-cli", &format!("vscode-cli-{}", $i));
+				let e = keyring::Entry::new("vscode-cli", &format!("vscode-cli-{}", $i)).unwrap();
 				$self.entries.push(e);
 				$self.entries.last().unwrap()
 			}
@@ -231,7 +292,7 @@ macro_rules! get_next_entry {
 }
 
 impl StorageImplementation for KeyringStorage {
-	fn read(&mut self) -> Result<Option<StoredCredential>, WrappedError> {
+	fn read(&mut self) -> Result<Option<StoredCredential>, AnyError> {
 		let mut str = String::new();
 
 		for i in 0.. {
@@ -239,7 +300,7 @@ impl StorageImplementation for KeyringStorage {
 			let next_chunk = match entry.get_password() {
 				Ok(value) => value,
 				Err(keyring::Error::NoEntry) => return Ok(None), // missing entries?
-				Err(e) => return Err(wrap(e, "error reading keyring")),
+				Err(e) => return Err(wrap(e, "error reading keyring").into()),
 			};
 
 			if next_chunk.ends_with(CONTINUE_MARKER) {
@@ -253,7 +314,7 @@ impl StorageImplementation for KeyringStorage {
 		Ok(unseal(&str))
 	}
 
-	fn store(&mut self, value: StoredCredential) -> Result<(), WrappedError> {
+	fn store(&mut self, value: StoredCredential) -> Result<(), AnyError> {
 		let sealed = seal(&value);
 		let step_size = KEYCHAIN_ENTRY_LIMIT - CONTINUE_MARKER.len();
 
@@ -270,14 +331,14 @@ impl StorageImplementation for KeyringStorage {
 			};
 
 			if let Err(e) = stored {
-				return Err(wrap(e, "error updating keyring"));
+				return Err(wrap(e, "error updating keyring").into());
 			}
 		}
 
 		Ok(())
 	}
 
-	fn clear(&mut self) -> Result<(), WrappedError> {
+	fn clear(&mut self) -> Result<(), AnyError> {
 		self.read().ok(); // make sure component parts are available
 		for entry in self.entries.iter() {
 			entry
@@ -293,16 +354,16 @@ impl StorageImplementation for KeyringStorage {
 struct FileStorage(PersistedState<Option<String>>);
 
 impl StorageImplementation for FileStorage {
-	fn read(&mut self) -> Result<Option<StoredCredential>, WrappedError> {
+	fn read(&mut self) -> Result<Option<StoredCredential>, AnyError> {
 		Ok(self.0.load().and_then(|s| unseal(&s)))
 	}
 
-	fn store(&mut self, value: StoredCredential) -> Result<(), WrappedError> {
-		self.0.save(Some(seal(&value)))
+	fn store(&mut self, value: StoredCredential) -> Result<(), AnyError> {
+		self.0.save(Some(seal(&value))).map_err(|e| e.into())
 	}
 
-	fn clear(&mut self) -> Result<(), WrappedError> {
-		self.0.save(None)
+	fn clear(&mut self) -> Result<(), AnyError> {
+		self.0.save(None).map_err(|e| e.into())
 	}
 }
 
@@ -325,11 +386,14 @@ impl Auth {
 			return op(s);
 		}
 
+		#[cfg(not(target_os = "linux"))]
 		let mut keyring_storage = KeyringStorage::default();
+		#[cfg(target_os = "linux")]
+		let mut keyring_storage = ThreadKeyringStorage::default();
 		let mut file_storage = FileStorage(PersistedState::new(self.file_storage_path.clone()));
 
 		let keyring_storage_result = match std::env::var("VSCODE_CLI_USE_FILE_KEYCHAIN") {
-			Ok(_) => Err(wrap("", "user prefers file storage")),
+			Ok(_) => Err(wrap("", "user prefers file storage").into()),
 			_ => keyring_storage.read(),
 		};
 
@@ -338,10 +402,17 @@ impl Auth {
 				last_read: Cell::new(Ok(v)),
 				storage: Box::new(keyring_storage),
 			},
-			Err(_) => StorageWithLastRead {
-				last_read: Cell::new(file_storage.read()),
-				storage: Box::new(file_storage),
-			},
+			Err(e) => {
+				debug!(self.log, "Using file keychain storage due to: {}", e);
+				StorageWithLastRead {
+					last_read: Cell::new(
+						file_storage
+							.read()
+							.map_err(|e| wrap(e, "could not read from file storage")),
+					),
+					storage: Box::new(file_storage),
+				}
+			}
 		};
 
 		let out = op(&mut storage);
@@ -374,7 +445,7 @@ impl Auth {
 	}
 
 	/// Clears login info from the keyring.
-	pub fn clear_credentials(&self) -> Result<(), WrappedError> {
+	pub fn clear_credentials(&self) -> Result<(), AnyError> {
 		self.with_storage(|storage| {
 			storage.storage.clear()?;
 			storage.last_read.set(Ok(None));
