@@ -7,11 +7,14 @@ pub mod code_server;
 pub mod dev_tunnels;
 pub mod legal;
 pub mod paths;
+pub mod protocol;
 pub mod shutdown_signal;
 pub mod singleton_client;
 pub mod singleton_server;
-pub mod protocol;
+pub mod forwarding;
 
+mod wsl_detect;
+mod challenge;
 mod control_server;
 mod nosleep;
 #[cfg(target_os = "linux")]
@@ -31,11 +34,9 @@ mod service_macos;
 #[cfg(target_os = "windows")]
 mod service_windows;
 mod socket_signal;
-mod wsl_server;
 
-pub use control_server::{serve, Next};
+pub use control_server::{serve, serve_stream, Next, ServeStreamParams, AuthRequired};
 pub use nosleep::SleepInhibitor;
 pub use service::{
 	create_service_manager, ServiceContainer, ServiceManager, SERVICE_LOG_FILE_NAME,
 };
-pub use wsl_server::serve_wsl;
