@@ -24,9 +24,6 @@ module.exports = withBrowserDefaults({
 		'keytar': 'commonjs keytar',
 	},
 	resolve: {
-		fallback: {
-			'querystring': require.resolve('querystring-es3')
-		},
 		alias: {
 			'./node/crypto': path.resolve(__dirname, 'src/browser/crypto'),
 			'./node/authServer': path.resolve(__dirname, 'src/browser/authServer'),
