@@ -3,18 +3,20 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
+import { Cancellation } from '@vscode/sync-api-common/lib/common/messageCancellation';
 import * as vscode from 'vscode';
-import type * as Proto from '../protocol';
-import { EventName } from '../protocol.const';
+import { TypeScriptServiceConfiguration } from '../configuration/configuration';
+import { TelemetryReporter } from '../logging/telemetry';
+import Tracer from '../logging/tracer';
 import { CallbackMap } from '../tsServer/callbackMap';
 import { RequestItem, RequestQueue, RequestQueueingType } from '../tsServer/requestQueue';
 import { TypeScriptServerError } from '../tsServer/serverError';
 import { ServerResponse, ServerType, TypeScriptRequests } from '../typescriptService';
-import { TypeScriptServiceConfiguration } from '../utils/configuration';
 import { Disposable } from '../utils/dispose';
-import { TelemetryReporter } from '../utils/telemetry';
-import Tracer from '../utils/tracer';
+import { isWebAndHasSharedArrayBuffers } from '../utils/platform';
 import { OngoingRequestCanceller } from './cancellation';
+import type * as Proto from './protocol/protocol';
+import { EventName } from './protocol/protocol.const';
 import { TypeScriptVersionManager } from './versionManager';
 import { TypeScriptVersion } from './versionProvider';
 
@@ -28,12 +30,16 @@ export interface TypeScriptServerExitEvent {
 	readonly signal: string | null;
 }
 
+export type TsServerLog =
+	{ readonly type: 'file'; readonly uri: vscode.Uri } |
+	{ readonly type: 'output'; readonly output: vscode.OutputChannel };
+
 export interface ITypeScriptServer {
 	readonly onEvent: vscode.Event<Proto.Event>;
 	readonly onExit: vscode.Event<TypeScriptServerExitEvent>;
 	readonly onError: vscode.Event<any>;
 
-	readonly tsServerLogFile: string | undefined;
+	readonly tsServerLog: TsServerLog | undefined;
 
 	kill(): void;
 
@@ -64,6 +70,7 @@ export interface TsServerProcessFactory {
 		kind: TsServerProcessKind,
 		configuration: TypeScriptServiceConfiguration,
 		versionManager: TypeScriptVersionManager,
+		tsServerLog: TsServerLog | undefined,
 	): TsServerProcess;
 }
 
@@ -77,7 +84,7 @@ export interface TsServerProcess {
 	kill(): void;
 }
 
-export class ProcessBasedTsServer extends Disposable implements ITypeScriptServer {
+export class SingleTsServer extends Disposable implements ITypeScriptServer {
 	private readonly _requestQueue = new RequestQueue();
 	private readonly _callbacks = new CallbackMap<Proto.Response>();
 	private readonly _pendingResponses = new Set<number>();
@@ -86,7 +93,7 @@ export class ProcessBasedTsServer extends Disposable implements ITypeScriptServe
 		private readonly _serverId: string,
 		private readonly _serverSource: ServerType,
 		private readonly _process: TsServerProcess,
-		private readonly _tsServerLogFile: string | undefined,
+		private readonly _tsServerLog: TsServerLog | undefined,
 		private readonly _requestCanceller: OngoingRequestCanceller,
 		private readonly _version: TypeScriptVersion,
 		private readonly _telemetryReporter: TelemetryReporter,
@@ -118,7 +125,7 @@ export class ProcessBasedTsServer extends Disposable implements ITypeScriptServe
 	private readonly _onError = this._register(new vscode.EventEmitter<any>());
 	public readonly onError = this._onError.event;
 
-	public get tsServerLogFile() { return this._tsServerLogFile; }
+	public get tsServerLog() { return this._tsServerLog; }
 
 	private write(serverRequest: Proto.Request) {
 		this._process.write(serverRequest);
@@ -171,17 +178,16 @@ export class ProcessBasedTsServer extends Disposable implements ITypeScriptServe
 		}
 	}
 
-	private tryCancelRequest(seq: number, command: string): boolean {
+	private tryCancelRequest(request: Proto.Request, command: string): boolean {
+		const seq = request.seq;
 		try {
 			if (this._requestQueue.tryDeletePendingRequest(seq)) {
 				this.logTrace(`Canceled request with sequence number ${seq}`);
 				return true;
 			}
-
 			if (this._requestCanceller.tryCancelOngoingRequest(seq)) {
 				return true;
 			}
-
 			this.logTrace(`Tried to cancel request with sequence number ${seq}. But request got already delivered.`);
 			return false;
 		} finally {
@@ -213,7 +219,7 @@ export class ProcessBasedTsServer extends Disposable implements ITypeScriptServe
 			request,
 			expectsResponse: executeInfo.expectsResult,
 			isAsync: executeInfo.isAsync,
-			queueingType: ProcessBasedTsServer.getQueueingType(command, executeInfo.lowPriority)
+			queueingType: SingleTsServer.getQueueingType(command, executeInfo.lowPriority)
 		};
 		let result: Promise<ServerResponse.Response<Proto.Response>> | undefined;
 		if (executeInfo.expectsResult) {
@@ -221,13 +227,19 @@ export class ProcessBasedTsServer extends Disposable implements ITypeScriptServe
 				this._callbacks.add(request.seq, { onSuccess: resolve as () => ServerResponse.Response<Proto.Response> | undefined, onError: reject, queuingStartTime: Date.now(), isAsync: executeInfo.isAsync }, executeInfo.isAsync);
 
 				if (executeInfo.token) {
+
+					const cancelViaSAB = isWebAndHasSharedArrayBuffers()
+						? Cancellation.addData(request)
+						: undefined;
+
 					executeInfo.token.onCancellationRequested(() => {
-						this.tryCancelRequest(request.seq, command);
+						cancelViaSAB?.();
+						this.tryCancelRequest(request, command);
 					});
 				}
 			}).catch((err: Error) => {
 				if (err instanceof TypeScriptServerError) {
-					if (!executeInfo.token || !executeInfo.token.isCancellationRequested) {
+					if (!executeInfo.token?.isCancellationRequested) {
 						/* __GDPR__
 							"languageServiceErrorResponse" : {
 								"owner": "mjbvz",
@@ -287,7 +299,7 @@ export class ProcessBasedTsServer extends Disposable implements ITypeScriptServe
 	}
 
 	private logTrace(message: string) {
-		this._tracer.logTrace(this._serverId, message);
+		this._tracer.trace(this._serverId, message);
 	}
 
 	private static readonly fenceCommands = new Set(['change', 'close', 'open', 'updateOpen']);
@@ -296,7 +308,7 @@ export class ProcessBasedTsServer extends Disposable implements ITypeScriptServe
 		command: string,
 		lowPriority?: boolean
 	): RequestQueueingType {
-		if (ProcessBasedTsServer.fenceCommands.has(command)) {
+		if (SingleTsServer.fenceCommands.has(command)) {
 			return RequestQueueingType.Fence;
 		}
 		return lowPriority ? RequestQueueingType.LowPriority : RequestQueueingType.Normal;
@@ -457,7 +469,7 @@ export class GetErrRoutingTsServer extends Disposable implements ITypeScriptServ
 	private readonly _onError = this._register(new vscode.EventEmitter<any>());
 	public readonly onError = this._onError.event;
 
-	public get tsServerLogFile() { return this.mainServer.tsServerLogFile; }
+	public get tsServerLog() { return this.mainServer.tsServerLog; }
 
 	public kill(): void {
 		this.getErrServer.kill();
@@ -483,6 +495,7 @@ export class SyntaxRoutingTsServer extends Disposable implements ITypeScriptServ
 		'format',
 		'formatonkey',
 		'docCommentTemplate',
+		'linkedEditingRange'
 	]);
 
 	/**
@@ -597,7 +610,7 @@ export class SyntaxRoutingTsServer extends Disposable implements ITypeScriptServ
 	private readonly _onError = this._register(new vscode.EventEmitter<any>());
 	public readonly onError = this._onError.event;
 
-	public get tsServerLogFile() { return this.semanticServer.tsServerLogFile; }
+	public get tsServerLog() { return this.semanticServer.tsServerLog; }
 
 	public kill(): void {
 		this.syntaxServer.kill();
