@@ -259,7 +259,7 @@ abstract class ViewItem<TLayoutContext> {
 
 	constructor(
 		protected container: HTMLElement,
-		private view: IView<TLayoutContext>,
+		readonly view: IView<TLayoutContext>,
 		size: ViewItemSize,
 		private disposable: IDisposable
 	) {
@@ -280,9 +280,8 @@ abstract class ViewItem<TLayoutContext> {
 
 	abstract layoutContainer(offset: number): void;
 
-	dispose(): IView<TLayoutContext> {
+	dispose(): void {
 		this.disposable.dispose();
-		return this.view;
 	}
 }
 
@@ -343,6 +342,12 @@ export type DistributeSizing = { type: 'distribute' };
  */
 export type SplitSizing = { type: 'split'; index: number };
 
+/**
+ * When adding a view, use DistributeSizing when all pre-existing views are
+ * distributed evenly, otherwise use SplitSizing.
+ */
+export type AutoSizing = { type: 'auto'; index: number };
+
 /**
  * When adding or removing views, assume the view is invisible.
  */
@@ -352,7 +357,7 @@ export type InvisibleSizing = { type: 'invisible'; cachedVisibleSize: number };
  * When adding or removing views, the sizing provides fine grained
  * control over how other views get resized.
  */
-export type Sizing = DistributeSizing | SplitSizing | InvisibleSizing;
+export type Sizing = DistributeSizing | SplitSizing | AutoSizing | InvisibleSizing;
 
 export namespace Sizing {
 
@@ -368,6 +373,12 @@ export namespace Sizing {
 	 */
 	export function Split(index: number): SplitSizing { return { type: 'split', index }; }
 
+	/**
+	 * When adding a view, use DistributeSizing when all pre-existing views are
+	 * distributed evenly, otherwise use SplitSizing.
+	 */
+	export function Auto(index: number): AutoSizing { return { type: 'auto', index }; }
+
 	/**
 	 * When adding or removing views, assume the view is invisible.
 	 */
@@ -646,9 +657,24 @@ export class SplitView<TLayoutContext = undefined> extends Disposable {
 			throw new Error('Index out of bounds');
 		}
 
+		if (sizing?.type === 'auto') {
+			if (this.areViewsDistributed()) {
+				sizing = { type: 'distribute' };
+			} else {
+				sizing = { type: 'split', index: sizing.index };
+			}
+		}
+
+		// Save referene view, in case of `split` sizing
+		const referenceViewItem = sizing?.type === 'split' ? this.viewItems[sizing.index] : undefined;
+
 		// Remove view
-		const viewItem = this.viewItems.splice(index, 1)[0];
-		const view = viewItem.dispose();
+		const viewItemToRemove = this.viewItems.splice(index, 1)[0];
+
+		// Resize reference view, in case of `split` sizing
+		if (referenceViewItem) {
+			referenceViewItem.size += viewItemToRemove.size;
+		}
 
 		// Remove sash
 		if (this.viewItems.length >= 1) {
@@ -664,7 +690,9 @@ export class SplitView<TLayoutContext = undefined> extends Disposable {
 			this.distributeViewSizes();
 		}
 
-		return view;
+		const result = viewItemToRemove.view;
+		viewItemToRemove.dispose();
+		return result;
 	}
 
 	/**
@@ -1054,12 +1082,22 @@ export class SplitView<TLayoutContext = undefined> extends Disposable {
 
 		if (typeof size === 'number') {
 			viewSize = size;
-		} else if (size.type === 'split') {
-			viewSize = this.getViewSize(size.index) / 2;
-		} else if (size.type === 'invisible') {
-			viewSize = { cachedVisibleSize: size.cachedVisibleSize };
 		} else {
-			viewSize = view.minimumSize;
+			if (size.type === 'auto') {
+				if (this.areViewsDistributed()) {
+					size = { type: 'distribute' };
+				} else {
+					size = { type: 'split', index: size.index };
+				}
+			}
+
+			if (size.type === 'split') {
+				viewSize = this.getViewSize(size.index) / 2;
+			} else if (size.type === 'invisible') {
+				viewSize = { cachedVisibleSize: size.cachedVisibleSize };
+			} else {
+				viewSize = view.minimumSize;
+			}
 		}
 
 		const item = this.orientation === Orientation.VERTICAL
@@ -1382,6 +1420,21 @@ export class SplitView<TLayoutContext = undefined> extends Disposable {
 		return undefined;
 	}
 
+	private areViewsDistributed() {
+		let min = undefined, max = undefined;
+
+		for (const view of this.viewItems) {
+			min = min === undefined ? view.size : Math.min(min, view.size);
+			max = max === undefined ? view.size : Math.max(max, view.size);
+
+			if (max - min > 2) {
+				return false;
+			}
+		}
+
+		return true;
+	}
+
 	override dispose(): void {
 		this.sashDragState?.disposable.dispose();
 
