@@ -1706,10 +1706,18 @@ export abstract class AbstractTree<T, TFilterData, TRef> implements IDisposable
 		return this.view.contentHeight;
 	}
 
+	get contentWidth(): number {
+		return this.view.contentWidth;
+	}
+
 	get onDidChangeContentHeight(): Event<number> {
 		return this.view.onDidChangeContentHeight;
 	}
 
+	get onDidChangeContentWidth(): Event<number> {
+		return this.view.onDidChangeContentWidth;
+	}
+
 	get scrollTop(): number {
 		return this.view.scrollTop;
 	}
