@@ -166,12 +166,14 @@ export interface ITask<T> {
  * 			throttler.queue(deliver);
  * 		}
  */
-export class Throttler {
+export class Throttler implements IDisposable {
 
 	private activePromise: Promise<any> | null;
 	private queuedPromise: Promise<any> | null;
 	private queuedPromiseFactory: ITask<Promise<any>> | null;
 
+	private isDisposed = false;
+
 	constructor() {
 		this.activePromise = null;
 		this.queuedPromise = null;
@@ -179,6 +181,10 @@ export class Throttler {
 	}
 
 	queue<T>(promiseFactory: ITask<Promise<T>>): Promise<T> {
+		if (this.isDisposed) {
+			throw new Error('Throttler is disposed');
+		}
+
 		if (this.activePromise) {
 			this.queuedPromiseFactory = promiseFactory;
 
@@ -186,6 +192,10 @@ export class Throttler {
 				const onComplete = () => {
 					this.queuedPromise = null;
 
+					if (this.isDisposed) {
+						return;
+					}
+
 					const result = this.queue(this.queuedPromiseFactory!);
 					this.queuedPromiseFactory = null;
 
@@ -214,6 +224,10 @@ export class Throttler {
 			});
 		});
 	}
+
+	dispose(): void {
+		this.isDisposed = true;
+	}
 }
 
 export class Sequencer {
@@ -403,6 +417,7 @@ export class ThrottledDelayer<T> {
 
 	dispose(): void {
 		this.delayer.dispose();
+		this.throttler.dispose();
 	}
 }
 
