@@ -19,15 +19,15 @@ import { IRemoteDiagnosticError, isRemoteDiagnosticError } from 'vs/platform/dia
 import { ByteSize } from 'vs/platform/files/common/files';
 import { ElectronIPCMainProcessService } from 'vs/platform/ipc/electron-sandbox/mainProcessService';
 import { ProcessExplorerData, ProcessExplorerStyles, ProcessExplorerWindowConfiguration } from 'vs/platform/issue/common/issue';
-import { INativeHostService } from 'vs/platform/native/electron-sandbox/native';
+import { INativeHostService } from 'vs/platform/native/common/native';
 import { NativeHostService } from 'vs/platform/native/electron-sandbox/nativeHostService';
 import { getIconsStyleSheet } from 'vs/platform/theme/browser/iconsStyleSheet';
 import { applyZoom, zoomIn, zoomOut } from 'vs/platform/window/electron-sandbox/window';
 import { StandardKeyboardEvent } from 'vs/base/browser/keyboardEvent';
 import { KeyCode } from 'vs/base/common/keyCodes';
 
-const DEBUG_FLAGS_PATTERN = /\s--(inspect|debug)(-brk|port)?=(\d+)?/;
-const DEBUG_PORT_PATTERN = /\s--(inspect|debug)-port=(\d+)/;
+const DEBUG_FLAGS_PATTERN = /\s--inspect(?:-brk|port)?=(?<port>\d+)?/;
+const DEBUG_PORT_PATTERN = /\s--inspect-port=(?<port>\d+)/;
 
 class ProcessListDelegate implements IListVirtualDelegate<MachineProcessInformation | ProcessItem | IRemoteDiagnosticError> {
 	getHeight(element: MachineProcessInformation | ProcessItem | IRemoteDiagnosticError) {
@@ -56,13 +56,13 @@ class ProcessListDelegate implements IListVirtualDelegate<MachineProcessInformat
 }
 
 interface IProcessItemTemplateData extends IProcessRowTemplateData {
-	CPU: HTMLElement;
-	memory: HTMLElement;
-	PID: HTMLElement;
+	readonly CPU: HTMLElement;
+	readonly memory: HTMLElement;
+	readonly PID: HTMLElement;
 }
 
 interface IProcessRowTemplateData {
-	name: HTMLElement;
+	readonly name: HTMLElement;
 }
 
 class ProcessTreeDataSource implements IDataSource<ProcessTree, ProcessInformation | MachineProcessInformation | ProcessItem | IRemoteDiagnosticError>  {
@@ -106,15 +106,16 @@ class ProcessTreeDataSource implements IDataSource<ProcessTree, ProcessInformati
 
 class ProcessHeaderTreeRenderer implements ITreeRenderer<ProcessInformation, void, IProcessItemTemplateData> {
 	templateId: string = 'header';
+
 	renderTemplate(container: HTMLElement): IProcessItemTemplateData {
-		const data = Object.create(null);
 		const row = append(container, $('.row'));
-		data.name = append(row, $('.nameLabel'));
-		data.CPU = append(row, $('.cpu'));
-		data.memory = append(row, $('.memory'));
-		data.PID = append(row, $('.pid'));
-		return data;
+		const name = append(row, $('.nameLabel'));
+		const CPU = append(row, $('.cpu'));
+		const memory = append(row, $('.memory'));
+		const PID = append(row, $('.pid'));
+		return { name, CPU, memory, PID };
 	}
+
 	renderElement(node: ITreeNode<ProcessInformation, void>, index: number, templateData: IProcessItemTemplateData, height: number | undefined): void {
 		templateData.name.textContent = localize('name', "Process Name");
 		templateData.CPU.textContent = localize('cpu', "CPU (%)");
@@ -122,6 +123,7 @@ class ProcessHeaderTreeRenderer implements ITreeRenderer<ProcessInformation, voi
 		templateData.memory.textContent = localize('memory', "Memory (MB)");
 
 	}
+
 	disposeTemplate(templateData: any): void {
 		// Nothing to do
 	}
@@ -161,30 +163,29 @@ class ErrorRenderer implements ITreeRenderer<IRemoteDiagnosticError, void, IProc
 
 
 class ProcessRenderer implements ITreeRenderer<ProcessItem, void, IProcessItemTemplateData> {
-	constructor(private platform: string, private totalMem: number, private mapPidToWindowTitle: Map<number, string>) { }
+	constructor(private platform: string, private totalMem: number, private mapPidToName: Map<number, string>) { }
 
 	templateId: string = 'process';
 	renderTemplate(container: HTMLElement): IProcessItemTemplateData {
-		const data = <IProcessItemTemplateData>Object.create(null);
 		const row = append(container, $('.row'));
 
-		data.name = append(row, $('.nameLabel'));
-		data.CPU = append(row, $('.cpu'));
-		data.memory = append(row, $('.memory'));
-		data.PID = append(row, $('.pid'));
+		const name = append(row, $('.nameLabel'));
+		const CPU = append(row, $('.cpu'));
+		const memory = append(row, $('.memory'));
+		const PID = append(row, $('.pid'));
 
-		return data;
+		return { name, CPU, PID, memory };
 	}
 	renderElement(node: ITreeNode<ProcessItem, void>, index: number, templateData: IProcessItemTemplateData, height: number | undefined): void {
 		const { element } = node;
 
-		let name = element.name;
-		if (name === 'window') {
-			const windowTitle = this.mapPidToWindowTitle.get(element.pid);
-			name = windowTitle !== undefined ? `${name} (${this.mapPidToWindowTitle.get(element.pid)})` : name;
-		}
 		const pid = element.pid.toFixed(0);
 
+		let name = element.name;
+		if (this.mapPidToName.has(element.pid)) {
+			name = this.mapPidToName.get(element.pid)!;
+		}
+
 		templateData.name.textContent = name;
 		templateData.name.title = element.cmd;
 
@@ -229,7 +230,7 @@ function isProcessItem(item: any): item is ProcessItem {
 class ProcessExplorer {
 	private lastRequestTime: number;
 
-	private mapPidToWindowTitle = new Map<number, string>();
+	private mapPidToName = new Map<number, string>();
 
 	private nativeHostService: INativeHostService;
 
@@ -242,10 +243,12 @@ class ProcessExplorer {
 		this.applyStyles(data.styles);
 		this.setEventHandlers(data);
 
-		// Map window process pids to titles, annotate process names with this when rendering to distinguish between them
-		ipcRenderer.on('vscode:windowsInfoResponse', (event: unknown, windows: any[]) => {
-			this.mapPidToWindowTitle = new Map<number, string>();
-			windows.forEach(window => this.mapPidToWindowTitle.set(window.pid, window.title));
+		ipcRenderer.on('vscode:pidToNameResponse', (event: unknown, pidToNames: [number, string][]) => {
+			this.mapPidToName = new Map<number, string>();
+
+			for (const [pid, name] of pidToNames) {
+				this.mapPidToName.set(pid, name);
+			}
 		});
 
 		ipcRenderer.on('vscode:listProcessesResponse', async (event: unknown, processRoots: MachineProcessInformation[]) => {
@@ -266,7 +269,7 @@ class ProcessExplorer {
 		});
 
 		this.lastRequestTime = Date.now();
-		ipcRenderer.send('vscode:windowsInfoRequest');
+		ipcRenderer.send('vscode:pidToNameRequest');
 		ipcRenderer.send('vscode:listProcesses');
 	}
 
@@ -303,7 +306,7 @@ class ProcessExplorer {
 		const { totalmem } = await this.nativeHostService.getOSStatistics();
 
 		const renderers = [
-			new ProcessRenderer(this.data.platform, totalmem, this.mapPidToWindowTitle),
+			new ProcessRenderer(this.data.platform, totalmem, this.mapPidToName),
 			new ProcessHeaderTreeRenderer(),
 			new MachineRenderer(),
 			new ErrorRenderer()
@@ -363,7 +366,7 @@ class ProcessExplorer {
 
 	private isDebuggable(cmd: string): boolean {
 		const matches = DEBUG_FLAGS_PATTERN.exec(cmd);
-		return (matches && matches.length >= 2) || cmd.indexOf('node ') >= 0 || cmd.indexOf('node.exe') >= 0;
+		return (matches && matches.groups!.port !== '0') || cmd.indexOf('node ') >= 0 || cmd.indexOf('node.exe') >= 0;
 	}
 
 	private attachTo(item: ProcessItem) {
@@ -374,12 +377,8 @@ class ProcessExplorer {
 		};
 
 		let matches = DEBUG_FLAGS_PATTERN.exec(item.cmd);
-		if (matches && matches.length >= 2) {
-			// attach via port
-			if (matches.length === 4 && matches[3]) {
-				config.port = parseInt(matches[3]);
-			}
-			config.protocol = matches[1] === 'debug' ? 'legacy' : 'inspector';
+		if (matches) {
+			config.port = Number(matches.groups!.port);
 		} else {
 			// no port -> try to attach via pid (send SIGUSR1)
 			config.processId = String(item.pid);
@@ -387,9 +386,9 @@ class ProcessExplorer {
 
 		// a debug-port=n or inspect-port=n overrides the port
 		matches = DEBUG_PORT_PATTERN.exec(item.cmd);
-		if (matches && matches.length === 3) {
+		if (matches) {
 			// override port
-			config.port = parseInt(matches[2]);
+			config.port = Number(matches.groups!.port);
 		}
 
 		ipcRenderer.send('vscode:workbenchCommand', { id: 'debug.startFromConfig', from: 'processExplorer', args: [config] });
@@ -487,6 +486,7 @@ class ProcessExplorer {
 
 		if (isLocal) {
 			items.push({
+				accelerator: 'Alt+E',
 				label: localize('killProcess', "Kill Process"),
 				click: () => {
 					this.nativeHostService.killProcess(pid, 'SIGTERM');
@@ -558,7 +558,7 @@ class ProcessExplorer {
 
 			// Wait at least a second between requests.
 			if (waited > 1000) {
-				ipcRenderer.send('vscode:windowsInfoRequest');
+				ipcRenderer.send('vscode:pidToNameRequest');
 				ipcRenderer.send('vscode:listProcesses');
 			} else {
 				this.requestProcessList(waited);
