@@ -3,24 +3,26 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
-import { Event } from 'vs/base/common/event';
 import { IKeyboardEvent } from 'vs/base/browser/keyboardEvent';
 import { IMouseEvent, IMouseWheelEvent } from 'vs/base/browser/mouseEvent';
-import { OverviewRulerPosition, ConfigurationChangedEvent, EditorLayoutInfo, IComputedEditorOptions, EditorOption, FindComputedEditorOptionValueById, IEditorOptions, IDiffEditorOptions } from 'vs/editor/common/config/editorOptions';
-import { ICursorPositionChangedEvent, ICursorSelectionChangedEvent } from 'vs/editor/common/cursorEvents';
+import { IBoundarySashes } from 'vs/base/browser/ui/sash/sash';
+import { Event } from 'vs/base/common/event';
+import { IEditorConstructionOptions } from 'vs/editor/browser/config/editorConfiguration';
+import { ConfigurationChangedEvent, EditorLayoutInfo, EditorOption, FindComputedEditorOptionValueById, IComputedEditorOptions, IDiffEditorOptions, IEditorOptions, OverviewRulerPosition } from 'vs/editor/common/config/editorOptions';
+import { IDimension } from 'vs/editor/common/core/dimension';
 import { IPosition, Position } from 'vs/editor/common/core/position';
 import { IRange, Range } from 'vs/editor/common/core/range';
 import { Selection } from 'vs/editor/common/core/selection';
-import * as editorCommon from 'vs/editor/common/editorCommon';
-import { IIdentifiedSingleEditOperation, IModelDecoration, IModelDeltaDecoration, ITextModel, ICursorStateComputer, PositionAffinity } from 'vs/editor/common/model';
 import { IWordAtPosition } from 'vs/editor/common/core/wordHelper';
+import { ICursorPositionChangedEvent, ICursorSelectionChangedEvent } from 'vs/editor/common/cursorEvents';
+import { IDiffComputationResult, ILineChange } from 'vs/editor/common/diff/smartLinesDiffComputer';
+import * as editorCommon from 'vs/editor/common/editorCommon';
+import { GlyphMarginLane, ICursorStateComputer, IIdentifiedSingleEditOperation, IModelDecoration, IModelDeltaDecoration, ITextModel, PositionAffinity } from 'vs/editor/common/model';
+import { InjectedText } from 'vs/editor/common/modelLineProjectionData';
 import { IModelContentChangedEvent, IModelDecorationsChangedEvent, IModelLanguageChangedEvent, IModelLanguageConfigurationChangedEvent, IModelOptionsChangedEvent, IModelTokensChangedEvent } from 'vs/editor/common/textModelEvents';
+import { IEditorWhitespace, IViewModel } from 'vs/editor/common/viewModel';
 import { OverviewRulerZone } from 'vs/editor/common/viewModel/overviewZoneManager';
 import { ServicesAccessor } from 'vs/platform/instantiation/common/instantiation';
-import { IEditorWhitespace, IViewModel } from 'vs/editor/common/viewModel';
-import { InjectedText } from 'vs/editor/common/modelLineProjectionData';
-import { ILineChange, IDiffComputationResult } from 'vs/editor/common/diff/smartLinesDiffComputer';
-import { IDimension } from 'vs/editor/common/core/dimension';
 
 /**
  * A view zone is a full horizontal rectangle that 'pushes' text down.
@@ -250,6 +252,43 @@ export interface IOverlayWidget {
 	getPosition(): IOverlayWidgetPosition | null;
 }
 
+/**
+ * A glyph margin widget renders in the editor glyph margin.
+ */
+export interface IGlyphMarginWidget {
+	/**
+	 * Get a unique identifier of the glyph widget.
+	 */
+	getId(): string;
+	/**
+	 * Get the dom node of the glyph widget.
+	 */
+	getDomNode(): HTMLElement;
+	/**
+	 * Get the placement of the glyph widget.
+	 */
+	getPosition(): IGlyphMarginWidgetPosition;
+}
+
+/**
+ * A position for rendering glyph margin widgets.
+ */
+export interface IGlyphMarginWidgetPosition {
+	/**
+	 * The glyph margin lane where the widget should be shown.
+	 */
+	lane: GlyphMarginLane;
+	/**
+	 * The priority order of the widget, used for determining which widget
+	 * to render when there are multiple.
+	 */
+	zIndex: number;
+	/**
+	 * The editor range that this widget applies to.
+	 */
+	range: IRange;
+}
+
 /**
  * Type of hit element with the mouse in the editor.
  */
@@ -467,12 +506,7 @@ export interface IEditorAriaOptions {
 	role?: string;
 }
 
-export interface IDiffEditorConstructionOptions extends IDiffEditorOptions {
-	/**
-	 * The initial editor dimension (to avoid measuring the container).
-	 */
-	dimension?: IDimension;
-
+export interface IDiffEditorConstructionOptions extends IDiffEditorOptions, IEditorConstructionOptions {
 	/**
 	 * Place overflow widgets inside an external DOM node.
 	 * Defaults to an internal DOM node.
@@ -992,6 +1026,20 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	 */
 	removeOverlayWidget(widget: IOverlayWidget): void;
 
+	/**
+	 * Add a glyph margin widget. Widgets must have unique ids, otherwise they will be overwritten.
+	 */
+	addGlyphMarginWidget(widget: IGlyphMarginWidget): void;
+	/**
+	 * Layout/Reposition a glyph margin widget. This is a ping to the editor to call widget.getPosition()
+	 * and update appropriately.
+	 */
+	layoutGlyphMarginWidget(widget: IGlyphMarginWidget): void;
+	/**
+	 * Remove a glyph margin widget.
+	 */
+	removeGlyphMarginWidget(widget: IGlyphMarginWidget): void;
+
 	/**
 	 * Change the view zones. View zones are lost when a new model is attached to the editor.
 	 */
@@ -1038,6 +1086,12 @@ export interface ICodeEditor extends editorCommon.IEditor {
 	hasModel(): this is IActiveCodeEditor;
 
 	setBanner(bannerDomNode: HTMLElement | null, height: number): void;
+
+	/**
+	 * Is called when the model has been set, view state was restored and options are updated.
+	 * This is the best place to compute data for the viewport (such as tokens).
+	 */
+	handleInitialized?(): void;
 }
 
 /**
@@ -1094,13 +1148,6 @@ export interface IActiveCodeEditor extends ICodeEditor {
 	getScrolledVisiblePosition(position: IPosition): { top: number; left: number; height: number };
 }
 
-/**
- * Information about a line in the diff editor
- */
-export interface IDiffLineInformation {
-	readonly equivalentLineNumber: number;
-}
-
 /**
  * @internal
  */
@@ -1163,6 +1210,8 @@ export interface IDiffEditor extends editorCommon.IEditor {
 	 */
 	getModel(): editorCommon.IDiffEditorModel | null;
 
+	createViewModel(model: editorCommon.IDiffEditorModel): editorCommon.IDiffEditorViewModel;
+
 	/**
 	 * Sets the current model attached to this editor.
 	 * If the previous model was created by the editor via the value key in the options
@@ -1171,7 +1220,7 @@ export interface IDiffEditor extends editorCommon.IEditor {
 	 * will not be destroyed.
 	 * It is safe to call setModel(null) to simply detach the current model from the editor.
 	 */
-	setModel(model: editorCommon.IDiffEditorModel | null): void;
+	setModel(model: editorCommon.IDiffEditorModel | editorCommon.IDiffEditorViewModel | null): void;
 
 	/**
 	 * Get the `original` editor.
@@ -1194,22 +1243,29 @@ export interface IDiffEditor extends editorCommon.IEditor {
 	 */
 	getDiffComputationResult(): IDiffComputationResult | null;
 
-	/**
-	 * Get information based on computed diff about a line number from the original model.
-	 * If the diff computation is not finished or the model is missing, will return null.
-	 */
-	getDiffLineInformationForOriginal(lineNumber: number): IDiffLineInformation | null;
-
-	/**
-	 * Get information based on computed diff about a line number from the modified model.
-	 * If the diff computation is not finished or the model is missing, will return null.
-	 */
-	getDiffLineInformationForModified(lineNumber: number): IDiffLineInformation | null;
-
 	/**
 	 * Update the editor's options after the editor has been created.
 	 */
 	updateOptions(newOptions: IDiffEditorOptions): void;
+
+	/**
+	 * @internal
+	 */
+	setBoundarySashes(sashes: IBoundarySashes): void;
+
+	/**
+	 * @internal
+	 */
+	goToDiff(target: 'next' | 'previous'): void;
+
+	/**
+	 * @internal
+	 */
+	revealFirstDiff(): unknown;
+
+	accessibleDiffViewerNext(): void;
+
+	accessibleDiffViewerPrev(): void;
 }
 
 /**
