@@ -49,6 +49,13 @@ export class SequenceDiff {
 	public join(other: SequenceDiff): SequenceDiff {
 		return new SequenceDiff(this.seq1Range.join(other.seq1Range), this.seq2Range.join(other.seq2Range));
 	}
+
+	public delta(offset: number): SequenceDiff {
+		if (offset === 0) {
+			return this;
+		}
+		return new SequenceDiff(this.seq1Range.delta(offset), this.seq2Range.delta(offset));
+	}
 }
 
 export interface ISequence {
