@@ -7,6 +7,6 @@ import { SmartLinesDiffComputer } from 'vs/editor/common/diff/smartLinesDiffComp
 import { StandardLinesDiffComputer } from 'vs/editor/common/diff/standardLinesDiffComputer';
 
 export const linesDiffComputers = {
-	legacy: new SmartLinesDiffComputer(),
-	advanced: new StandardLinesDiffComputer(),
+	getLegacy: () => new SmartLinesDiffComputer(),
+	getAdvanced: () => new StandardLinesDiffComputer(),
 };
