@@ -75,7 +75,7 @@ export class SmartLinesDiffComputer implements ILinesDiffComputer {
 			);
 		});
 
-		return new LinesDiff(changes, result.quitEarly);
+		return new LinesDiff(changes, [], result.quitEarly);
 	}
 }
 
@@ -85,13 +85,14 @@ export interface IDiffComputationResult {
 
 	/**
 	 * The changes as (legacy) line change array.
+	 * @deprecated Use `changes2` instead.
 	 */
 	changes: ILineChange[];
 
 	/**
 	 * The changes as (modern) line range mapping array.
 	 */
-	changes2: LineRangeMapping[];
+	changes2: readonly LineRangeMapping[];
 }
 
 /**
