@@ -11,15 +11,34 @@ import { Position } from 'vs/editor/common/core/position';
 import { Range } from 'vs/editor/common/core/range';
 import { DateTimeout, ISequence, ITimeout, InfiniteTimeout, SequenceDiff } from 'vs/editor/common/diff/algorithms/diffAlgorithm';
 import { DynamicProgrammingDiffing } from 'vs/editor/common/diff/algorithms/dynamicProgrammingDiffing';
-import { optimizeSequenceDiffs, smoothenSequenceDiffs } from 'vs/editor/common/diff/algorithms/joinSequenceDiffs';
+import { optimizeSequenceDiffs, removeRandomMatches, smoothenSequenceDiffs } from 'vs/editor/common/diff/algorithms/joinSequenceDiffs';
 import { MyersDiffAlgorithm } from 'vs/editor/common/diff/algorithms/myersDiffAlgorithm';
-import { ILinesDiffComputer, ILinesDiffComputerOptions, LineRangeMapping, LinesDiff, RangeMapping } from 'vs/editor/common/diff/linesDiffComputer';
+import { ILinesDiffComputer, ILinesDiffComputerOptions, LineRangeMapping, LinesDiff, MovedText, RangeMapping, SimpleLineRangeMapping } from 'vs/editor/common/diff/linesDiffComputer';
 
 export class StandardLinesDiffComputer implements ILinesDiffComputer {
 	private readonly dynamicProgrammingDiffing = new DynamicProgrammingDiffing();
 	private readonly myersDiffingAlgorithm = new MyersDiffAlgorithm();
 
 	computeDiff(originalLines: string[], modifiedLines: string[], options: ILinesDiffComputerOptions): LinesDiff {
+		if (originalLines.length === 1 && originalLines[0].length === 0 || modifiedLines.length === 1 && modifiedLines[0].length === 0) {
+			return {
+				changes: [
+					new LineRangeMapping(
+						new LineRange(1, originalLines.length + 1),
+						new LineRange(1, modifiedLines.length + 1),
+						[
+							new RangeMapping(
+								new Range(1, 1, originalLines.length, originalLines[0].length + 1),
+								new Range(1, 1, modifiedLines.length, modifiedLines[0].length + 1)
+							)
+						]
+					)
+				],
+				hitTimeout: false,
+				moves: [],
+			};
+		}
+
 		const timeout = options.maxComputationTimeMs === 0 ? InfiniteTimeout.instance : new DateTimeout(options.maxComputationTimeMs);
 		const considerWhitespaceChanges = !options.ignoreTrimWhitespace;
 
@@ -116,27 +135,91 @@ export class StandardLinesDiffComputer implements ILinesDiffComputer {
 		scanForWhitespaceChanges(originalLines.length - seq1LastStart);
 
 		const changes = lineRangeMappingFromRangeMappings(alignments, originalLines, modifiedLines);
-		return new LinesDiff(changes, hitTimeout);
+
+		const moves: MovedText[] = [];
+		if (options.computeMoves) {
+			const deletions = changes
+				.filter(c => c.modifiedRange.isEmpty && c.originalRange.length >= 3)
+				.map(d => new LineRangeFragment(d.originalRange, originalLines));
+			const insertions = new Set(changes
+				.filter(c => c.originalRange.isEmpty && c.modifiedRange.length >= 3)
+				.map(d => new LineRangeFragment(d.modifiedRange, modifiedLines)));
+
+			for (const deletion of deletions) {
+				let highestSimilarity = -1;
+				let best: LineRangeFragment | undefined;
+				for (const insertion of insertions) {
+					const similarity = deletion.computeSimilarity(insertion);
+					if (similarity > highestSimilarity) {
+						highestSimilarity = similarity;
+						best = insertion;
+					}
+				}
+
+				if (highestSimilarity > 0.90 && best) {
+					const moveChanges = this.refineDiff(originalLines, modifiedLines, new SequenceDiff(
+						new OffsetRange(deletion.range.startLineNumber - 1, deletion.range.endLineNumberExclusive - 1),
+						new OffsetRange(best.range.startLineNumber - 1, best.range.endLineNumberExclusive - 1),
+					), timeout, considerWhitespaceChanges);
+					const mappings = lineRangeMappingFromRangeMappings(moveChanges.mappings, originalLines, modifiedLines, true);
+
+					insertions.delete(best);
+					moves.push(new MovedText(new SimpleLineRangeMapping(deletion.range, best.range), mappings));
+				}
+			}
+		}
+
+		// Make sure all ranges are valid
+		assertFn(() => {
+			function validatePosition(pos: Position, lines: string[]): boolean {
+				if (pos.lineNumber < 1 || pos.lineNumber > lines.length) { return false; }
+				const line = lines[pos.lineNumber - 1];
+				if (pos.column < 1 || pos.column > line.length + 1) { return false; }
+				return true;
+			}
+
+			function validateRange(range: LineRange, lines: string[]): boolean {
+				if (range.startLineNumber < 1 || range.startLineNumber > lines.length + 1) { return false; }
+				if (range.endLineNumberExclusive < 1 || range.endLineNumberExclusive > lines.length + 1) { return false; }
+				return true;
+			}
+
+			for (const c of changes) {
+				if (!c.innerChanges) { return false; }
+				for (const ic of c.innerChanges) {
+					const valid = validatePosition(ic.modifiedRange.getStartPosition(), modifiedLines) && validatePosition(ic.modifiedRange.getEndPosition(), modifiedLines) &&
+						validatePosition(ic.originalRange.getStartPosition(), originalLines) && validatePosition(ic.originalRange.getEndPosition(), originalLines);
+					if (!valid) { return false; }
+				}
+				if (!validateRange(c.modifiedRange, modifiedLines) || !validateRange(c.originalRange, originalLines)) {
+					return false;
+				}
+			}
+			return true;
+		});
+
+		return new LinesDiff(changes, moves, hitTimeout);
 	}
 
 	private refineDiff(originalLines: string[], modifiedLines: string[], diff: SequenceDiff, timeout: ITimeout, considerWhitespaceChanges: boolean): { mappings: RangeMapping[]; hitTimeout: boolean } {
-		const sourceSlice = new Slice(originalLines, diff.seq1Range, considerWhitespaceChanges);
-		const targetSlice = new Slice(modifiedLines, diff.seq2Range, considerWhitespaceChanges);
+		const slice1 = new LinesSliceCharSequence(originalLines, diff.seq1Range, considerWhitespaceChanges);
+		const slice2 = new LinesSliceCharSequence(modifiedLines, diff.seq2Range, considerWhitespaceChanges);
 
-		const diffResult = sourceSlice.length + targetSlice.length < 500
-			? this.dynamicProgrammingDiffing.compute(sourceSlice, targetSlice, timeout)
-			: this.myersDiffingAlgorithm.compute(sourceSlice, targetSlice, timeout);
+		const diffResult = slice1.length + slice2.length < 500
+			? this.dynamicProgrammingDiffing.compute(slice1, slice2, timeout)
+			: this.myersDiffingAlgorithm.compute(slice1, slice2, timeout);
 
 		let diffs = diffResult.diffs;
-		diffs = optimizeSequenceDiffs(sourceSlice, targetSlice, diffs);
-		diffs = coverFullWords(sourceSlice, targetSlice, diffs);
-		diffs = smoothenSequenceDiffs(sourceSlice, targetSlice, diffs);
+		diffs = optimizeSequenceDiffs(slice1, slice2, diffs);
+		diffs = coverFullWords(slice1, slice2, diffs);
+		diffs = smoothenSequenceDiffs(slice1, slice2, diffs);
+		diffs = removeRandomMatches(slice1, slice2, diffs);
 
 		const result = diffs.map(
 			(d) =>
 				new RangeMapping(
-					sourceSlice.translateRange(d.seq1Range),
-					targetSlice.translateRange(d.seq2Range)
+					slice1.translateRange(d.seq1Range),
+					slice2.translateRange(d.seq2Range)
 				)
 		);
 
@@ -149,7 +232,7 @@ export class StandardLinesDiffComputer implements ILinesDiffComputer {
 	}
 }
 
-function coverFullWords(sequence1: Slice, sequence2: Slice, sequenceDiffs: SequenceDiff[]): SequenceDiff[] {
+function coverFullWords(sequence1: LinesSliceCharSequence, sequence2: LinesSliceCharSequence, sequenceDiffs: SequenceDiff[]): SequenceDiff[] {
 	const additional: SequenceDiff[] = [];
 
 	let lastModifiedWord: { added: number; deleted: number; count: number; s1Range: OffsetRange; s2Range: OffsetRange } | undefined = undefined;
@@ -244,7 +327,7 @@ function mergeSequenceDiffs(sequenceDiffs1: SequenceDiff[], sequenceDiffs2: Sequ
 	return result;
 }
 
-export function lineRangeMappingFromRangeMappings(alignments: RangeMapping[], originalLines: string[], modifiedLines: string[]): LineRangeMapping[] {
+export function lineRangeMappingFromRangeMappings(alignments: RangeMapping[], originalLines: string[], modifiedLines: string[], dontAssertStartLine: boolean = false): LineRangeMapping[] {
 	const changes: LineRangeMapping[] = [];
 	for (const g of group(
 		alignments.map(a => getLineRangeMapping(a, originalLines, modifiedLines)),
@@ -263,6 +346,11 @@ export function lineRangeMappingFromRangeMappings(alignments: RangeMapping[], or
 	}
 
 	assertFn(() => {
+		if (!dontAssertStartLine) {
+			if (changes.length > 0 && changes[0].originalRange.startLineNumber !== changes[0].modifiedRange.startLineNumber) {
+				return false;
+			}
+		}
 		return checkAdjacentItems(changes,
 			(m1, m2) => m2.originalRange.startLineNumber - m1.originalRange.endLineNumberExclusive === m2.modifiedRange.startLineNumber - m1.modifiedRange.endLineNumberExclusive &&
 				// There has to be an unchanged line in between (otherwise both diffs should have been joined)
@@ -280,19 +368,23 @@ export function getLineRangeMapping(rangeMapping: RangeMapping, originalLines: s
 
 	// rangeMapping describes the edit that replaces `rangeMapping.originalRange` with `newText := getText(modifiedLines, rangeMapping.modifiedRange)`.
 
-	// original: xxx[ \n <- this line is not modified
-	// modified: xxx[ \n
-	if (rangeMapping.modifiedRange.startColumn - 1 >= modifiedLines[rangeMapping.modifiedRange.startLineNumber - 1].length
-		&& rangeMapping.originalRange.startColumn - 1 >= originalLines[rangeMapping.originalRange.startLineNumber - 1].length) {
-		lineStartDelta = 1; // +1 is always possible, as startLineNumber < endLineNumber + 1
-	}
-
 	// original: ]xxx \n <- this line is not modified
 	// modified: ]xx  \n
 	if (rangeMapping.modifiedRange.endColumn === 1 && rangeMapping.originalRange.endColumn === 1
 		&& rangeMapping.originalRange.startLineNumber + lineStartDelta <= rangeMapping.originalRange.endLineNumber
 		&& rangeMapping.modifiedRange.startLineNumber + lineStartDelta <= rangeMapping.modifiedRange.endLineNumber) {
-		lineEndDelta = -1; // We can only do this if the range is not empty yet
+		// We can only do this if the range is not empty yet
+		lineEndDelta = -1;
+	}
+
+	// original: xxx[ \n <- this line is not modified
+	// modified: xxx[ \n
+	if (rangeMapping.modifiedRange.startColumn - 1 >= modifiedLines[rangeMapping.modifiedRange.startLineNumber - 1].length
+		&& rangeMapping.originalRange.startColumn - 1 >= originalLines[rangeMapping.originalRange.startLineNumber - 1].length
+		&& rangeMapping.originalRange.startLineNumber <= rangeMapping.originalRange.endLineNumber + lineEndDelta
+		&& rangeMapping.modifiedRange.startLineNumber <= rangeMapping.modifiedRange.endLineNumber + lineEndDelta) {
+		// We can only do this if the range is not empty yet
+		lineStartDelta = 1;
 	}
 
 	const originalLineRange = new LineRange(
@@ -355,7 +447,7 @@ function getIndentation(str: string): number {
 	return i;
 }
 
-class Slice implements ISequence {
+export class LinesSliceCharSequence implements ISequence {
 	private readonly elements: number[] = [];
 	private readonly firstCharOffsetByLineMinusOne: number[] = [];
 	public readonly lineRange: OffsetRange;
@@ -409,7 +501,11 @@ class Slice implements ISequence {
 	}
 
 	get text(): string {
-		return [...this.elements].map(e => String.fromCharCode(e)).join('');
+		return this.getText(new OffsetRange(0, this.length));
+	}
+
+	getText(range: OffsetRange): string {
+		return this.elements.slice(range.start, range.endExclusive).map(e => String.fromCharCode(e)).join('');
 	}
 
 	getElement(offset: number): number {
@@ -463,8 +559,8 @@ class Slice implements ISequence {
 			}
 		}
 
-		const offsetOfPrevLineBreak = i === 0 ? 0 : this.firstCharOffsetByLineMinusOne[i - 1];
-		return new Position(this.lineRange.start + i + 1, offset - offsetOfPrevLineBreak + 1 + this.offsetByLine[i]);
+		const offsetOfFirstCharInLine = i === 0 ? 0 : this.firstCharOffsetByLineMinusOne[i - 1];
+		return new Position(this.lineRange.start + i + 1, offset - offsetOfFirstCharInLine + 1 + this.offsetByLine[i]);
 	}
 
 	public translateRange(range: OffsetRange): Range {
@@ -497,6 +593,10 @@ class Slice implements ISequence {
 
 		return new OffsetRange(start, end);
 	}
+
+	public countLinesIn(range: OffsetRange): number {
+		return this.translateOffset(range.endExclusive).lineNumber - this.translateOffset(range.start).lineNumber;
+	}
 }
 
 function isWordChar(charCode: number): boolean {
@@ -554,3 +654,47 @@ function getCategory(charCode: number): CharBoundaryCategory {
 function isSpace(charCode: number): boolean {
 	return charCode === CharCode.Space || charCode === CharCode.Tab;
 }
+
+const chrKeys = new Map<string, number>();
+function getKey(chr: string): number {
+	let key = chrKeys.get(chr);
+	if (key === undefined) {
+		key = chrKeys.size;
+		chrKeys.set(chr, key);
+	}
+	return key;
+}
+
+class LineRangeFragment {
+	private readonly totalCount: number;
+	private readonly histogram: number[] = [];
+	constructor(
+		public readonly range: LineRange,
+		public readonly lines: string[],
+	) {
+		let counter = 0;
+		for (let i = range.startLineNumber - 1; i < range.endLineNumberExclusive - 1; i++) {
+			const line = lines[i];
+			for (let j = 0; j < line.length; j++) {
+				counter++;
+				const chr = line[j];
+				const key = getKey(chr);
+				this.histogram[key] = (this.histogram[key] || 0) + 1;
+			}
+			counter++;
+			const key = getKey('\n');
+			this.histogram[key] = (this.histogram[key] || 0) + 1;
+		}
+
+		this.totalCount = counter;
+	}
+
+	public computeSimilarity(other: LineRangeFragment): number {
+		let sumDifferences = 0;
+		const maxLength = Math.max(this.histogram.length, other.histogram.length);
+		for (let i = 0; i < maxLength; i++) {
+			sumDifferences += Math.abs((this.histogram[i] ?? 0) - (other.histogram[i] ?? 0));
+		}
+		return 1 - (sumDifferences / (this.totalCount + other.totalCount));
+	}
+}
