@@ -13,6 +13,7 @@ import { TextModelPart } from 'vs/editor/common/model/textModelPart';
 import { computeIndentLevel } from 'vs/editor/common/model/utils';
 import { ILanguageConfigurationService, ResolvedLanguageConfiguration } from 'vs/editor/common/languages/languageConfigurationRegistry';
 import { BracketGuideOptions, HorizontalGuidesState, IActiveIndentGuideInfo, IGuidesTextModelPart, IndentGuide, IndentGuideHorizontalLine } from 'vs/editor/common/textModelGuides';
+import { BugIndicatingError } from 'vs/base/common/errors';
 
 export class GuidesTextModelPart extends TextModelPart implements IGuidesTextModelPart {
 	constructor(
@@ -46,7 +47,7 @@ export class GuidesTextModelPart extends TextModelPart implements IGuidesTextMod
 		const lineCount = this.textModel.getLineCount();
 
 		if (lineNumber < 1 || lineNumber > lineCount) {
-			throw new Error('Illegal value for lineNumber');
+			throw new BugIndicatingError('Illegal value for lineNumber');
 		}
 
 		const foldingRules = this.getLanguageConfiguration(
@@ -292,7 +293,7 @@ export class GuidesTextModelPart extends TextModelPart implements IGuidesTextMod
 					endLineNumber,
 					this.textModel.getLineMaxColumn(endLineNumber)
 				)
-			);
+			).toArray();
 
 		let activeBracketPairRange: Range | undefined = undefined;
 		if (activePosition && bracketPairs.length > 0) {
@@ -303,7 +304,7 @@ export class GuidesTextModelPart extends TextModelPart implements IGuidesTextMod
 					? bracketPairs
 					: this.textModel.bracketPairs.getBracketPairsInRange(
 						Range.fromPositions(activePosition)
-					)
+					).toArray()
 			).filter((bp) => Range.strictContainsPosition(bp.range, activePosition));
 
 			activeBracketPairRange = findLast(
