@@ -20,7 +20,7 @@ import { IModelContentChangedEvent } from 'vs/editor/common/textModelEvents';
 import { inlineSuggestCommitId } from 'vs/editor/contrib/inlineCompletions/browser/commandIds';
 import { GhostTextWidget } from 'vs/editor/contrib/inlineCompletions/browser/ghostTextWidget';
 import { InlineCompletionContextKeys } from 'vs/editor/contrib/inlineCompletions/browser/inlineCompletionContextKeys';
-import { InlineSuggestionHintsContentWidget } from 'vs/editor/contrib/inlineCompletions/browser/inlineCompletionsHintsWidget';
+import { InlineCompletionsHintsWidget, InlineSuggestionHintsContentWidget } from 'vs/editor/contrib/inlineCompletions/browser/inlineCompletionsHintsWidget';
 import { InlineCompletionsModel, VersionIdChangeReason } from 'vs/editor/contrib/inlineCompletions/browser/inlineCompletionsModel';
 import { SuggestWidgetAdaptor } from 'vs/editor/contrib/inlineCompletions/browser/suggestWidgetInlineCompletionProvider';
 import { AudioCue, IAudioCueService } from 'vs/platform/audioCues/browser/audioCueService';
@@ -172,25 +172,23 @@ export class InlineCompletionsController extends Disposable {
 		this._register(autorun('play audio cue & read suggestion', reader => {
 			const model = this.model.read(reader);
 			const state = model?.state.read(reader);
-			if (!model || !state || !state.completion) {
+			if (!model || !state || !state.inlineCompletion) {
 				lastInlineCompletionId = undefined;
 				return;
 			}
 
-			if (state.completion.semanticId !== lastInlineCompletionId) {
-				lastInlineCompletionId = state.completion.semanticId;
-				if (model.isNavigatingCurrentInlineCompletion) {
-					return;
-				}
-
+			if (state.inlineCompletion.semanticId !== lastInlineCompletionId) {
+				lastInlineCompletionId = state.inlineCompletion.semanticId;
+				const lineText = model.textModel.getLineContent(state.ghostText.lineNumber);
 				this.audioCueService.playAudioCue(AudioCue.inlineSuggestion).then(() => {
 					if (this.editor.getOption(EditorOption.screenReaderAnnounceInlineSuggestion)) {
-						const lineText = model.textModel.getLineContent(state.ghostText.lineNumber);
 						alert(state.ghostText.renderForScreenReader(lineText));
 					}
 				});
 			}
 		}));
+
+		this._register(new InlineCompletionsHintsWidget(this.editor, this.model, this.instantiationService));
 	}
 
 	/**
