@@ -44,9 +44,6 @@ export class InlineCompletionsModel extends Disposable {
 	private _isAcceptingPartially = false;
 	public get isAcceptingPartially() { return this._isAcceptingPartially; }
 
-	private _isNavigatingCurrentInlineCompletion = false;
-	public get isNavigatingCurrentInlineCompletion() { return this._isNavigatingCurrentInlineCompletion; }
-
 	constructor(
 		public readonly textModel: ITextModel,
 		public readonly selectedSuggestItem: IObservable<SuggestItemInfo | undefined>,
@@ -68,7 +65,7 @@ export class InlineCompletionsModel extends Disposable {
 		let lastItem: InlineCompletionWithUpdatedRange | undefined = undefined;
 		this._register(autorun('call handleItemDidShow', reader => {
 			const item = this.state.read(reader);
-			const completion = item?.completion;
+			const completion = item?.inlineCompletion;
 			if (completion?.semanticId !== lastItem?.semanticId) {
 				lastItem = completion;
 				if (completion) {
@@ -194,7 +191,7 @@ export class InlineCompletionsModel extends Disposable {
 
 	public readonly state = derived<{
 		suggestItem: SuggestItemInfo | undefined;
-		completion: InlineCompletionWithUpdatedRange | undefined;
+		inlineCompletion: InlineCompletionWithUpdatedRange | undefined;
 		ghostText: GhostTextOrReplacement;
 	} | undefined>('ghostTextAndCompletion', (reader) => {
 		const model = this.textModel;
@@ -228,7 +225,7 @@ export class InlineCompletionsModel extends Disposable {
 
 			// Show an invisible ghost text to reserve space
 			const ghostText = newGhostText ?? new GhostText(edit.range.endLineNumber, []);
-			return { ghostText, completion: augmentedCompletion?.completion, suggestItem };
+			return { ghostText, inlineCompletion: augmentedCompletion?.completion, suggestItem };
 		} else {
 			if (!this._isActive.read(reader)) { return undefined; }
 			const item = this.selectedInlineCompletion.read(reader);
@@ -238,7 +235,7 @@ export class InlineCompletionsModel extends Disposable {
 			const mode = this._inlineSuggestMode.read(reader);
 			const cursor = this.cursorPosition.read(reader);
 			const ghostText = replacement.computeGhostText(model, mode, cursor);
-			return ghostText ? { ghostText, completion: item, suggestItem: undefined } : undefined;
+			return ghostText ? { ghostText, inlineCompletion: item, suggestItem: undefined } : undefined;
 		}
 	});
 
@@ -251,17 +248,12 @@ export class InlineCompletionsModel extends Disposable {
 	private async _deltaSelectedInlineCompletionIndex(delta: 1 | -1): Promise<void> {
 		await this.triggerExplicitly();
 
-		this._isNavigatingCurrentInlineCompletion = true;
-		try {
-			const completions = this._filteredInlineCompletionItems.get() || [];
-			if (completions.length > 0) {
-				const newIdx = (this.selectedInlineCompletionIndex.get() + delta + completions.length) % completions.length;
-				this._selectedInlineCompletionId.set(completions[newIdx].semanticId, undefined);
-			} else {
-				this._selectedInlineCompletionId.set(undefined, undefined);
-			}
-		} finally {
-			this._isNavigatingCurrentInlineCompletion = false;
+		const completions = this._filteredInlineCompletionItems.get() || [];
+		if (completions.length > 0) {
+			const newIdx = (this.selectedInlineCompletionIndex.get() + delta + completions.length) % completions.length;
+			this._selectedInlineCompletionId.set(completions[newIdx].semanticId, undefined);
+		} else {
+			this._selectedInlineCompletionId.set(undefined, undefined);
 		}
 	}
 
@@ -278,11 +270,11 @@ export class InlineCompletionsModel extends Disposable {
 			throw new BugIndicatingError();
 		}
 
-		const ghostText = this.ghostText.get();
-		const completion = this.selectedInlineCompletion.get()?.toInlineCompletion(undefined);
-		if (!ghostText || !completion) {
+		const state = this.state.get();
+		if (!state || state.ghostText.isEmpty() || !state.inlineCompletion) {
 			return;
 		}
+		const completion = state.inlineCompletion.toInlineCompletion(undefined);
 
 		editor.pushUndoStop();
 		if (completion.snippetInfo) {
@@ -306,20 +298,28 @@ export class InlineCompletionsModel extends Disposable {
 		}
 
 		if (completion.command) {
-			await this._commandService
-				.executeCommand(completion.command.id, ...(completion.command.arguments || []))
-				.then(undefined, onUnexpectedExternalError);
+			// Make sure the completion list will not be disposed.
+			completion.source.addRef();
 		}
+
+		// Reset before invoking the command, since the command might cause a follow up trigger.
 		transaction(tx => {
 			this._source.clear(tx);
 			// Potentially, isActive will get set back to true by the typing or accept inline suggest event
 			// if automatic inline suggestions are enabled.
 			this._isActive.set(false, tx);
 		});
+
+		if (completion.command) {
+			await this._commandService
+				.executeCommand(completion.command.id, ...(completion.command.arguments || []))
+				.then(undefined, onUnexpectedExternalError);
+			completion.source.removeRef();
+		}
 	}
 
-	public acceptNextWord(editor: ICodeEditor): void {
-		this._acceptNext(editor, (pos, text) => {
+	public async acceptNextWord(editor: ICodeEditor): Promise<void> {
+		await this._acceptNext(editor, (pos, text) => {
 			const langId = this.textModel.getLanguageIdAtPosition(pos.lineNumber, pos.column);
 			const config = this._languageConfigurationService.getLanguageConfiguration(langId);
 			const wordRegExp = new RegExp(config.wordDefinition.source, config.wordDefinition.flags.replace('g', ''));
@@ -347,8 +347,8 @@ export class InlineCompletionsModel extends Disposable {
 		});
 	}
 
-	public acceptNextLine(editor: ICodeEditor): void {
-		this._acceptNext(editor, (pos, text) => {
+	public async acceptNextLine(editor: ICodeEditor): Promise<void> {
+		await this._acceptNext(editor, (pos, text) => {
 			const m = text.match(/\n/);
 			if (m && m.index !== undefined) {
 				return m.index + 1;
@@ -357,26 +357,24 @@ export class InlineCompletionsModel extends Disposable {
 		});
 	}
 
-	private _acceptNext(editor: ICodeEditor, getAcceptUntilIndex: (position: Position, text: string) => number): void {
+	private async _acceptNext(editor: ICodeEditor, getAcceptUntilIndex: (position: Position, text: string) => number): Promise<void> {
 		if (editor.getModel() !== this.textModel) {
 			throw new BugIndicatingError();
 		}
 
-		const ghostText = this.ghostText.get();
-		const completion = this.selectedInlineCompletion.get()?.toInlineCompletion(undefined);
-		if (!ghostText || !completion) {
+		const state = this.state.get();
+		if (!state || state.ghostText.isEmpty() || !state.inlineCompletion) {
 			return;
 		}
+		const ghostText = state.ghostText;
+		const completion = state.inlineCompletion.toInlineCompletion(undefined);
 
 		if (completion.snippetInfo || completion.filterText !== completion.insertText) {
 			// not in WYSIWYG mode, partial commit might change completion, thus it is not supported
-			this.accept(editor);
+			await this.accept(editor);
 			return;
 		}
 
-		if (ghostText.parts.length === 0) {
-			return;
-		}
 		const firstPart = ghostText.parts[0];
 		const position = new Position(ghostText.lineNumber, firstPart.column);
 		const line = firstPart.lines.join('\n');
