@@ -31,6 +31,7 @@ import { NullTelemetryService } from 'vs/platform/telemetry/common/telemetryUtil
 import { IWorkspaceContextService } from 'vs/platform/workspace/common/workspace';
 import { LanguageFeaturesService } from 'vs/editor/common/services/languageFeaturesService';
 import { ILanguageFeaturesService } from 'vs/editor/common/services/languageFeatures';
+import { IEnvironmentService } from 'vs/platform/environment/common/environment';
 
 suite('SuggestController', function () {
 
@@ -73,6 +74,10 @@ suite('SuggestController', function () {
 			}],
 			[ILabelService, new class extends mock<ILabelService>() { }],
 			[IWorkspaceContextService, new class extends mock<IWorkspaceContextService>() { }],
+			[IEnvironmentService, new class extends mock<IEnvironmentService>() {
+				override isBuilt: boolean = true;
+				override isExtensionDevelopment: boolean = false;
+			}],
 		);
 
 		model = disposables.add(createTextModel('', undefined, undefined, URI.from({ scheme: 'test-ctrl', path: '/path.tst' })));
@@ -84,6 +89,7 @@ suite('SuggestController', function () {
 
 	test('postfix completion reports incorrect position #86984', async function () {
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 				return {
 					suggestions: [{
@@ -120,6 +126,7 @@ suite('SuggestController', function () {
 	test('use additionalTextEdits sync when possible', async function () {
 
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 				return {
 					suggestions: [{
@@ -161,6 +168,7 @@ suite('SuggestController', function () {
 		let resolveCallCount = 0;
 
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 				return {
 					suggestions: [{
@@ -213,6 +221,7 @@ suite('SuggestController', function () {
 		let resolveCallCount = 0;
 		let resolve: Function = () => { };
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 				return {
 					suggestions: [{
@@ -269,6 +278,7 @@ suite('SuggestController', function () {
 		let resolveCallCount = 0;
 		let resolve: Function = () => { };
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 				return {
 					suggestions: [{
@@ -318,6 +328,7 @@ suite('SuggestController', function () {
 		let resolveCallCount = 0;
 		let resolve: Function = () => { };
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 				return {
 					suggestions: [{
@@ -372,6 +383,7 @@ suite('SuggestController', function () {
 
 		const resolve: Function[] = [];
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 				return {
 					suggestions: [{
@@ -429,6 +441,7 @@ suite('SuggestController', function () {
 
 
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 				return {
 					suggestions: [{
@@ -465,6 +478,7 @@ suite('SuggestController', function () {
 
 	test('Pressing enter on autocomplete should always apply the selected dropdown completion, not a different, hidden one #161883', async function () {
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 
 				const word = doc.getWordUntilPosition(pos);
@@ -511,6 +525,7 @@ suite('SuggestController', function () {
 
 	test('Fast autocomple typing selects the previous autocomplete suggestion, #71795', async function () {
 		disposables.add(languageFeaturesService.completionProvider.register({ scheme: 'test-ctrl' }, {
+			_debugDisplayName: 'test',
 			provideCompletionItems(doc, pos) {
 
 				const word = doc.getWordUntilPosition(pos);
