@@ -54,16 +54,18 @@ export function getEncodedLanguageId(languageId: string): number {
  * @event
  */
 export function onLanguage(languageId: string, callback: () => void): IDisposable {
-	const languageService = StandaloneServices.get(ILanguageService);
-	const disposable = languageService.onDidRequestRichLanguageFeatures((encounteredLanguageId) => {
-		if (encounteredLanguageId === languageId) {
-			// stop listening
-			disposable.dispose();
-			// invoke actual listener
-			callback();
-		}
+	return StandaloneServices.withServices(() => {
+		const languageService = StandaloneServices.get(ILanguageService);
+		const disposable = languageService.onDidRequestRichLanguageFeatures((encounteredLanguageId) => {
+			if (encounteredLanguageId === languageId) {
+				// stop listening
+				disposable.dispose();
+				// invoke actual listener
+				callback();
+			}
+		});
+		return disposable;
 	});
-	return disposable;
 }
 
 /**
@@ -72,16 +74,18 @@ export function onLanguage(languageId: string, callback: () => void): IDisposabl
  * @event
  */
 export function onLanguageEncountered(languageId: string, callback: () => void): IDisposable {
-	const languageService = StandaloneServices.get(ILanguageService);
-	const disposable = languageService.onDidRequestBasicLanguageFeatures((encounteredLanguageId) => {
-		if (encounteredLanguageId === languageId) {
-			// stop listening
-			disposable.dispose();
-			// invoke actual listener
-			callback();
-		}
+	return StandaloneServices.withServices(() => {
+		const languageService = StandaloneServices.get(ILanguageService);
+		const disposable = languageService.onDidRequestBasicLanguageFeatures((encounteredLanguageId) => {
+			if (encounteredLanguageId === languageId) {
+				// stop listening
+				disposable.dispose();
+				// invoke actual listener
+				callback();
+			}
+		});
+		return disposable;
 	});
-	return disposable;
 }
 
 /**
