@@ -9,8 +9,7 @@ import { RangeMapping } from 'vs/editor/common/diff/linesDiffComputer';
 import { getLineRangeMapping } from 'vs/editor/common/diff/standardLinesDiffComputer';
 
 suite('lineRangeMapping', () => {
-	test('lineRangeMapping', () => {
-		// {[2,1 -> 3,1]->[2,1 -> 2,1]}
+	test('1', () => {
 		assert.deepStrictEqual(
 			getLineRangeMapping(
 				new RangeMapping(
@@ -27,7 +26,29 @@ suite('lineRangeMapping', () => {
 					''
 				]
 			).toString(),
-			"{[3,4)->[3,3)}"
+			"{[2,3)->[2,2)}"
+		);
+	});
+
+	test('2', () => {
+		assert.deepStrictEqual(
+			getLineRangeMapping(
+				new RangeMapping(
+					new Range(2, 1, 2, 1),
+					new Range(2, 1, 4, 1),
+				),
+				[
+					'',
+					'',
+				],
+				[
+					'',
+					'',
+					'',
+					'',
+				]
+			).toString(),
+			"{[2,2)->[2,4)}"
 		);
 	});
 });
