@@ -4,10 +4,12 @@
  *--------------------------------------------------------------------------------------------*/
 
 import { addDisposableListener } from 'vs/base/browser/dom';
+import { StandardMouseEvent } from 'vs/base/browser/mouseEvent';
 import { IToolBarOptions, ToolBar } from 'vs/base/browser/ui/toolbar/toolbar';
 import { IAction, Separator, SubmenuAction, toAction, WorkbenchActionExecutedClassification, WorkbenchActionExecutedEvent } from 'vs/base/common/actions';
 import { coalesceInPlace } from 'vs/base/common/arrays';
 import { BugIndicatingError } from 'vs/base/common/errors';
+import { Emitter, Event } from 'vs/base/common/event';
 import { DisposableStore } from 'vs/base/common/lifecycle';
 import { localize } from 'vs/nls';
 import { createAndFillInActionBarActions } from 'vs/platform/actions/browser/menuEntryActionViewItem';
@@ -94,13 +96,15 @@ export class WorkbenchToolBar extends ToolBar {
 			..._options,
 			// mandatory (overide options)
 			allowContextMenu: true,
+			skipTelemetry: typeof _options?.telemetrySource === 'string',
 		});
 
 		// telemetry logic
-		if (_options?.telemetrySource) {
+		const telemetrySource = _options?.telemetrySource;
+		if (telemetrySource) {
 			this._store.add(this.actionBar.onDidRun(e => telemetryService.publicLog2<WorkbenchActionExecutedEvent, WorkbenchActionExecutedClassification>(
 				'workbenchActionExecuted',
-				{ id: e.action.id, from: _options!.telemetrySource! })
+				{ id: e.action.id, from: telemetrySource })
 			));
 		}
 	}
@@ -111,6 +115,7 @@ export class WorkbenchToolBar extends ToolBar {
 		const primary = _primary.slice();
 		const secondary = _secondary.slice();
 		const toggleActions: IAction[] = [];
+		let toggleActionsCheckedCount: number = 0;
 
 		const extraSecondary: IAction[] = [];
 
@@ -129,6 +134,9 @@ export class WorkbenchToolBar extends ToolBar {
 
 				// collect all toggle actions
 				toggleActions.push(action.hideActions.toggle);
+				if (action.hideActions.toggle.checked) {
+					toggleActionsCheckedCount++;
+				}
 
 				// hidden items move into overflow or ignore
 				if (action.hideActions.isHidden) {
@@ -163,19 +171,37 @@ export class WorkbenchToolBar extends ToolBar {
 		// add context menu for toggle actions
 		if (toggleActions.length > 0) {
 			this._sessionDisposables.add(addDisposableListener(this.getElement(), 'contextmenu', e => {
+				const event = new StandardMouseEvent(e);
 
-				const action = this.getItemAction(<HTMLElement>e.target);
+				const action = this.getItemAction(event.target);
 				if (!(action)) {
 					return;
 				}
-				e.preventDefault();
-				e.stopPropagation();
+				event.preventDefault();
+				event.stopPropagation();
 
-				let actions = toggleActions;
+				let noHide = false;
+
+				// last item cannot be hidden when using ignore strategy
+				if (toggleActionsCheckedCount === 1 && this._options?.hiddenItemStrategy === HiddenItemStrategy.Ignore) {
+					noHide = true;
+					for (let i = 0; i < toggleActions.length; i++) {
+						if (toggleActions[i].checked) {
+							toggleActions[i] = toAction({
+								id: action.id,
+								label: action.label,
+								checked: true,
+								enabled: false,
+								run() { }
+							});
+							break; // there is only one
+						}
+					}
+				}
 
 				// add "hide foo" actions
 				let hideAction: IAction;
-				if (action instanceof MenuItemAction || action instanceof SubmenuItemAction) {
+				if (!noHide && (action instanceof MenuItemAction || action instanceof SubmenuItemAction)) {
 					if (!action.hideActions) {
 						// no context menu for MenuItemAction instances that support no hiding
 						// those are fake actions and need to be cleaned up
@@ -191,7 +217,8 @@ export class WorkbenchToolBar extends ToolBar {
 						run() { }
 					});
 				}
-				actions = [hideAction, new Separator(), ...toggleActions];
+
+				const actions = Separator.join([hideAction], toggleActions);
 
 				// add "Reset Menu" action
 				if (this._options?.resetMenu && !menuIds) {
@@ -207,11 +234,12 @@ export class WorkbenchToolBar extends ToolBar {
 				}
 
 				this._contextMenuService.showContextMenu({
-					getAnchor: () => e,
+					getAnchor: () => event,
 					getActions: () => actions,
 					// add context menu actions (iff appicable)
 					menuId: this._options?.contextMenu,
 					menuActionOptions: { renderShortTitle: true, ...this._options?.menuOptions },
+					skipTelemetry: typeof this._options?.telemetrySource === 'string',
 					contextKeyService: this._contextKeyService,
 				});
 			}));
@@ -261,6 +289,9 @@ export interface IMenuWorkbenchToolBarOptions extends IWorkbenchToolBarOptions {
  */
 export class MenuWorkbenchToolBar extends WorkbenchToolBar {
 
+	private readonly _onDidChangeMenuItems = this._store.add(new Emitter<this>());
+	readonly onDidChangeMenuItems: Event<this> = this._onDidChangeMenuItems.event;
+
 	constructor(
 		container: HTMLElement,
 		menuId: MenuId,
@@ -287,7 +318,10 @@ export class MenuWorkbenchToolBar extends WorkbenchToolBar {
 			super.setActions(primary, secondary);
 		};
 
-		this._store.add(menu.onDidChange(updateToolbar));
+		this._store.add(menu.onDidChange(() => {
+			updateToolbar();
+			this._onDidChangeMenuItems.fire(this);
+		}));
 		updateToolbar();
 	}
 
