@@ -4,6 +4,7 @@
  *--------------------------------------------------------------------------------------------*/
 
 import { ICommonEncryptionService } from 'vs/platform/encryption/common/encryptionService';
+import { ILogService } from 'vs/platform/log/common/log';
 
 export interface Encryption {
 	encrypt(salt: string, value: string): Promise<string>;
@@ -12,29 +13,52 @@ export interface Encryption {
 export class EncryptionMainService implements ICommonEncryptionService {
 	declare readonly _serviceBrand: undefined;
 	constructor(
-		private machineId: string) {
-
-	}
+		private machineId: string,
+		@ILogService private readonly logService: ILogService
+	) { }
 
 	private encryption(): Promise<Encryption> {
 		return new Promise((resolve, reject) => require(['vscode-encrypt'], resolve, reject));
 	}
 
 	async encrypt(value: string): Promise<string> {
+		let encryption: Encryption;
+		try {
+			encryption = await this.encryption();
+		} catch (e) {
+			return value;
+		}
+
 		try {
-			const encryption = await this.encryption();
 			return encryption.encrypt(this.machineId, value);
 		} catch (e) {
+			this.logService.error(e);
 			return value;
 		}
 	}
 
 	async decrypt(value: string): Promise<string> {
+		let encryption: Encryption;
 		try {
-			const encryption = await this.encryption();
-			return encryption.decrypt(this.machineId, value);
+			encryption = await this.encryption();
 		} catch (e) {
 			return value;
 		}
+
+		try {
+			return encryption.decrypt(this.machineId, value);
+		} catch (e) {
+			this.logService.error(e);
+			return value;
+		}
+	}
+
+	async isEncryptionAvailable(): Promise<boolean> {
+		try {
+			await this.encryption();
+			return true;
+		} catch (e) {
+			return false;
+		}
 	}
 }
