@@ -111,15 +111,31 @@ export class Progress<T> implements IProgress<T> {
 
 	static readonly None = Object.freeze<IProgress<unknown>>({ report() { } });
 
+	report: (item: T) => void;
+
 	private _value?: T;
 	get value(): T | undefined { return this._value; }
 
-	constructor(private callback: (data: T) => void) { }
+	private _lastTask?: Promise<unknown>;
 
-	report(item: T) {
+	constructor(private callback: (data: T) => unknown, opts?: { async?: boolean }) {
+		this.report = opts?.async
+			? this._reportAsync.bind(this)
+			: this._reportSync.bind(this);
+	}
+
+	private _reportSync(item: T) {
 		this._value = item;
 		this.callback(this._value);
 	}
+
+	private _reportAsync(item: T) {
+		Promise.resolve(this._lastTask).finally(() => {
+			this._value = item;
+			const r = this.callback(this._value);
+			this._lastTask = Promise.resolve(r).finally(() => this._lastTask = undefined);
+		});
+	}
 }
 
 /**
