@@ -28,7 +28,6 @@ export interface IQuickPickItem {
 	type?: 'item';
 	id?: string;
 	label: string;
-	meta?: string;
 	ariaLabel?: string;
 	description?: string;
 	detail?: string;
@@ -40,6 +39,8 @@ export interface IQuickPickItem {
 	 */
 	keybinding?: ResolvedKeybinding;
 	iconClasses?: readonly string[];
+	iconPath?: { dark: URI; light?: URI };
+	iconClass?: string;
 	italic?: boolean;
 	strikethrough?: boolean;
 	highlights?: IQuickPickItemHighlights;
@@ -350,6 +351,8 @@ export interface IQuickPick<T extends IQuickPickItem> extends IQuickInput {
 
 	validationMessage: string | undefined;
 
+	severity: Severity;
+
 	inputHasFocus(): boolean;
 
 	focusOnInput(): void;
@@ -361,6 +364,11 @@ export interface IQuickPick<T extends IQuickPickItem> extends IQuickInput {
 	 */
 	hideInput: boolean;
 
+	/**
+	 * Allows to control if the count for the items should be shown
+	 */
+	hideCountBadge: boolean;
+
 	hideCheckAll: boolean;
 
 	/**
