@@ -57,7 +57,7 @@ export class RequestService extends AbstractRequestService implements IRequestSe
 		@ILogService private readonly logService: ILogService,
 		@ILoggerService loggerService: ILoggerService
 	) {
-		super(!!environmentService.isRemoteServer, loggerService);
+		super(loggerService);
 		this.configure();
 		this._register(configurationService.onDidChangeConfiguration(e => {
 			if (e.affectsConfiguration('http')) {
