@@ -8,7 +8,7 @@ import { Emitter, Event, PauseableEmitter } from 'vs/base/common/event';
 import { Disposable, dispose, MutableDisposable } from 'vs/base/common/lifecycle';
 import { mark } from 'vs/base/common/performance';
 import { isUndefinedOrNull } from 'vs/base/common/types';
-import { InMemoryStorageDatabase, IStorage, Storage, StorageHint } from 'vs/base/parts/storage/common/storage';
+import { InMemoryStorageDatabase, IStorage, IStorageChangeEvent, Storage, StorageHint, StorageValue } from 'vs/base/parts/storage/common/storage';
 import { createDecorator } from 'vs/platform/instantiation/common/instantiation';
 import { isUserDataProfile, IUserDataProfile } from 'vs/platform/userDataProfile/common/userDataProfile';
 import { IAnyWorkspaceIdentifier } from 'vs/platform/workspace/common/workspace';
@@ -35,6 +35,13 @@ export interface IWillSaveStateEvent {
 	readonly reason: WillSaveStateReason;
 }
 
+export interface IStorageEntry {
+	readonly key: string;
+	readonly value: StorageValue;
+	readonly scope: StorageScope;
+	readonly target: StorageTarget;
+}
+
 export interface IStorageService {
 
 	readonly _serviceBrand: undefined;
@@ -119,7 +126,16 @@ export interface IStorageService {
 	 * @param target allows to define the target of the storage operation
 	 * to either the current machine or user.
 	 */
-	store(key: string, value: string | boolean | number | undefined | null | object, scope: StorageScope, target: StorageTarget): void;
+	store(key: string, value: StorageValue, scope: StorageScope, target: StorageTarget): void;
+
+	/**
+	 * Allows to store multiple values in a bulk operation. Events will only
+	 * be emitted when all values have been stored.
+	 *
+	 * @param external a hint to indicate the source of the operation is external,
+	 * such as settings sync or profile changes.
+	 */
+	storeAll(entries: Array<IStorageEntry>, external: boolean): void;
 
 	/**
 	 * Delete an element stored under the provided key from storage.
@@ -229,6 +245,15 @@ export interface IStorageValueChangeEvent {
 	 * removed.
 	 */
 	readonly target: StorageTarget | undefined;
+
+	/**
+	 * A hint how the storage change event was triggered. If
+	 * `true`, the storage change was triggered by an external
+	 * source, such as:
+	 * - another process (for example another window)
+	 * - operations such as settings sync or profiles change
+	 */
+	readonly external?: boolean;
 }
 
 export interface IStorageTargetChangeEvent {
@@ -332,7 +357,8 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 		return this.initializationPromise;
 	}
 
-	protected emitDidChangeValue(scope: StorageScope, key: string): void {
+	protected emitDidChangeValue(scope: StorageScope, event: IStorageChangeEvent): void {
+		const { key, external } = event;
 
 		// Specially handle `TARGET_KEY`
 		if (key === TARGET_KEY) {
@@ -356,7 +382,7 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 
 		// Emit any other key to outside
 		else {
-			this._onDidChangeValue.fire({ scope, key, target: this.getKeyTargets(scope)[key] });
+			this._onDidChangeValue.fire({ scope, key, target: this.getKeyTargets(scope)[key], external });
 		}
 	}
 
@@ -388,11 +414,19 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 		return this.getStorage(scope)?.getObject(key, fallbackValue);
 	}
 
-	store(key: string, value: string | boolean | number | undefined | null | object, scope: StorageScope, target: StorageTarget): void {
+	storeAll(entries: Array<IStorageEntry>, external: boolean): void {
+		this.withPausedEmitters(() => {
+			for (const entry of entries) {
+				this.store(entry.key, entry.value, entry.scope, entry.target, external);
+			}
+		});
+	}
+
+	store(key: string, value: StorageValue, scope: StorageScope, target: StorageTarget, external = false): void {
 
 		// We remove the key for undefined/null values
 		if (isUndefinedOrNull(value)) {
-			this.remove(key, scope);
+			this.remove(key, scope, external);
 			return;
 		}
 
@@ -403,11 +437,11 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 			this.updateKeyTarget(key, scope, target);
 
 			// Store actual value
-			this.getStorage(scope)?.set(key, value);
+			this.getStorage(scope)?.set(key, value, external);
 		});
 	}
 
-	remove(key: string, scope: StorageScope): void {
+	remove(key: string, scope: StorageScope, external = false): void {
 
 		// Update our datastructures but send events only after
 		this.withPausedEmitters(() => {
@@ -416,7 +450,7 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 			this.updateKeyTarget(key, scope, undefined);
 
 			// Remove actual key
-			this.getStorage(scope)?.delete(key);
+			this.getStorage(scope)?.delete(key, external);
 		});
 	}
 
@@ -450,14 +484,14 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 		return keys;
 	}
 
-	private updateKeyTarget(key: string, scope: StorageScope, target: StorageTarget | undefined): void {
+	private updateKeyTarget(key: string, scope: StorageScope, target: StorageTarget | undefined, external = false): void {
 
 		// Add
 		const keyTargets = this.getKeyTargets(scope);
 		if (typeof target === 'number') {
 			if (keyTargets[key] !== target) {
 				keyTargets[key] = target;
-				this.getStorage(scope)?.set(TARGET_KEY, JSON.stringify(keyTargets));
+				this.getStorage(scope)?.set(TARGET_KEY, JSON.stringify(keyTargets), external);
 			}
 		}
 
@@ -465,7 +499,7 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 		else {
 			if (typeof keyTargets[key] === 'number') {
 				delete keyTargets[key];
-				this.getStorage(scope)?.set(TARGET_KEY, JSON.stringify(keyTargets));
+				this.getStorage(scope)?.set(TARGET_KEY, JSON.stringify(keyTargets), external);
 			}
 		}
 	}
@@ -589,32 +623,22 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 		return true;
 	}
 
-	protected switchData(oldStorage: Map<string, string>, newStorage: IStorage, scope: StorageScope, preserveData: boolean): void {
+	protected switchData(oldStorage: Map<string, string>, newStorage: IStorage, scope: StorageScope): void {
 		this.withPausedEmitters(() => {
+			// Signal storage keys that have changed
+			const handledkeys = new Set<string>();
+			for (const [key, oldValue] of oldStorage) {
+				handledkeys.add(key);
 
-			// Copy over previous keys if `preserveData`
-			if (preserveData) {
-				for (const [key, value] of oldStorage) {
-					newStorage.set(key, value);
+				const newValue = newStorage.get(key);
+				if (newValue !== oldValue) {
+					this.emitDidChangeValue(scope, { key, external: true });
 				}
 			}
 
-			// Otherwise signal storage keys that have changed
-			else {
-				const handledkeys = new Set<string>();
-				for (const [key, oldValue] of oldStorage) {
-					handledkeys.add(key);
-
-					const newValue = newStorage.get(key);
-					if (newValue !== oldValue) {
-						this.emitDidChangeValue(scope, key);
-					}
-				}
-
-				for (const [key] of newStorage.items) {
-					if (!handledkeys.has(key)) {
-						this.emitDidChangeValue(scope, key);
-					}
+			for (const [key] of newStorage.items) {
+				if (!handledkeys.has(key)) {
+					this.emitDidChangeValue(scope, { key, external: true });
 				}
 			}
 		});
@@ -635,7 +659,7 @@ export abstract class AbstractStorageService extends Disposable implements IStor
 }
 
 export function isProfileUsingDefaultStorage(profile: IUserDataProfile): boolean {
-	return profile.isDefault || !!profile.useDefaultFlags?.uiState;
+	return profile.isDefault || !!profile.useDefaultFlags?.globalState;
 }
 
 export class InMemoryStorageService extends AbstractStorageService {
@@ -647,9 +671,9 @@ export class InMemoryStorageService extends AbstractStorageService {
 	constructor() {
 		super();
 
-		this._register(this.workspaceStorage.onDidChangeStorage(key => this.emitDidChangeValue(StorageScope.WORKSPACE, key)));
-		this._register(this.profileStorage.onDidChangeStorage(key => this.emitDidChangeValue(StorageScope.PROFILE, key)));
-		this._register(this.applicationStorage.onDidChangeStorage(key => this.emitDidChangeValue(StorageScope.APPLICATION, key)));
+		this._register(this.workspaceStorage.onDidChangeStorage(e => this.emitDidChangeValue(StorageScope.WORKSPACE, e)));
+		this._register(this.profileStorage.onDidChangeStorage(e => this.emitDidChangeValue(StorageScope.PROFILE, e)));
+		this._register(this.applicationStorage.onDidChangeStorage(e => this.emitDidChangeValue(StorageScope.APPLICATION, e)));
 	}
 
 	protected getStorage(scope: StorageScope): IStorage {
