@@ -13,9 +13,9 @@ import { IProcessEnvironment, isLinux, isMacintosh, isWindows } from 'vs/base/co
 import { URI } from 'vs/base/common/uri';
 import { Promises } from 'vs/base/node/pfs';
 import { localize } from 'vs/nls';
-import { ILogService } from 'vs/platform/log/common/log';
+import { ILogService, LogLevel } from 'vs/platform/log/common/log';
 import { IProductService } from 'vs/platform/product/common/productService';
-import { FlowControlConstants, IShellLaunchConfig, ITerminalChildProcess, ITerminalLaunchError, IProcessProperty, IProcessPropertyMap as IProcessPropertyMap, ProcessPropertyType, TerminalShellType, IProcessReadyEvent, ITerminalProcessOptions, PosixShellType } from 'vs/platform/terminal/common/terminal';
+import { FlowControlConstants, IShellLaunchConfig, ITerminalChildProcess, ITerminalLaunchError, IProcessProperty, IProcessPropertyMap as IProcessPropertyMap, ProcessPropertyType, TerminalShellType, IProcessReadyEvent, ITerminalProcessOptions, PosixShellType, IProcessReadyWindowsPty } from 'vs/platform/terminal/common/terminal';
 import { ChildProcessMonitor } from 'vs/platform/terminal/node/childProcessMonitor';
 import { findExecutable, getShellIntegrationInjection, getWindowsBuildNumber, IShellIntegrationConfigInjection } from 'vs/platform/terminal/node/terminalEnvironment';
 import { WindowsShellHelper } from 'vs/platform/terminal/node/windowsShellHelper';
@@ -236,7 +236,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 			}
 			return undefined;
 		} catch (err) {
-			this._logService.trace('IPty#spawn native exception', err);
+			this._logService.trace('node-pty.node-pty.IPty#spawn native exception', err);
 			return { message: `A native exception occurred during launch (${err.message})` };
 		}
 	}
@@ -294,7 +294,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 	): Promise<void> {
 		const args = shellIntegrationInjection?.newArgs || shellLaunchConfig.args || [];
 		await this._throttleKillSpawn();
-		this._logService.trace('IPty#spawn', shellLaunchConfig.executable, args, options);
+		this._logService.trace('node-pty.IPty#spawn', shellLaunchConfig.executable, args, options);
 		const ptyProcess = spawn(shellLaunchConfig.executable!, args, options);
 		this._ptyProcess = ptyProcess;
 		this._childProcessMonitor = this._register(new ChildProcessMonitor(ptyProcess.pid, this._logService));
@@ -312,7 +312,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 			}
 
 			// Refire the data event
-			this._logService.trace('IPty#onData', data);
+			this._logService.trace('node-pty.IPty#onData', data);
 			this._onProcessData.fire(data);
 			if (this._closeTimeout) {
 				this._queueProcessExit();
@@ -353,6 +353,9 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 	// Allow any trailing data events to be sent before the exit event is sent.
 	// See https://github.com/Tyriar/node-pty/issues/72
 	private _queueProcessExit() {
+		if (this._logService.getLevel() === LogLevel.Trace) {
+			this._logService.trace('TerminalProcess#_queueProcessExit', new Error().stack?.replace(/^Error/, ''));
+		}
 		if (this._closeTimeout) {
 			clearTimeout(this._closeTimeout);
 		}
@@ -374,8 +377,8 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 		try {
 			if (this._ptyProcess) {
 				await this._throttleKillSpawn();
-				this._logService.trace('IPty#kill');
-				this._ptyProcess.kill(!isWindows ? 'SIGKILL' : undefined);
+				this._logService.trace('node-pty.IPty#kill');
+				this._ptyProcess.kill();
 			}
 		} catch (ex) {
 			// Swallow, the pty has already been killed
@@ -398,7 +401,11 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 	}
 
 	private _sendProcessId(pid: number) {
-		this._onProcessReady.fire({ pid, cwd: this._initialCwd, requiresWindowsMode: isWindows && getWindowsBuildNumber() < 21376 });
+		this._onProcessReady.fire({
+			pid,
+			cwd: this._initialCwd,
+			windowsPty: this.getWindowsPty()
+		});
 	}
 
 	private _sendProcessTitle(ptyProcess: IPty): void {
@@ -413,6 +420,9 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 	}
 
 	shutdown(immediate: boolean): void {
+		if (this._logService.getLevel() === LogLevel.Trace) {
+			this._logService.trace('TerminalProcess#shutdown', new Error().stack?.replace(/^Error/, ''));
+		}
 		// don't force immediate disposal of the terminal processes on Windows as an additional
 		// mitigation for https://github.com/microsoft/vscode/issues/71966 which causes the pty host
 		// to become unresponsive, disconnecting all terminals across all windows.
@@ -504,7 +514,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 
 	private _doWrite(): void {
 		const object = this._writeQueue.shift()!;
-		this._logService.trace('IPty#write', object.data);
+		this._logService.trace('node-pty.IPty#write', object.data);
 		if (object.isBinary) {
 			this._ptyProcess!.write(Buffer.from(object.data, 'binary') as any);
 		} else {
@@ -533,12 +543,12 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 				return;
 			}
 
-			this._logService.trace('IPty#resize', cols, rows);
+			this._logService.trace('node-pty.IPty#resize', cols, rows);
 			try {
 				this._ptyProcess.resize(cols, rows);
 			} catch (e) {
 				// Swallow error if the pty has already exited
-				this._logService.trace('IPty#resize exception ' + e.message);
+				this._logService.trace('node-pty.IPty#resize exception ' + e.message);
 				if (this._exitCode !== undefined &&
 					e.message !== 'ioctl(2) failed, EBADF' &&
 					e.message !== 'Cannot resize a pty that has already exited') {
@@ -548,6 +558,10 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 		}
 	}
 
+	clearBuffer(): void {
+		this._ptyProcess?.clear();
+	}
+
 	acknowledgeDataEvent(charCount: number): void {
 		// Prevent lower than 0 to heal from errors
 		this._unacknowledgedCharCount = Math.max(this._unacknowledgedCharCount - charCount, 0);
@@ -586,7 +600,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 					resolve(this._initialCwd);
 					return;
 				}
-				this._logService.trace('IPty#pid');
+				this._logService.trace('node-pty.IPty#pid');
 				exec('lsof -OPln -p ' + this._ptyProcess.pid + ' | grep cwd', { env: { ...process.env, LANG: 'en_US.UTF-8' } }, (error, stdout, stderr) => {
 					if (!error && stdout !== '') {
 						resolve(stdout.substring(stdout.indexOf('/'), stdout.length - 1));
@@ -602,7 +616,7 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 			if (!this._ptyProcess) {
 				return this._initialCwd;
 			}
-			this._logService.trace('IPty#pid');
+			this._logService.trace('node-pty.IPty#pid');
 			try {
 				return await Promises.readlink(`/proc/${this._ptyProcess.pid}/cwd`);
 			} catch (error) {
@@ -613,8 +627,11 @@ export class TerminalProcess extends Disposable implements ITerminalChildProcess
 		return this._initialCwd;
 	}
 
-	getLatency(): Promise<number> {
-		return Promise.resolve(0);
+	getWindowsPty(): IProcessReadyWindowsPty | undefined {
+		return isWindows ? {
+			backend: 'useConpty' in this._ptyOptions && this._ptyOptions.useConpty ? 'conpty' : 'winpty',
+			buildNumber: getWindowsBuildNumber()
+		} : undefined;
 	}
 }
 
