@@ -15,10 +15,10 @@ import { URI } from 'vs/base/common/uri';
 import { IConfigurationService } from 'vs/platform/configuration/common/configuration';
 import { IEnvironmentService } from 'vs/platform/environment/common/environment';
 import { GlobalExtensionEnablementService } from 'vs/platform/extensionManagement/common/extensionEnablementService';
-import { IExtensionGalleryService, IExtensionManagementService, IGlobalExtensionEnablementService, ILocalExtension, ExtensionManagementError, ExtensionManagementErrorCode, IGalleryExtension, DISABLED_EXTENSIONS_STORAGE_PATH } from 'vs/platform/extensionManagement/common/extensionManagement';
+import { IExtensionGalleryService, IExtensionManagementService, IGlobalExtensionEnablementService, ILocalExtension, ExtensionManagementError, ExtensionManagementErrorCode, IGalleryExtension, DISABLED_EXTENSIONS_STORAGE_PATH, EXTENSION_INSTALL_SKIP_WALKTHROUGH_CONTEXT, EXTENSION_INSTALL_SYNC_CONTEXT, InstallExtensionInfo } from 'vs/platform/extensionManagement/common/extensionManagement';
 import { areSameExtensions } from 'vs/platform/extensionManagement/common/extensionManagementUtil';
 import { ExtensionStorageService, IExtensionStorageService } from 'vs/platform/extensionManagement/common/extensionStorage';
-import { ExtensionType, IExtensionIdentifier } from 'vs/platform/extensions/common/extensions';
+import { ExtensionType, IExtensionIdentifier, isApplicationScopedExtension } from 'vs/platform/extensions/common/extensions';
 import { IFileService } from 'vs/platform/files/common/files';
 import { IInstantiationService } from 'vs/platform/instantiation/common/instantiation';
 import { ServiceCollection } from 'vs/platform/instantiation/common/serviceCollection';
@@ -30,16 +30,16 @@ import { IUserDataProfile, IUserDataProfilesService } from 'vs/platform/userData
 import { AbstractInitializer, AbstractSynchroniser, getSyncResourceLogLabel, IAcceptResult, IMergeResult, IResourcePreview } from 'vs/platform/userDataSync/common/abstractSynchronizer';
 import { IMergeResult as IExtensionMergeResult, merge } from 'vs/platform/userDataSync/common/extensionsMerge';
 import { IIgnoredExtensionsManagementService } from 'vs/platform/userDataSync/common/ignoredExtensions';
-import { Change, IRemoteUserData, ISyncData, ISyncExtension, ISyncExtensionWithVersion, IUserDataSyncBackupStoreService, IUserDataSynchroniser, IUserDataSyncLogService, IUserDataSyncEnablementService, IUserDataSyncStoreService, SyncResource, USER_DATA_SYNC_SCHEME } from 'vs/platform/userDataSync/common/userDataSync';
+import { Change, IRemoteUserData, ISyncData, ISyncExtension, IUserDataSyncBackupStoreService, IUserDataSynchroniser, IUserDataSyncLogService, IUserDataSyncEnablementService, IUserDataSyncStoreService, SyncResource, USER_DATA_SYNC_SCHEME, ILocalSyncExtension } from 'vs/platform/userDataSync/common/userDataSync';
 import { IUserDataProfileStorageService } from 'vs/platform/userDataProfile/common/userDataProfileStorageService';
 
 type IExtensionResourceMergeResult = IAcceptResult & IExtensionMergeResult;
 
 interface IExtensionResourcePreview extends IResourcePreview {
-	readonly localExtensions: ISyncExtensionWithVersion[];
+	readonly localExtensions: ILocalSyncExtension[];
 	readonly remoteExtensions: ISyncExtension[] | null;
 	readonly skippedExtensions: ISyncExtension[];
-	readonly builtinExtensions: IExtensionIdentifier[];
+	readonly builtinExtensions: IExtensionIdentifier[] | null;
 	readonly previewResult: IExtensionResourceMergeResult;
 }
 
@@ -101,7 +101,8 @@ export class ExtensionsSynchroniser extends AbstractSynchroniser implements IUse
 	*/
 	/* Version 4: Change settings from `sync.${setting}` to `settingsSync.{setting}` */
 	/* Version 5: Introduce extension state */
-	protected readonly version: number = 5;
+	/* Version 6: Added isApplicationScoped property */
+	protected readonly version: number = 6;
 
 	private readonly previewResource: URI = this.extUri.joinPath(this.syncPreviewFolder, 'extensions.json');
 	private readonly baseResource: URI = this.previewResource.with({ scheme: USER_DATA_SYNC_SCHEME, authority: 'base' });
@@ -142,10 +143,10 @@ export class ExtensionsSynchroniser extends AbstractSynchroniser implements IUse
 	}
 
 	protected async generateSyncPreview(remoteUserData: IRemoteUserData, lastSyncUserData: ILastSyncUserData | null): Promise<IExtensionResourcePreview[]> {
-		const remoteExtensions: ISyncExtension[] | null = remoteUserData.syncData ? await parseAndMigrateExtensions(remoteUserData.syncData, this.extensionManagementService) : null;
-		const skippedExtensions: ISyncExtension[] = lastSyncUserData?.skippedExtensions || [];
-		const builtinExtensions: IExtensionIdentifier[] = lastSyncUserData?.builtinExtensions || [];
-		const lastSyncExtensions: ISyncExtension[] | null = lastSyncUserData?.syncData ? await parseAndMigrateExtensions(lastSyncUserData.syncData, this.extensionManagementService) : null;
+		const remoteExtensions = remoteUserData.syncData ? await parseAndMigrateExtensions(remoteUserData.syncData, this.extensionManagementService) : null;
+		const skippedExtensions = lastSyncUserData?.skippedExtensions ?? [];
+		const builtinExtensions = lastSyncUserData?.builtinExtensions ?? null;
+		const lastSyncExtensions = lastSyncUserData?.syncData ? await parseAndMigrateExtensions(lastSyncUserData.syncData, this.extensionManagementService) : null;
 
 		const { localExtensions, ignoredExtensions } = await this.localExtensionsProvider.getLocalExtensions(this.syncResource.profile);
 
@@ -278,7 +279,7 @@ export class ExtensionsSynchroniser extends AbstractSynchroniser implements IUse
 	}
 
 	protected async applyResult(remoteUserData: IRemoteUserData, lastSyncUserData: IRemoteUserData | null, resourcePreviews: [IExtensionResourcePreview, IExtensionResourceMergeResult][], force: boolean): Promise<void> {
-		let { skippedExtensions, localExtensions } = resourcePreviews[0][0];
+		let { skippedExtensions, builtinExtensions, localExtensions } = resourcePreviews[0][0];
 		const { local, remote, localChange, remoteChange } = resourcePreviews[0][1];
 
 		if (localChange === Change.None && remoteChange === Change.None) {
@@ -301,12 +302,32 @@ export class ExtensionsSynchroniser extends AbstractSynchroniser implements IUse
 		if (lastSyncUserData?.ref !== remoteUserData.ref) {
 			// update last sync
 			this.logService.trace(`${this.syncResourceLogLabel}: Updating last synchronized extensions...`);
-			const builtinExtensions = localExtensions.filter(e => !e.installed).map(e => e.identifier);
+			builtinExtensions = this.computeBuiltinExtensions(localExtensions, builtinExtensions);
 			await this.updateLastSyncUserData(remoteUserData, { skippedExtensions, builtinExtensions });
 			this.logService.info(`${this.syncResourceLogLabel}: Updated last synchronized extensions.${skippedExtensions.length ? ` Skipped: ${JSON.stringify(skippedExtensions.map(e => e.identifier.id))}.` : ''}`);
 		}
 	}
 
+	private computeBuiltinExtensions(localExtensions: ILocalSyncExtension[], previousBuiltinExtensions: IExtensionIdentifier[] | null): IExtensionIdentifier[] {
+		const localExtensionsSet = new Set<string>();
+		const builtinExtensions: IExtensionIdentifier[] = [];
+		for (const localExtension of localExtensions) {
+			localExtensionsSet.add(localExtension.identifier.id.toLowerCase());
+			if (!localExtension.installed) {
+				builtinExtensions.push(localExtension.identifier);
+			}
+		}
+		if (previousBuiltinExtensions) {
+			for (const builtinExtension of previousBuiltinExtensions) {
+				// Add previous builtin extension if it does not exist in local extensions
+				if (!localExtensionsSet.has(builtinExtension.id.toLowerCase())) {
+					builtinExtensions.push(builtinExtension);
+				}
+			}
+		}
+		return builtinExtensions;
+	}
+
 	async resolveContent(uri: URI): Promise<string | null> {
 		if (this.extUri.isEqual(this.remoteResource, uri)
 			|| this.extUri.isEqual(this.baseResource, uri)
@@ -348,15 +369,18 @@ export class LocalExtensionsProvider {
 		@IUserDataSyncLogService private readonly logService: IUserDataSyncLogService,
 	) { }
 
-	async getLocalExtensions(profile: IUserDataProfile): Promise<{ localExtensions: ISyncExtensionWithVersion[]; ignoredExtensions: string[] }> {
+	async getLocalExtensions(profile: IUserDataProfile): Promise<{ localExtensions: ILocalSyncExtension[]; ignoredExtensions: string[] }> {
 		const installedExtensions = await this.extensionManagementService.getInstalled(undefined, profile.extensionsResource);
 		const ignoredExtensions = this.ignoredExtensionsManagementService.getIgnoredExtensions(installedExtensions);
 		const localExtensions = await this.withProfileScopedServices(profile, async (extensionEnablementService, extensionStorageService) => {
 			const disabledExtensions = extensionEnablementService.getDisabledExtensions();
 			return installedExtensions
 				.map(extension => {
-					const { identifier, isBuiltin, manifest, preRelease } = extension;
-					const syncExntesion: ISyncExtensionWithVersion = { identifier, preRelease, version: manifest.version };
+					const { identifier, isBuiltin, manifest, preRelease, pinned, isApplicationScoped } = extension;
+					const syncExntesion: ILocalSyncExtension = { identifier, preRelease, version: manifest.version, pinned: !!pinned };
+					if (isApplicationScoped && !isApplicationScopedExtension(manifest)) {
+						syncExntesion.isApplicationScoped = isApplicationScoped;
+					}
 					if (disabledExtensions.some(disabledExtension => areSameExtensions(disabledExtension, identifier))) {
 						syncExntesion.disabled = true;
 					}
@@ -385,22 +409,15 @@ export class LocalExtensionsProvider {
 
 	async updateLocalExtensions(added: ISyncExtension[], removed: IExtensionIdentifier[], updated: ISyncExtension[], skippedExtensions: ISyncExtension[], profile: IUserDataProfile): Promise<ISyncExtension[]> {
 		const syncResourceLogLabel = getSyncResourceLogLabel(SyncResource.Extensions, profile);
-		return this.withProfileScopedServices(profile, async (extensionEnablementService, extensionStorageService) => {
-			const removeFromSkipped: IExtensionIdentifier[] = [];
-			const addToSkipped: ISyncExtension[] = [];
-			const installedExtensions = await this.extensionManagementService.getInstalled(undefined, profile.extensionsResource);
+		const extensionsToInstall: InstallExtensionInfo[] = [];
+		const syncExtensionsToInstall = new Map<string, ISyncExtension>();
+		const removeFromSkipped: IExtensionIdentifier[] = [];
+		const addToSkipped: ISyncExtension[] = [];
+		const installedExtensions = await this.extensionManagementService.getInstalled(undefined, profile.extensionsResource);
 
-			if (removed.length) {
-				const extensionsToRemove = installedExtensions.filter(({ identifier, isBuiltin }) => !isBuiltin && removed.some(r => areSameExtensions(identifier, r)));
-				await Promises.settled(extensionsToRemove.map(async extensionToRemove => {
-					this.logService.trace(`${syncResourceLogLabel}: Uninstalling local extension...`, extensionToRemove.identifier.id);
-					await this.extensionManagementService.uninstall(extensionToRemove, { donotIncludePack: true, donotCheckDependents: true, profileLocation: profile.extensionsResource });
-					this.logService.info(`${syncResourceLogLabel}: Uninstalled local extension.`, extensionToRemove.identifier.id);
-					removeFromSkipped.push(extensionToRemove.identifier);
-				}));
-			}
-
-			if (added.length || updated.length) {
+		// 1. Sync extensions state first so that the storage is flushed and updated in all opened windows
+		if (added.length || updated.length) {
+			await this.withProfileScopedServices(profile, async (extensionEnablementService, extensionStorageService) => {
 				await Promises.settled([...added, ...updated].map(async e => {
 					const installedExtension = installedExtensions.find(installed => areSameExtensions(installed.identifier, e.identifier));
 
@@ -426,14 +443,15 @@ export class LocalExtensionsProvider {
 					}
 
 					// User Extension Sync: Install/Update, Enablement & State
-					const extension = (await this.extensionGalleryService.getExtensions([{ ...e.identifier, preRelease: e.preRelease }], CancellationToken.None))[0];
+					const version = e.pinned ? e.version : undefined;
+					const extension = (await this.extensionGalleryService.getExtensions([{ ...e.identifier, version, preRelease: version ? undefined : e.preRelease }], CancellationToken.None))[0];
 
 					/* Update extension state only if
 					 *	extension is installed and version is same as synced version or
 					 *	extension is not installed and installable
 					 */
 					if (e.state &&
-						(installedExtension ? installedExtension.manifest.version === e.version /* Installed and has same version */
+						(installedExtension ? installedExtension.manifest.version === e.version /* Installed and remote has same version */
 							: !!extension /* Installable */)
 					) {
 						this.updateExtensionState(e.state, installedExtension || extension, installedExtension?.manifest.version, extensionStorageService);
@@ -456,12 +474,22 @@ export class LocalExtensionsProvider {
 
 							if (!installedExtension // Install if the extension does not exist
 								|| installedExtension.preRelease !== e.preRelease // Install if the extension pre-release preference has changed
+								|| installedExtension.pinned !== e.pinned  // Install if the extension pinned preference has changed
+								|| (version && installedExtension.manifest.version !== version)  // Install if the extension version has changed
 							) {
 								if (await this.extensionManagementService.canInstall(extension)) {
-									this.logService.trace(`${syncResourceLogLabel}: Installing extension...`, e.identifier.id, extension.version);
-									await this.extensionManagementService.installFromGallery(extension, { isMachineScoped: false, donotIncludePackAndDependencies: true, installPreReleaseVersion: e.preRelease, profileLocation: profile.extensionsResource } /* set isMachineScoped value to prevent install and sync dialog in web */);
-									this.logService.info(`${syncResourceLogLabel}: Installed extension.`, e.identifier.id, extension.version);
-									removeFromSkipped.push(extension.identifier);
+									extensionsToInstall.push({
+										extension, options: {
+											isMachineScoped: false /* set isMachineScoped value to prevent install and sync dialog in web */,
+											donotIncludePackAndDependencies: true,
+											installGivenVersion: e.pinned && !!e.version,
+											installPreReleaseVersion: e.preRelease,
+											profileLocation: profile.extensionsResource,
+											isApplicationScoped: e.isApplicationScoped,
+											context: { [EXTENSION_INSTALL_SKIP_WALKTHROUGH_CONTEXT]: true, [EXTENSION_INSTALL_SYNC_CONTEXT]: true }
+										}
+									});
+									syncExtensionsToInstall.set(extension.identifier.id.toLowerCase(), e);
 								} else {
 									this.logService.info(`${syncResourceLogLabel}: Skipped synchronizing extension because it cannot be installed.`, extension.displayName || extension.identifier.id);
 									addToSkipped.push(e);
@@ -469,33 +497,61 @@ export class LocalExtensionsProvider {
 							}
 						} catch (error) {
 							addToSkipped.push(e);
-							if (error instanceof ExtensionManagementError && [ExtensionManagementErrorCode.Incompatible, ExtensionManagementErrorCode.IncompatiblePreRelease, ExtensionManagementErrorCode.IncompatibleTargetPlatform].includes(error.code)) {
-								this.logService.info(`${syncResourceLogLabel}: Skipped synchronizing extension because the compatible extension is not found.`, extension.displayName || extension.identifier.id);
-							} else {
-								this.logService.error(error);
-								this.logService.info(`${syncResourceLogLabel}: Skipped synchronizing extension`, extension.displayName || extension.identifier.id);
-							}
+							this.logService.error(error);
+							this.logService.info(`${syncResourceLogLabel}: Skipped synchronizing extension`, extension.displayName || extension.identifier.id);
 						}
 					} else {
 						addToSkipped.push(e);
 						this.logService.info(`${syncResourceLogLabel}: Skipped synchronizing extension because the extension is not found.`, e.identifier.id);
 					}
 				}));
-			}
+			});
+		}
 
-			const newSkippedExtensions: ISyncExtension[] = [];
-			for (const skippedExtension of skippedExtensions) {
-				if (!removeFromSkipped.some(e => areSameExtensions(e, skippedExtension.identifier))) {
-					newSkippedExtensions.push(skippedExtension);
+		// 2. Next uninstall the removed extensions
+		if (removed.length) {
+			const extensionsToRemove = installedExtensions.filter(({ identifier, isBuiltin }) => !isBuiltin && removed.some(r => areSameExtensions(identifier, r)));
+			await Promises.settled(extensionsToRemove.map(async extensionToRemove => {
+				this.logService.trace(`${syncResourceLogLabel}: Uninstalling local extension...`, extensionToRemove.identifier.id);
+				await this.extensionManagementService.uninstall(extensionToRemove, { donotIncludePack: true, donotCheckDependents: true, profileLocation: profile.extensionsResource });
+				this.logService.info(`${syncResourceLogLabel}: Uninstalled local extension.`, extensionToRemove.identifier.id);
+				removeFromSkipped.push(extensionToRemove.identifier);
+			}));
+		}
+
+		// 3. Install extensions at the end
+		const results = await this.extensionManagementService.installGalleryExtensions(extensionsToInstall);
+		for (const { identifier, local, error, source } of results) {
+			const gallery = source as IGalleryExtension;
+			if (local) {
+				this.logService.info(`${syncResourceLogLabel}: Installed extension.`, identifier.id, gallery.version);
+				removeFromSkipped.push(identifier);
+			} else {
+				const e = syncExtensionsToInstall.get(identifier.id.toLowerCase());
+				if (e) {
+					addToSkipped.push(e);
+					this.logService.info(`${syncResourceLogLabel}: Skipped synchronizing extension`, gallery.displayName || gallery.identifier.id);
+				}
+				if (error instanceof ExtensionManagementError && [ExtensionManagementErrorCode.Incompatible, ExtensionManagementErrorCode.IncompatibleTargetPlatform].includes(error.code)) {
+					this.logService.info(`${syncResourceLogLabel}: Skipped synchronizing extension because the compatible extension is not found.`, gallery.displayName || gallery.identifier.id);
+				} else if (error) {
+					this.logService.error(error);
 				}
 			}
-			for (const skippedExtension of addToSkipped) {
-				if (!newSkippedExtensions.some(e => areSameExtensions(e.identifier, skippedExtension.identifier))) {
-					newSkippedExtensions.push(skippedExtension);
-				}
+		}
+
+		const newSkippedExtensions: ISyncExtension[] = [];
+		for (const skippedExtension of skippedExtensions) {
+			if (!removeFromSkipped.some(e => areSameExtensions(e, skippedExtension.identifier))) {
+				newSkippedExtensions.push(skippedExtension);
 			}
-			return newSkippedExtensions;
-		});
+		}
+		for (const skippedExtension of addToSkipped) {
+			if (!newSkippedExtensions.some(e => areSameExtensions(e.identifier, skippedExtension.identifier))) {
+				newSkippedExtensions.push(skippedExtension);
+			}
+		}
+		return newSkippedExtensions;
 	}
 
 	private updateExtensionState(state: IStringDictionary<any>, extension: ILocalExtension | IGalleryExtension, version: string | undefined, extensionStorageService: IExtensionStorageService): void {
