@@ -7,39 +7,59 @@ import type * as vscode from 'vscode';
 import { createDecorator } from 'vs/platform/instantiation/common/instantiation';
 import { Event, Emitter } from 'vs/base/common/event';
 import { ExtHostTelemetryShape } from 'vs/workbench/api/common/extHost.protocol';
-import { TelemetryLevel } from 'vs/platform/telemetry/common/telemetry';
-import { ILogger, ILoggerService } from 'vs/platform/log/common/log';
+import { ICommonProperties, TelemetryLevel } from 'vs/platform/telemetry/common/telemetry';
+import { ILogger, ILoggerService, LogLevel, isLogLevel } from 'vs/platform/log/common/log';
 import { IExtHostInitDataService } from 'vs/workbench/api/common/extHostInitDataService';
 import { ExtensionIdentifier, IExtensionDescription } from 'vs/platform/extensions/common/extensions';
 import { UIKind } from 'vs/workbench/services/extensions/common/extensionHostProtocol';
 import { getRemoteName } from 'vs/platform/remote/common/remoteHosts';
-import { cleanData, cleanRemoteAuthority } from 'vs/platform/telemetry/common/telemetryUtils';
+import { cleanData, cleanRemoteAuthority, extensionTelemetryLogChannelId } from 'vs/platform/telemetry/common/telemetryUtils';
 import { mixin } from 'vs/base/common/objects';
 import { URI } from 'vs/base/common/uri';
+import { Disposable } from 'vs/base/common/lifecycle';
+import { localize } from 'vs/nls';
 
-export class ExtHostTelemetry implements ExtHostTelemetryShape {
-	private readonly _onDidChangeTelemetryEnabled = new Emitter<boolean>();
+export class ExtHostTelemetry extends Disposable implements ExtHostTelemetryShape {
+
+	readonly _serviceBrand: undefined;
+
+	private readonly _onDidChangeTelemetryEnabled = this._register(new Emitter<boolean>());
 	readonly onDidChangeTelemetryEnabled: Event<boolean> = this._onDidChangeTelemetryEnabled.event;
 
-	private readonly _onDidChangeTelemetryConfiguration = new Emitter<vscode.TelemetryConfiguration>();
+	private readonly _onDidChangeTelemetryConfiguration = this._register(new Emitter<vscode.TelemetryConfiguration>());
 	readonly onDidChangeTelemetryConfiguration: Event<vscode.TelemetryConfiguration> = this._onDidChangeTelemetryConfiguration.event;
 
 	private _productConfig: { usage: boolean; error: boolean } = { usage: true, error: true };
 	private _level: TelemetryLevel = TelemetryLevel.NONE;
+	// This holds whether or not we're running with --disable-telemetry, etc. Usings supportsTelemtry() from the main thread
+	private _telemetryIsSupported: boolean = false;
 	private _oldTelemetryEnablement: boolean | undefined;
-	private _inLoggingOnlyMode: boolean = false;
+	private readonly _inLoggingOnlyMode: boolean = false;
+	private readonly extHostTelemetryLogFile: URI;
 	private readonly _outputLogger: ILogger;
-	private readonly _telemetryLoggers = new Map<string, ExtHostTelemetryLogger>();
+	private readonly _telemetryLoggers = new Map<string, ExtHostTelemetryLogger[]>();
 
 	constructor(
 		@IExtHostInitDataService private readonly initData: IExtHostInitDataService,
-		@ILoggerService loggerService: ILoggerService,
+		@ILoggerService private readonly loggerService: ILoggerService,
 	) {
-		this._outputLogger = loggerService.createLogger(URI.revive(this.initData.environment.extensionTelemetryLogResource));
+		super();
+		this.extHostTelemetryLogFile = URI.revive(this.initData.environment.extensionTelemetryLogResource);
+		this._inLoggingOnlyMode = this.initData.environment.isExtensionTelemetryLoggingOnly;
+		this._outputLogger = loggerService.createLogger(this.extHostTelemetryLogFile, { id: extensionTelemetryLogChannelId, name: localize('extensionTelemetryLog', "Extension Telemetry{0}", this._inLoggingOnlyMode ? ' (Not Sent)' : ''), hidden: true });
+		this._register(loggerService.onDidChangeLogLevel(arg => {
+			if (isLogLevel(arg)) {
+				this.updateLoggerVisibility();
+			}
+		}));
 		this._outputLogger.info('Below are logs for extension telemetry events sent to the telemetry output channel API once the log level is set to trace.');
 		this._outputLogger.info('===========================================================');
 	}
 
+	private updateLoggerVisibility(): void {
+		this.loggerService.setVisibility(this.extHostTelemetryLogFile, this._telemetryIsSupported && this.loggerService.getLogLevel() === LogLevel.Trace);
+	}
+
 	getTelemetryConfiguration(): boolean {
 		return this._level === TelemetryLevel.USAGE;
 	}
@@ -52,31 +72,34 @@ export class ExtHostTelemetry implements ExtHostTelemetryShape {
 		};
 	}
 
-	instantiateLogger(extension: IExtensionDescription, appender: vscode.TelemetryAppender) {
+	instantiateLogger(extension: IExtensionDescription, sender: vscode.TelemetrySender, options?: vscode.TelemetryLoggerOptions) {
 		const telemetryDetails = this.getTelemetryDetails();
 		const logger = new ExtHostTelemetryLogger(
-			appender,
+			sender,
+			options,
 			extension,
 			this._outputLogger,
 			this._inLoggingOnlyMode,
 			this.getBuiltInCommonProperties(extension),
 			{ isUsageEnabled: telemetryDetails.isUsageEnabled, isErrorsEnabled: telemetryDetails.isErrorsEnabled }
 		);
-		this._telemetryLoggers.set(extension.identifier.value, logger);
+		const loggers = this._telemetryLoggers.get(extension.identifier.value) ?? [];
+		this._telemetryLoggers.set(extension.identifier.value, [...loggers, logger]);
 		return logger.apiTelemetryLogger;
 	}
 
-	$initializeTelemetryLevel(level: TelemetryLevel, loggingOnlyMode: boolean, productConfig?: { usage: boolean; error: boolean }): void {
+	$initializeTelemetryLevel(level: TelemetryLevel, supportsTelemetry: boolean, productConfig?: { usage: boolean; error: boolean }): void {
 		this._level = level;
-		this._inLoggingOnlyMode = loggingOnlyMode;
+		this._telemetryIsSupported = supportsTelemetry;
 		this._productConfig = productConfig ?? { usage: true, error: true };
+		this.updateLoggerVisibility();
 	}
 
-	getBuiltInCommonProperties(extension: IExtensionDescription): Record<string, string | boolean | number | undefined> {
-		const commonProperties: Record<string, string | boolean | number | undefined> = {};
+	getBuiltInCommonProperties(extension: IExtensionDescription): ICommonProperties {
+		const commonProperties: ICommonProperties = Object.create(null);
 		// TODO @lramos15, does os info like node arch, platform version, etc exist here.
 		// Or will first party extensions just mix this in
-		commonProperties['common.extname'] = extension.name;
+		commonProperties['common.extname'] = `${extension.publisher}.${extension.name}`;
 		commonProperties['common.extversion'] = extension.version;
 		commonProperties['common.vscodemachineid'] = this.initData.telemetryInfo.machineId;
 		commonProperties['common.vscodesessionid'] = this.initData.telemetryInfo.sessionId;
@@ -104,42 +127,87 @@ export class ExtHostTelemetry implements ExtHostTelemetryShape {
 		this._oldTelemetryEnablement = this.getTelemetryConfiguration();
 		this._level = level;
 		const telemetryDetails = this.getTelemetryDetails();
-
+		// Remove all disposed loggers
+		this._telemetryLoggers.forEach((loggers, key) => {
+			const newLoggers = loggers.filter(l => !l.isDisposed);
+			if (newLoggers.length === 0) {
+				this._telemetryLoggers.delete(key);
+			} else {
+				this._telemetryLoggers.set(key, newLoggers);
+			}
+		});
 		// Loop through all loggers and update their level
-		this._telemetryLoggers.forEach(logger => {
-			logger.updateTelemetryEnablements(telemetryDetails.isUsageEnabled, telemetryDetails.isErrorsEnabled);
+		this._telemetryLoggers.forEach(loggers => {
+			for (const logger of loggers) {
+				logger.updateTelemetryEnablements(telemetryDetails.isUsageEnabled, telemetryDetails.isErrorsEnabled);
+			}
 		});
 
 		if (this._oldTelemetryEnablement !== this.getTelemetryConfiguration()) {
 			this._onDidChangeTelemetryEnabled.fire(this.getTelemetryConfiguration());
 		}
 		this._onDidChangeTelemetryConfiguration.fire(this.getTelemetryDetails());
+		this.updateLoggerVisibility();
 	}
 
 	onExtensionError(extension: ExtensionIdentifier, error: Error): boolean {
-		const logger = this._telemetryLoggers.get(extension.value);
-		if (!logger) {
+		const loggers = this._telemetryLoggers.get(extension.value);
+		const nonDisposedLoggers = loggers?.filter(l => !l.isDisposed);
+		if (!nonDisposedLoggers) {
+			this._telemetryLoggers.delete(extension.value);
 			return false;
 		}
-		logger.logError(error);
-		return true;
+		let errorEmitted = false;
+		for (const logger of nonDisposedLoggers) {
+			if (logger.ignoreUnhandledExtHostErrors) {
+				continue;
+			}
+			logger.logError(error);
+			errorEmitted = true;
+		}
+		return errorEmitted;
 	}
 }
 
 export class ExtHostTelemetryLogger {
-	private _appender: vscode.TelemetryAppender;
+
+	static validateSender(sender: vscode.TelemetrySender): void {
+		if (typeof sender !== 'object') {
+			throw new TypeError('TelemetrySender argument is invalid');
+		}
+		if (typeof sender.sendEventData !== 'function') {
+			throw new TypeError('TelemetrySender.sendEventData must be a function');
+		}
+		if (typeof sender.sendErrorData !== 'function') {
+			throw new TypeError('TelemetrySender.sendErrorData must be a function');
+		}
+		if (typeof sender.flush !== 'undefined' && typeof sender.flush !== 'function') {
+			throw new TypeError('TelemetrySender.flush must be a function or undefined');
+		}
+	}
+
 	private readonly _onDidChangeEnableStates = new Emitter<vscode.TelemetryLogger>();
+	private readonly _ignoreBuiltinCommonProperties: boolean;
+	private readonly _additionalCommonProperties: Record<string, any> | undefined;
+	public readonly ignoreUnhandledExtHostErrors: boolean;
+
 	private _telemetryEnablements: { isUsageEnabled: boolean; isErrorsEnabled: boolean };
 	private _apiObject: vscode.TelemetryLogger | undefined;
+	private _sender: vscode.TelemetrySender | undefined;
+
 	constructor(
-		appender: vscode.TelemetryAppender,
+		sender: vscode.TelemetrySender,
+		options: vscode.TelemetryLoggerOptions | undefined,
 		private readonly _extension: IExtensionDescription,
 		private readonly _logger: ILogger,
 		private readonly _inLoggingOnlyMode: boolean,
 		private readonly _commonProperties: Record<string, any>,
 		telemetryEnablements: { isUsageEnabled: boolean; isErrorsEnabled: boolean }
 	) {
-		this._appender = appender;
+		this.ignoreUnhandledExtHostErrors = options?.ignoreUnhandledErrors ?? false;
+		this._ignoreBuiltinCommonProperties = options?.ignoreBuiltInCommonProperties ?? false;
+		this._additionalCommonProperties = options?.additionalCommonProperties;
+		this._sender = sender;
 		this._telemetryEnablements = { isUsageEnabled: telemetryEnablements.isUsageEnabled, isErrorsEnabled: telemetryEnablements.isErrorsEnabled };
 	}
 
@@ -153,21 +221,20 @@ export class ExtHostTelemetryLogger {
 	mixInCommonPropsAndCleanData(data: Record<string, any>): Record<string, any> {
 		// Some telemetry modules prefer to break properties and measurmements up
 		// We mix common properties into the properties tab.
-		// TODO @lramos15 should this be up to the implementer and not done here?
-		let updatedData = data.properties ?? data;
+		let updatedData = 'properties' in data ? (data.properties ?? {}) : data;
 
 		// We don't clean measurements since they are just numbers
 		updatedData = cleanData(updatedData, []);
 
-		if (this._appender.additionalCommonProperties) {
-			updatedData = mixin(updatedData, this._appender.additionalCommonProperties);
+		if (this._additionalCommonProperties) {
+			updatedData = mixin(updatedData, this._additionalCommonProperties);
 		}
 
-		if (!this._appender.ignoreBuiltInCommonProperties) {
+		if (!this._ignoreBuiltinCommonProperties) {
 			updatedData = mixin(updatedData, this._commonProperties);
 		}
 
-		if (data.properties) {
+		if ('properties' in data) {
 			data.properties = updatedData;
 		} else {
 			data = updatedData;
@@ -177,6 +244,10 @@ export class ExtHostTelemetryLogger {
 	}
 
 	private logEvent(eventName: string, data?: Record<string, any>): void {
+		// No sender means likely disposed of, we should no-op
+		if (!this._sender) {
+			return;
+		}
 		// If it's a built-in extension (vscode publisher) we don't prefix the publisher and only the ext name
 		if (this._extension.publisher === 'vscode') {
 			eventName = this._extension.name + '/' + eventName;
@@ -185,7 +256,7 @@ export class ExtHostTelemetryLogger {
 		}
 		data = this.mixInCommonPropsAndCleanData(data || {});
 		if (!this._inLoggingOnlyMode) {
-			this._appender.logEvent(eventName, data);
+			this._sender?.sendEventData(eventName, data);
 		}
 		this._logger.trace(eventName, data);
 	}
@@ -198,14 +269,14 @@ export class ExtHostTelemetryLogger {
 	}
 
 	logError(eventNameOrException: Error | string, data?: Record<string, any>): void {
-		if (!this._telemetryEnablements.isErrorsEnabled) {
+		if (!this._telemetryEnablements.isErrorsEnabled || !this._sender) {
 			return;
 		}
 		if (typeof eventNameOrException === 'string') {
 			this.logEvent(eventNameOrException, data);
 		} else {
 			// TODO @lramos15, implement cleaning for and logging for this case
-			this._appender.logException(eventNameOrException, data);
+			this._sender.sendErrorData(eventNameOrException, data);
 		}
 	}
 
@@ -229,9 +300,18 @@ export class ExtHostTelemetryLogger {
 		return this._apiObject;
 	}
 
+	get isDisposed(): boolean {
+		return !this._sender;
+	}
+
 	dispose(): void {
-		if (this._appender?.flush) {
-			this._appender.flush();
+		if (this._sender?.flush) {
+			let tempSender: vscode.TelemetrySender | undefined = this._sender;
+			this._sender = undefined;
+			Promise.resolve(tempSender.flush!()).then(tempSender = undefined);
+			this._apiObject = undefined;
+		} else {
+			this._sender = undefined;
 		}
 	}
 }
