@@ -699,32 +699,32 @@ export class ExtHostWorkspace implements ExtHostWorkspaceShape, IExtHostWorkspac
 
 	// --- canonical uri identity ---
 
-	private readonly _canonicalUriIdentityProviders = new Map<string, vscode.CanonicalUriIdentityProvider>();
+	private readonly _canonicalUriProviders = new Map<string, vscode.CanonicalUriProvider>();
 
 	// called by ext host
-	registerCanonicalUriIdentityProvider(scheme: string, provider: vscode.CanonicalUriIdentityProvider) {
-		if (this._canonicalUriIdentityProviders.has(scheme)) {
+	registerCanonicalUriProvider(scheme: string, provider: vscode.CanonicalUriProvider) {
+		if (this._canonicalUriProviders.has(scheme)) {
 			throw new Error(`A provider has already been registered for scheme ${scheme}`);
 		}
 
-		this._canonicalUriIdentityProviders.set(scheme, provider);
+		this._canonicalUriProviders.set(scheme, provider);
 		const outgoingScheme = this._uriTransformerService.transformOutgoingScheme(scheme);
 		const handle = this._providerHandlePool++;
-		this._proxy.$registerCanonicalUriIdentityProvider(handle, outgoingScheme);
+		this._proxy.$registerCanonicalUriProvider(handle, outgoingScheme);
 
 		return toDisposable(() => {
-			this._canonicalUriIdentityProviders.delete(scheme);
-			this._proxy.$unregisterCanonicalUriIdentityProvider(handle);
+			this._canonicalUriProviders.delete(scheme);
+			this._proxy.$unregisterCanonicalUriProvider(handle);
 		});
 	}
 
-	async provideCanonicalUriIdentity(uri: URI, cancellationToken: CancellationToken): Promise<URI | undefined> {
-		const provider = this._canonicalUriIdentityProviders.get(uri.scheme);
+	async provideCanonicalUri(uri: URI, options: vscode.CanonicalUriRequestOptions, cancellationToken: CancellationToken): Promise<URI | undefined> {
+		const provider = this._canonicalUriProviders.get(uri.scheme);
 		if (!provider) {
 			return undefined;
 		}
 
-		const result = await provider.provideCanonicalUriIdentity?.(URI.revive(uri), cancellationToken);
+		const result = await provider.provideCanonicalUri?.(URI.revive(uri), options, cancellationToken);
 		if (!result) {
 			return undefined;
 		}
@@ -733,8 +733,8 @@ export class ExtHostWorkspace implements ExtHostWorkspaceShape, IExtHostWorkspac
 	}
 
 	// called by main thread
-	async $provideCanonicalUriIdentity(uri: UriComponents, cancellationToken: CancellationToken): Promise<UriComponents | undefined> {
-		return this.provideCanonicalUriIdentity(URI.revive(uri), cancellationToken);
+	async $provideCanonicalUri(uri: UriComponents, targetScheme: string, cancellationToken: CancellationToken): Promise<UriComponents | undefined> {
+		return this.provideCanonicalUri(URI.revive(uri), { targetScheme }, cancellationToken);
 	}
 }
 
