@@ -32,14 +32,13 @@ export interface IConsolePatchFn {
 	(mainThreadConsole: MainThreadConsoleShape): any;
 }
 
-abstract class ErrorHandler {
-
-	static {
-		// increase number of stack frames (from 10, https://github.com/v8/v8/wiki/Stack-Trace-API)
-		Error.stackTraceLimit = 100;
-	}
+export abstract class ErrorHandler {
 
 	static async installEarlyHandler(accessor: ServicesAccessor): Promise<void> {
+
+		// increase number of stack frames (from 10, https://github.com/v8/v8/wiki/Stack-Trace-API)
+		Error.stackTraceLimit = 100;
+
 		// does NOT dependent of extension information, can be installed immediately, and simply forwards
 		// to the log service and main thread errors
 		const logService = accessor.get(ILogService);
@@ -65,12 +64,15 @@ abstract class ErrorHandler {
 		const mainThreadErrors = rpcService.getProxy(MainContext.MainThreadErrors);
 
 		const map = await extensionService.getExtensionPathIndex();
-		const extensionErrors = new WeakMap<Error, ExtensionIdentifier | undefined>();
+		const extensionErrors = new WeakMap<Error, { extensionIdentifier: ExtensionIdentifier | undefined; stack: string }>();
 
 		// PART 1
 		// set the prepareStackTrace-handle and use it as a side-effect to associate errors
 		// with extensions - this works by looking up callsites in the extension path index
 		function prepareStackTraceAndFindExtension(error: Error, stackTrace: errors.V8CallSite[]) {
+			if (extensionErrors.has(error)) {
+				return extensionErrors.get(error)!.stack;
+			}
 			let stackTraceMessage = '';
 			let extension: IExtensionDescription | undefined;
 			let fileName: string | null;
@@ -81,21 +83,31 @@ abstract class ErrorHandler {
 					extension = map.findSubstr(URI.file(fileName));
 				}
 			}
-			extensionErrors.set(error, extension?.identifier);
-			return `${error.name || 'Error'}: ${error.message || ''}${stackTraceMessage}`;
+			const result = `${error.name || 'Error'}: ${error.message || ''}${stackTraceMessage}`;
+			extensionErrors.set(error, { extensionIdentifier: extension?.identifier, stack: result });
+			return result;
 		}
 
+		const _wasWrapped = Symbol('prepareStackTrace wrapped');
 		let _prepareStackTrace = prepareStackTraceAndFindExtension;
+
 		Object.defineProperty(Error, 'prepareStackTrace', {
 			configurable: false,
 			get() {
 				return _prepareStackTrace;
 			},
 			set(v) {
+				if (v === prepareStackTraceAndFindExtension || v[_wasWrapped]) {
+					_prepareStackTrace = v;
+					return;
+				}
+
 				_prepareStackTrace = function (error, stackTrace) {
 					prepareStackTraceAndFindExtension(error, stackTrace);
 					return v.call(Error, error, stackTrace);
 				};
+
+				Object.assign(_prepareStackTrace, { [_wasWrapped]: true });
 			},
 		});
 
@@ -106,16 +118,16 @@ abstract class ErrorHandler {
 		errors.setUnexpectedErrorHandler(err => {
 			logService.error(err);
 
-			const data = errors.transformErrorForSerialization(err);
-			const extension = extensionErrors.get(err);
-			if (!extension) {
-				mainThreadErrors.$onUnexpectedError(data);
+			const errorData = errors.transformErrorForSerialization(err);
+			const stackData = extensionErrors.get(err);
+			if (!stackData?.extensionIdentifier) {
+				mainThreadErrors.$onUnexpectedError(errorData);
 				return;
 			}
 
-			mainThreadExtensions.$onExtensionRuntimeError(extension, data);
-			const reported = extensionTelemetry.onExtensionError(extension, err);
-			logService.trace('forwarded error to extension?', reported, extension);
+			mainThreadExtensions.$onExtensionRuntimeError(stackData.extensionIdentifier, errorData);
+			const reported = extensionTelemetry.onExtensionError(stackData.extensionIdentifier, err);
+			logService.trace('forwarded error to extension?', reported, stackData);
 		});
 	}
 }
