@@ -656,7 +656,7 @@ export class CompositeBar extends Widget implements ICompositeBar {
 
 		const event = new StandardMouseEvent(e);
 		this.contextMenuService.showContextMenu({
-			getAnchor: () => { return { x: event.posx, y: event.posy }; },
+			getAnchor: () => event,
 			getActions: () => this.getContextMenuActions(e)
 		});
 	}
