@@ -21,6 +21,7 @@ import { ProgressBar } from 'vs/base/browser/ui/progressbar/progressbar';
 import { Severity } from 'vs/platform/notification/common/notification';
 import { isNonEmptyArray } from 'vs/base/common/arrays';
 import { Codicon } from 'vs/base/common/codicons';
+import { ThemeIcon } from 'vs/base/common/themables';
 import { DropdownMenuActionViewItem } from 'vs/base/browser/ui/dropdown/dropdownActionViewItem';
 import { DomEmitter } from 'vs/base/browser/event';
 import { Gesture, EventType as GestureEventType } from 'vs/base/browser/touch';
@@ -82,7 +83,10 @@ export class NotificationsListDelegate implements IListVirtualDelegate<INotifica
 	private computePreferredHeight(notification: INotificationViewItem): number {
 
 		// Prepare offset helper depending on toolbar actions count
-		let actions = 1; // close
+		let actions = 0;
+		if (!notification.hasProgress) {
+			actions++; // close
+		}
 		if (notification.canCollapse) {
 			actions++; // expand/collapse
 		}
@@ -131,8 +135,9 @@ export interface INotificationTemplateData {
 }
 
 interface IMessageActionHandler {
+	readonly toDispose: DisposableStore;
+
 	callback: (href: string) => void;
-	toDispose: DisposableStore;
 }
 
 class NotificationMessageRenderer {
@@ -380,10 +385,10 @@ export class NotificationTemplateRenderer extends Disposable {
 		// first remove, then set as the codicon class names overlap
 		NotificationTemplateRenderer.SEVERITIES.forEach(severity => {
 			if (notification.severity !== severity) {
-				this.template.icon.classList.remove(...this.toSeverityIcon(severity).classNamesArray);
+				this.template.icon.classList.remove(...ThemeIcon.asClassNameArray(this.toSeverityIcon(severity)));
 			}
 		});
-		this.template.icon.classList.add(...this.toSeverityIcon(notification.severity).classNamesArray);
+		this.template.icon.classList.add(...ThemeIcon.asClassNameArray(this.toSeverityIcon(notification.severity)));
 	}
 
 	private renderMessage(notification: INotificationViewItem): boolean {
@@ -536,7 +541,7 @@ export class NotificationTemplateRenderer extends Disposable {
 		}
 	}
 
-	private toSeverityIcon(severity: Severity): Codicon {
+	private toSeverityIcon(severity: Severity): ThemeIcon {
 		switch (severity) {
 			case Severity.Warning:
 				return Codicon.warning;
