@@ -101,8 +101,8 @@ export class Workbench extends Layout {
 		}
 		type AnnotatedError = AnnotatedLoadingError | AnnotatedFactoryError | AnnotatedValidationError;
 
-		if (typeof (<any>window).require.config === 'function') {
-			(<any>window).require.config({
+		if (typeof window.require?.config === 'function') {
+			window.require.config({
 				onError: (err: AnnotatedError) => {
 					if (err.phase === 'loading') {
 						onUnexpectedError(new Error(localize('loaderErrorNative', "Failed to load a required file. Please restart the application to try again. Details: {0}", JSON.stringify(err))));
