@@ -6,7 +6,7 @@
 import { IInstantiationService, IConstructorSignature, ServicesAccessor, BrandedService } from 'vs/platform/instantiation/common/instantiation';
 import { ILifecycleService, LifecyclePhase } from 'vs/workbench/services/lifecycle/common/lifecycle';
 import { Registry } from 'vs/platform/registry/common/platform';
-import { runWhenIdle, IdleDeadline } from 'vs/base/common/async';
+import { runWhenIdle, IdleDeadline, DeferredPromise } from 'vs/base/common/async';
 import { mark } from 'vs/base/common/performance';
 import { ILogService } from 'vs/platform/log/common/log';
 import { IEnvironmentService } from 'vs/platform/environment/common/environment';
@@ -27,8 +27,14 @@ type IWorkbenchContributionSignature<Service extends BrandedService[]> = new (..
 export interface IWorkbenchContributionsRegistry {
 
 	/**
-	 * Registers a workbench contribution to the platform that will be loaded when the workbench starts and disposed when
-	 * the workbench shuts down.
+	 * Registers a workbench contribution to the platform that will
+	 * be loaded when the workbench starts and disposed when the
+	 * workbench shuts down.
+	 *
+	 * The parameter `phase` controls when the contribution is instantiated.
+	 * Phases `Starting` and `Ready` are synchronous, all other phases are
+	 * delayed until the workbench is idle. Contributions are guaranteed to
+	 * be created in the order of their phases, even when delayed to idle.
 	 *
 	 * @param phase the lifecycle phase when to instantiate the contribution.
 	 */
@@ -47,24 +53,25 @@ class WorkbenchContributionsRegistry implements IWorkbenchContributionsRegistry
 	private logService: ILogService | undefined;
 	private environmentService: IEnvironmentService | undefined;
 
-	private readonly toBeInstantiated = new Map<LifecyclePhase, IConstructorSignature<IWorkbenchContribution>[]>();
+	private readonly contributions = new Map<LifecyclePhase, IConstructorSignature<IWorkbenchContribution>[]>();
+	private readonly pendingRestoredContributions = new DeferredPromise<void>();
 
-	registerWorkbenchContribution(ctor: IConstructorSignature<IWorkbenchContribution>, phase: LifecyclePhase = LifecyclePhase.Starting): void {
+	registerWorkbenchContribution(contribution: IConstructorSignature<IWorkbenchContribution>, phase: LifecyclePhase = LifecyclePhase.Starting): void {
 
 		// Instantiate directly if we are already matching the provided phase
 		if (this.instantiationService && this.lifecycleService && this.logService && this.environmentService && this.lifecycleService.phase >= phase) {
-			this.safeCreateInstance(this.instantiationService, this.logService, this.environmentService, ctor, phase);
+			this.safeCreateContribution(this.instantiationService, this.logService, this.environmentService, contribution, phase);
 		}
 
 		// Otherwise keep contributions by lifecycle phase
 		else {
-			let toBeInstantiated = this.toBeInstantiated.get(phase);
-			if (!toBeInstantiated) {
-				toBeInstantiated = [];
-				this.toBeInstantiated.set(phase, toBeInstantiated);
+			let contributions = this.contributions.get(phase);
+			if (!contributions) {
+				contributions = [];
+				this.contributions.set(phase, contributions);
 			}
 
-			toBeInstantiated.push(ctor as IConstructorSignature<IWorkbenchContribution>);
+			contributions.push(contribution);
 		}
 	}
 
@@ -92,59 +99,92 @@ class WorkbenchContributionsRegistry implements IWorkbenchContributionsRegistry
 		}
 	}
 
-	private doInstantiateByPhase(instantiationService: IInstantiationService, logService: ILogService, environmentService: IEnvironmentService, phase: LifecyclePhase): void {
-		const toBeInstantiated = this.toBeInstantiated.get(phase);
-		if (toBeInstantiated) {
-			this.toBeInstantiated.delete(phase);
-			if (phase !== LifecyclePhase.Eventually) {
+	private async doInstantiateByPhase(instantiationService: IInstantiationService, logService: ILogService, environmentService: IEnvironmentService, phase: LifecyclePhase): Promise<void> {
+		const contributions = this.contributions.get(phase);
+		if (contributions) {
+			this.contributions.delete(phase);
 
-				// instantiate everything synchronously and blocking
-				// measure the time it takes as perf marks for diagnosis
+			switch (phase) {
+				case LifecyclePhase.Starting:
+				case LifecyclePhase.Ready: {
 
-				mark(`code/willCreateWorkbenchContributions/${phase}`);
+					// instantiate everything synchronously and blocking
+					// measure the time it takes as perf marks for diagnosis
 
-				for (const ctor of toBeInstantiated) {
-					this.safeCreateInstance(instantiationService, logService, environmentService, ctor, phase); // catch error so that other contributions are still considered
+					mark(`code/willCreateWorkbenchContributions/${phase}`);
+
+					for (const contribution of contributions) {
+						this.safeCreateContribution(instantiationService, logService, environmentService, contribution, phase);
+					}
+
+					mark(`code/didCreateWorkbenchContributions/${phase}`);
+
+					break;
 				}
 
-				mark(`code/didCreateWorkbenchContributions/${phase}`);
-			} else {
+				case LifecyclePhase.Restored:
+				case LifecyclePhase.Eventually: {
 
-				// for the Eventually-phase we instantiate contributions
-				// only when idle. this might take a few idle-busy-cycles
-				// but will finish within the timeouts
+					// for the Restored/Eventually-phase we instantiate contributions
+					// only when idle. this might take a few idle-busy-cycles but will
+					// finish within the timeouts
+					// given that, we must ensure to await the contributions from the
+					// Restored-phase before we instantiate the Eventually-phase
 
-				const forcedTimeout = 3000;
-				let i = 0;
-				const instantiateSome = (idle: IdleDeadline) => {
-					while (i < toBeInstantiated.length) {
-						const ctor = toBeInstantiated[i++];
-						this.safeCreateInstance(instantiationService, logService, environmentService, ctor, phase); // catch error so that other contributions are still considered
-						if (idle.timeRemaining() < 1) {
-							// time is up -> reschedule
-							runWhenIdle(instantiateSome, forcedTimeout);
-							break;
-						}
+					if (phase === LifecyclePhase.Eventually) {
+						await this.pendingRestoredContributions.p;
 					}
-				};
-				runWhenIdle(instantiateSome, forcedTimeout);
+
+					this.doInstantiateWhenIdle(contributions, instantiationService, logService, environmentService, phase);
+
+					break;
+				}
 			}
 		}
 	}
 
-	private safeCreateInstance(instantiationService: IInstantiationService, logService: ILogService, environmentService: IEnvironmentService, ctor: IConstructorSignature<IWorkbenchContribution>, phase: LifecyclePhase): void {
+	private doInstantiateWhenIdle(contributions: IConstructorSignature<IWorkbenchContribution>[], instantiationService: IInstantiationService, logService: ILogService, environmentService: IEnvironmentService, phase: LifecyclePhase): void {
+		mark(`code/willCreateWorkbenchContributions/${phase}`);
+
+		let i = 0;
+		const forcedTimeout = phase === LifecyclePhase.Eventually ? 3000 : 500;
+
+		const instantiateSome = (idle: IdleDeadline) => {
+			while (i < contributions.length) {
+				const contribution = contributions[i++];
+				this.safeCreateContribution(instantiationService, logService, environmentService, contribution, phase);
+				if (idle.timeRemaining() < 1) {
+					// time is up -> reschedule
+					runWhenIdle(instantiateSome, forcedTimeout);
+					break;
+				}
+			}
+
+			if (i === contributions.length) {
+				mark(`code/didCreateWorkbenchContributions/${phase}`);
+
+				if (phase === LifecyclePhase.Restored) {
+					this.pendingRestoredContributions.complete();
+				}
+			}
+		};
+
+		runWhenIdle(instantiateSome, forcedTimeout);
+	}
+
+	private safeCreateContribution(instantiationService: IInstantiationService, logService: ILogService, environmentService: IEnvironmentService, contribution: IConstructorSignature<IWorkbenchContribution>, phase: LifecyclePhase): void {
 		const now: number | undefined = phase < LifecyclePhase.Restored ? Date.now() : undefined;
 
 		try {
-			instantiationService.createInstance(ctor);
+			instantiationService.createInstance(contribution);
 		} catch (error) {
-			logService.error(`Unable to instantiate workbench contribution ${ctor.name}.`, error);
+			logService.error(`Unable to create workbench contribution ${contribution.name}.`, error);
 		}
 
 		if (typeof now === 'number' && !environmentService.isBuilt /* only log out of sources where we have good ctor names */) {
 			const time = Date.now() - now;
 			if (time > 20) {
-				logService.warn(`Workbench contribution ${ctor.name} blocked restore phase by ${time}ms.`);
+				logService.warn(`Workbench contribution ${contribution.name} blocked restore phase by ${time}ms.`);
 			}
 		}
 	}
