@@ -277,12 +277,31 @@ export class BulkEditService implements IBulkEditService {
 	}
 
 	private async _shouldVeto(label: string | undefined, reason: ShutdownReason): Promise<boolean> {
-		label = label || localize('fileOperation', "File operation");
-		const reasonLabel = reason === ShutdownReason.CLOSE ? localize('closeTheWindow', "Close Window") : reason === ShutdownReason.LOAD ? localize('changeWorkspace', "Change Workspace") :
-			reason === ShutdownReason.RELOAD ? localize('reloadTheWindow', "Reload Window") : localize('quit', "Quit");
+		let message: string;
+		let primaryButton: string;
+		switch (reason) {
+			case ShutdownReason.CLOSE:
+				message = localize('closeTheWindow.message', "Are you sure you want to close the window?");
+				primaryButton = localize({ key: 'closeTheWindow', comment: ['&& denotes a mnemonic'] }, "&&Close Window");
+				break;
+			case ShutdownReason.LOAD:
+				message = localize('changeWorkspace.message', "Are you sure you want to change the workspace?");
+				primaryButton = localize({ key: 'changeWorkspace', comment: ['&& denotes a mnemonic'] }, "Change &&Workspace");
+				break;
+			case ShutdownReason.RELOAD:
+				message = localize('reloadTheWindow.message', "Are you sure you want to reload the window?");
+				primaryButton = localize({ key: 'reloadTheWindow', comment: ['&& denotes a mnemonic'] }, "&&Reload Window");
+				break;
+			default:
+				message = localize('quit.message', "Are you sure you want to quit?");
+				primaryButton = localize({ key: 'quit', comment: ['&& denotes a mnemonic'] }, "&&Quit");
+				break;
+		}
+
 		const result = await this._dialogService.confirm({
-			message: localize('areYouSureQuiteBulkEdit', "Are you sure you want to {0}? '{1}' is in progress.", reasonLabel.toLowerCase(), label),
-			primaryButton: reasonLabel
+			message,
+			detail: localize('areYouSureQuiteBulkEdit.detail', "'{0}' is in progress.", label || localize('fileOperation', "File operation")),
+			primaryButton
 		});
 
 		return !result.confirmed;
