@@ -9,49 +9,54 @@ import { localize } from 'vs/nls';
 import { Action2, MenuId, registerAction2 } from 'vs/platform/actions/common/actions';
 import { CHAT_CATEGORY } from 'vs/workbench/contrib/chat/browser/actions/chatActions';
 import { IChatWidget } from 'vs/workbench/contrib/chat/browser/chat';
-import { CONTEXT_INTERACTIVE_INPUT_HAS_TEXT, CONTEXT_INTERACTIVE_REQUEST_IN_PROGRESS } from 'vs/workbench/contrib/chat/common/chatContextKeys';
+import { CONTEXT_CHAT_INPUT_HAS_TEXT, CONTEXT_CHAT_REQUEST_IN_PROGRESS } from 'vs/workbench/contrib/chat/common/chatContextKeys';
 import { IChatService } from 'vs/workbench/contrib/chat/common/chatService';
 
 export interface IChatExecuteActionContext {
 	widget: IChatWidget;
+	inputValue?: string;
 }
 
 function isExecuteActionContext(thing: unknown): thing is IChatExecuteActionContext {
 	return typeof thing === 'object' && thing !== null && 'widget' in thing;
 }
 
-export function registerChatExecuteActions() {
-	registerAction2(class SubmitAction extends Action2 {
-		constructor() {
-			super({
-				id: 'workbench.action.chat.submit',
-				title: {
-					value: localize('interactive.submit.label', "Submit"),
-					original: 'Submit'
-				},
-				f1: false,
-				category: CHAT_CATEGORY,
-				icon: Codicon.send,
-				precondition: CONTEXT_INTERACTIVE_INPUT_HAS_TEXT,
-				menu: {
-					id: MenuId.ChatExecute,
-					when: CONTEXT_INTERACTIVE_REQUEST_IN_PROGRESS.negate(),
-					group: 'navigation',
-				}
-			});
-		}
+export class SubmitAction extends Action2 {
+	static readonly ID = 'workbench.action.chat.submit';
 
-		run(accessor: ServicesAccessor, ...args: any[]) {
-			const context = args[0];
-			if (!isExecuteActionContext(context)) {
-				return;
+	constructor() {
+		super({
+			id: SubmitAction.ID,
+			title: {
+				value: localize('interactive.submit.label', "Submit"),
+				original: 'Submit'
+			},
+			f1: false,
+			category: CHAT_CATEGORY,
+			icon: Codicon.send,
+			precondition: CONTEXT_CHAT_INPUT_HAS_TEXT,
+			menu: {
+				id: MenuId.ChatExecute,
+				when: CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate(),
+				group: 'navigation',
 			}
+		});
+	}
 
-			context.widget.acceptInput();
+	run(accessor: ServicesAccessor, ...args: any[]) {
+		const context = args[0];
+		if (!isExecuteActionContext(context)) {
+			return;
 		}
-	});
 
-	registerAction2(class SubmitAction extends Action2 {
+		context.widget.acceptInput(context.inputValue);
+	}
+}
+
+export function registerChatExecuteActions() {
+	registerAction2(SubmitAction);
+
+	registerAction2(class CancelAction extends Action2 {
 		constructor() {
 			super({
 				id: 'workbench.action.chat.cancel',
@@ -64,7 +69,7 @@ export function registerChatExecuteActions() {
 				icon: Codicon.debugStop,
 				menu: {
 					id: MenuId.ChatExecute,
-					when: CONTEXT_INTERACTIVE_REQUEST_IN_PROGRESS,
+					when: CONTEXT_CHAT_REQUEST_IN_PROGRESS,
 					group: 'navigation',
 				}
 			});
