@@ -22,7 +22,7 @@ import { registerChatExecuteActions } from 'vs/workbench/contrib/chat/browser/ac
 import { registerChatQuickQuestionActions } from 'vs/workbench/contrib/chat/browser/actions/chatQuickInputActions';
 import { registerChatTitleActions } from 'vs/workbench/contrib/chat/browser/actions/chatTitleActions';
 import { registerChatExportActions } from 'vs/workbench/contrib/chat/browser/actions/chatImportExport';
-import { IChatWidgetService } from 'vs/workbench/contrib/chat/browser/chat';
+import { IChatAccessibilityService, IChatWidget, IChatWidgetService } from 'vs/workbench/contrib/chat/browser/chat';
 import { ChatContributionService } from 'vs/workbench/contrib/chat/browser/chatContributionServiceImpl';
 import { ChatEditor, IChatEditorOptions } from 'vs/workbench/contrib/chat/browser/chatEditor';
 import { ChatEditorInput, ChatEditorInputSerializer } from 'vs/workbench/contrib/chat/browser/chatEditorInput';
@@ -35,6 +35,15 @@ import { ChatWidgetHistoryService, IChatWidgetHistoryService } from 'vs/workbenc
 import { IEditorResolverService, RegisteredEditorPriority } from 'vs/workbench/services/editor/common/editorResolverService';
 import { LifecyclePhase } from 'vs/workbench/services/lifecycle/common/lifecycle';
 import '../common/chatColors';
+import { registerMoveActions } from 'vs/workbench/contrib/chat/browser/actions/chatMoveActions';
+import { registerClearActions } from 'vs/workbench/contrib/chat/browser/actions/chatClearActions';
+import { AccessibilityVerbositySettingId, AccessibleViewAction } from 'vs/workbench/contrib/accessibility/browser/accessibilityContribution';
+import { AccessibleViewType, IAccessibleViewService } from 'vs/workbench/contrib/accessibility/browser/accessibleView';
+import { isResponseVM } from 'vs/workbench/contrib/chat/common/chatViewModel';
+import { CONTEXT_IN_CHAT_SESSION } from 'vs/workbench/contrib/chat/common/chatContextKeys';
+import { ChatAccessibilityService } from 'vs/workbench/contrib/chat/browser/chatAccessibilityService';
+import { ICodeEditorService } from 'vs/editor/browser/services/codeEditorService';
+import { QuickQuestionMode } from 'vs/workbench/contrib/chat/browser/actions/quickQuestionActions/quickQuestionAction';
 
 // Register configuration
 const configurationRegistry = Registry.as<IConfigurationRegistry>(ConfigurationExtensions.Configuration);
@@ -45,35 +54,48 @@ configurationRegistry.registerConfiguration({
 	properties: {
 		'chat.editor.fontSize': {
 			type: 'number',
-			description: nls.localize('interactiveSession.editor.fontSize', "Controls the font size in pixels in Interactive Sessions."),
+			description: nls.localize('interactiveSession.editor.fontSize', "Controls the font size in pixels in chat codeblocks."),
 			default: isMacintosh ? 12 : 14,
 		},
 		'chat.editor.fontFamily': {
 			type: 'string',
-			description: nls.localize('interactiveSession.editor.fontFamily', "Controls the font family in Interactive Sessions."),
+			description: nls.localize('interactiveSession.editor.fontFamily', "Controls the font family in chat codeblocks."),
 			default: 'default'
 		},
 		'chat.editor.fontWeight': {
 			type: 'string',
-			description: nls.localize('interactiveSession.editor.fontWeight', "Controls the font weight in Interactive Sessions."),
+			description: nls.localize('interactiveSession.editor.fontWeight', "Controls the font weight in chat codeblocks."),
 			default: 'default'
 		},
 		'chat.editor.wordWrap': {
 			type: 'string',
-			description: nls.localize('interactiveSession.editor.wordWrap', "Controls whether lines should wrap in Interactive Sessions."),
+			description: nls.localize('interactiveSession.editor.wordWrap', "Controls whether lines should wrap in chat codeblocks."),
 			default: 'off',
 			enum: ['on', 'off']
 		},
 		'chat.editor.lineHeight': {
 			type: 'number',
-			description: nls.localize('interactiveSession.editor.lineHeight', "Controls the line height in pixels in Interactive Sessions. Use 0 to compute the line height from the font size."),
+			description: nls.localize('interactiveSession.editor.lineHeight', "Controls the line height in pixels in chat codeblocks. Use 0 to compute the line height from the font size."),
 			default: 0
 		},
-		'chat.experimental.quickQuestion.enable': {
-			type: 'boolean',
-			description: nls.localize('interactiveSession.experimental.quickQuestion.enable', "Controls whether the quick question feature is enabled."),
-			default: false,
-			tags: ['experimental']
+		'chat.experimental.defaultMode': {
+			type: 'string',
+			tags: ['experimental'],
+			enum: ['chatView', 'quickQuestion', 'both'],
+			enumDescriptions: [
+				nls.localize('interactiveSession.defaultMode.chatView', "Use the chat view as the default mode. Displays the chat icon in the Activity Bar."),
+				nls.localize('interactiveSession.defaultMode.quickQuestion', "Use the quick question as the default mode. Displays the chat icon in the Title Bar."),
+				nls.localize('interactiveSession.defaultMode.both', "Displays the chat icon in the Activity Bar and the Title Bar which open their respective chat modes.")
+			],
+			description: nls.localize('interactiveSession.defaultMode', "Controls the default mode of the chat experience."),
+			default: 'chatView'
+		},
+		'chat.experimental.quickQuestion.mode': {
+			type: 'string',
+			tags: ['experimental'],
+			enum: [QuickQuestionMode.SingleQuestion, QuickQuestionMode.InputOnTopChat, QuickQuestionMode.InputOnBottomChat],
+			description: nls.localize('interactiveSession.quickQuestion.mode', "Controls the mode of quick question chat experience."),
+			default: QuickQuestionMode.InputOnTopChat,
 		}
 	}
 });
@@ -98,7 +120,7 @@ class ChatResolverContribution extends Disposable {
 		super();
 
 		this._register(editorResolverService.registerEditor(
-			`${Schemas.vscodeInteractiveSesssion}:**/**`,
+			`${Schemas.vscodeChatSesssion}:**/**`,
 			{
 				id: ChatEditorInput.EditorID,
 				label: nls.localize('chat', "Chat"),
@@ -106,7 +128,7 @@ class ChatResolverContribution extends Disposable {
 			},
 			{
 				singlePerResource: true,
-				canSupportResource: resource => resource.scheme === Schemas.vscodeInteractiveSesssion
+				canSupportResource: resource => resource.scheme === Schemas.vscodeChatSesssion
 			},
 			{
 				createEditorInput: ({ resource, options }) => {
@@ -117,8 +139,73 @@ class ChatResolverContribution extends Disposable {
 	}
 }
 
+class ChatAccessibleViewContribution extends Disposable {
+	static ID: 'chatAccessibleViewContribution';
+	constructor() {
+		super();
+		this._register(AccessibleViewAction.addImplementation(100, 'panelChat', accessor => {
+			const accessibleViewService = accessor.get(IAccessibleViewService);
+			const widgetService = accessor.get(IChatWidgetService);
+			const codeEditorService = accessor.get(ICodeEditorService);
+			return renderAccessibleView(accessibleViewService, widgetService, codeEditorService, true);
+			function renderAccessibleView(accessibleViewService: IAccessibleViewService, widgetService: IChatWidgetService, codeEditorService: ICodeEditorService, initialRender?: boolean): boolean {
+				const widget = widgetService.lastFocusedWidget;
+				if (!widget) {
+					return false;
+				}
+				const chatInputFocused = initialRender && !!codeEditorService.getFocusedCodeEditor();
+				if (initialRender && chatInputFocused) {
+					widget.focusLastMessage();
+				}
+
+				if (!widget) {
+					return false;
+				}
+
+				const verifiedWidget: IChatWidget = widget;
+				const focusedItem = verifiedWidget.getFocus();
+
+				if (!focusedItem) {
+					return false;
+				}
+
+				widget.focus(focusedItem);
+
+				const responseContent = isResponseVM(focusedItem) ? focusedItem.response.value : undefined;
+				if (!responseContent) {
+					return false;
+				}
+
+				accessibleViewService.show({
+					verbositySettingKey: AccessibilityVerbositySettingId.Chat,
+					provideContent(): string { return responseContent; },
+					onClose() {
+						verifiedWidget.reveal(focusedItem);
+						if (chatInputFocused) {
+							verifiedWidget.focusInput();
+						} else {
+							verifiedWidget.focus(focusedItem);
+						}
+					},
+					next() {
+						verifiedWidget.moveFocus(focusedItem, 'next');
+						renderAccessibleView(accessibleViewService, widgetService, codeEditorService);
+					},
+					previous() {
+						verifiedWidget.moveFocus(focusedItem, 'previous');
+						renderAccessibleView(accessibleViewService, widgetService, codeEditorService);
+					},
+					options: { ariaLabel: nls.localize('chatAccessibleView', "Chat Accessible View"), type: AccessibleViewType.View }
+				});
+				return true;
+			}
+		}, CONTEXT_IN_CHAT_SESSION));
+	}
+}
+
 const workbenchContributionsRegistry = Registry.as<IWorkbenchContributionsRegistry>(WorkbenchExtensions.Workbench);
 workbenchContributionsRegistry.registerWorkbenchContribution(ChatResolverContribution, LifecyclePhase.Starting);
+workbenchContributionsRegistry.registerWorkbenchContribution(ChatAccessibleViewContribution, LifecyclePhase.Eventually);
 Registry.as<IEditorFactoryRegistry>(EditorExtensions.EditorFactory).registerEditorSerializer(ChatEditorInput.TypeID, ChatEditorInputSerializer);
 
 registerChatActions();
@@ -128,9 +215,12 @@ registerChatTitleActions();
 registerChatExecuteActions();
 registerChatQuickQuestionActions();
 registerChatExportActions();
+registerMoveActions();
+registerClearActions();
 
 registerSingleton(IChatService, ChatService, InstantiationType.Delayed);
 registerSingleton(IChatContributionService, ChatContributionService, InstantiationType.Delayed);
 registerSingleton(IChatWidgetService, ChatWidgetService, InstantiationType.Delayed);
+registerSingleton(IChatAccessibilityService, ChatAccessibilityService, InstantiationType.Delayed);
 registerSingleton(IChatWidgetHistoryService, ChatWidgetHistoryService, InstantiationType.Delayed);
 
