@@ -5,14 +5,15 @@
 
 import * as dom from 'vs/base/browser/dom';
 import { ITreeContextMenuEvent, ITreeElement } from 'vs/base/browser/ui/tree/tree';
+import { disposableTimeout } from 'vs/base/common/async';
 import { CancellationToken } from 'vs/base/common/cancellation';
 import { Emitter } from 'vs/base/common/event';
 import { Disposable, DisposableStore, IDisposable, combinedDisposable, toDisposable } from 'vs/base/common/lifecycle';
 import { isEqual } from 'vs/base/common/resources';
+import { withNullAsUndefined } from 'vs/base/common/types';
 import { URI } from 'vs/base/common/uri';
 import 'vs/css!./media/chat';
 import { ICodeEditor } from 'vs/editor/browser/editorBrowser';
-import { EditorOption } from 'vs/editor/common/config/editorOptions';
 import { localize } from 'vs/nls';
 import { MenuId } from 'vs/platform/actions/common/actions';
 import { IContextKey, IContextKeyService } from 'vs/platform/contextkey/common/contextkey';
@@ -22,16 +23,16 @@ import { ServiceCollection } from 'vs/platform/instantiation/common/serviceColle
 import { WorkbenchObjectTree } from 'vs/platform/list/browser/listService';
 import { IViewsService } from 'vs/workbench/common/views';
 import { clearChatSession } from 'vs/workbench/contrib/chat/browser/actions/chatClear';
-import { IChatCodeBlockInfo, IChatWidget, IChatWidgetService, IChatWidgetViewContext } from 'vs/workbench/contrib/chat/browser/chat';
+import { ChatTreeItem, IChatAccessibilityService, IChatCodeBlockInfo, IChatWidget, IChatWidgetService, IChatWidgetViewContext } from 'vs/workbench/contrib/chat/browser/chat';
 import { ChatInputPart } from 'vs/workbench/contrib/chat/browser/chatInputPart';
-import { IChatRendererDelegate, InteractiveListItemRenderer, ChatAccessibilityProvider, ChatListDelegate, InteractiveTreeItem } from 'vs/workbench/contrib/chat/browser/chatListRenderer';
+import { ChatAccessibilityProvider, ChatListDelegate, ChatListItemRenderer, IChatRendererDelegate } from 'vs/workbench/contrib/chat/browser/chatListRenderer';
 import { ChatEditorOptions } from 'vs/workbench/contrib/chat/browser/chatOptions';
 import { ChatViewPane } from 'vs/workbench/contrib/chat/browser/chatViewPane';
-import { CONTEXT_INTERACTIVE_REQUEST_IN_PROGRESS, CONTEXT_IN_INTERACTIVE_SESSION } from 'vs/workbench/contrib/chat/common/chatContextKeys';
+import { CONTEXT_CHAT_REQUEST_IN_PROGRESS, CONTEXT_IN_CHAT_SESSION } from 'vs/workbench/contrib/chat/common/chatContextKeys';
 import { IChatContributionService } from 'vs/workbench/contrib/chat/common/chatContributionService';
 import { IChatModel } from 'vs/workbench/contrib/chat/common/chatModel';
 import { IChatReplyFollowup, IChatService, ISlashCommand } from 'vs/workbench/contrib/chat/common/chatService';
-import { IInteractiveResponseViewModel, ChatViewModel, isRequestVM, isResponseVM, isWelcomeVM } from 'vs/workbench/contrib/chat/common/chatViewModel';
+import { ChatViewModel, IChatResponseViewModel, isRequestVM, isResponseVM, isWelcomeVM } from 'vs/workbench/contrib/chat/common/chatViewModel';
 
 const $ = dom.$;
 
@@ -60,8 +61,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	private _onDidChangeViewModel = this._register(new Emitter<void>());
 	readonly onDidChangeViewModel = this._onDidChangeViewModel.event;
 
-	private tree!: WorkbenchObjectTree<InteractiveTreeItem>;
-	private renderer!: InteractiveListItemRenderer;
+	private tree!: WorkbenchObjectTree<ChatTreeItem>;
+	private renderer!: ChatListItemRenderer;
 
 	private inputPart!: ChatInputPart;
 	private editorOptions!: ChatEditorOptions;
@@ -76,7 +77,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 
 	private previousTreeScrollHeight: number = 0;
 
-	private viewModelDisposables = new DisposableStore();
+	private viewModelDisposables = this._register(new DisposableStore());
 	private _viewModel: ChatViewModel | undefined;
 	private set viewModel(viewModel: ChatViewModel | undefined) {
 		if (this._viewModel === viewModel) {
@@ -92,8 +93,11 @@ export class ChatWidget extends Disposable implements IChatWidget {
 
 		this.slashCommandsPromise = undefined;
 		this.lastSlashCommands = undefined;
+
 		this.getSlashCommands().then(() => {
-			this.onDidChangeItems();
+			if (!this._isDisposed) {
+				this.onDidChangeItems();
+			}
 		});
 
 		this._onDidChangeViewModel.fire();
@@ -114,10 +118,12 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		@IChatService private readonly chatService: IChatService,
 		@IChatWidgetService chatWidgetService: IChatWidgetService,
 		@IContextMenuService private readonly contextMenuService: IContextMenuService,
+		@IChatAccessibilityService private readonly _chatAccessibilityService: IChatAccessibilityService,
+		@IInstantiationService private readonly _instantiationService: IInstantiationService
 	) {
 		super();
-		CONTEXT_IN_INTERACTIVE_SESSION.bindTo(contextKeyService).set(true);
-		this.requestInProgress = CONTEXT_INTERACTIVE_REQUEST_IN_PROGRESS.bindTo(contextKeyService);
+		CONTEXT_IN_CHAT_SESSION.bindTo(contextKeyService).set(true);
+		this.requestInProgress = CONTEXT_CHAT_REQUEST_IN_PROGRESS.bindTo(contextKeyService);
 
 		this._register((chatWidgetService as ChatWidgetService).register(this));
 	}
@@ -134,14 +140,27 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		return this.inputPart.inputUri;
 	}
 
-	render(parent: HTMLElement): void {
-		this.container = dom.append(parent, $('.interactive-session'));
-		this.listContainer = dom.append(this.container, $(`.interactive-list`));
+	private _isDisposed: boolean = false;
+	public override dispose(): void {
+		this._isDisposed = true;
+		super.dispose();
+	}
 
+	render(parent: HTMLElement): void {
 		const viewId = 'viewId' in this.viewContext ? this.viewContext.viewId : undefined;
 		this.editorOptions = this._register(this.instantiationService.createInstance(ChatEditorOptions, viewId, this.styles.listForeground, this.styles.inputEditorBackground, this.styles.resultEditorBackground));
+		const renderInputOnTop = this.viewContext.renderInputOnTop ?? false;
+
+		this.container = dom.append(parent, $('.interactive-session'));
+		if (renderInputOnTop) {
+			this.createInput(this.container, { renderFollowups: false });
+			this.listContainer = dom.append(this.container, $(`.interactive-list`));
+		} else {
+			this.listContainer = dom.append(this.container, $(`.interactive-list`));
+			this.createInput(this.container);
+		}
+
 		this.createList(this.listContainer);
-		this.createInput(this.container);
 
 		this._register(this.editorOptions.onDidChange(() => this.onDidStyleChange()));
 		this.onDidStyleChange();
@@ -159,11 +178,28 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		this.inputPart.focus();
 	}
 
+	moveFocus(item: ChatTreeItem, type: 'next' | 'previous'): void {
+		const items = this.viewModel?.getItems();
+		if (!items) {
+			return;
+		}
+		const responseItems = items.filter(i => isResponseVM(i));
+		const targetIndex = responseItems.indexOf(item);
+		if (targetIndex === undefined) {
+			return;
+		}
+		const indexToFocus = type === 'next' ? targetIndex + 1 : targetIndex - 1;
+		if (indexToFocus < 0 || indexToFocus > responseItems.length - 1) {
+			return;
+		}
+		this.focus(responseItems[indexToFocus]);
+	}
+
 	private onDidChangeItems() {
 		if (this.tree && this.visible) {
 			const treeItems = (this.viewModel?.getItems() ?? [])
 				.map(item => {
-					return <ITreeElement<InteractiveTreeItem>>{
+					return <ITreeElement<ChatTreeItem>>{
 						element: item,
 						collapsed: false,
 						collapsible: false
@@ -174,6 +210,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				diffIdentityProvider: {
 					getId: (element) => {
 						return ((isResponseVM(element) || isRequestVM(element)) ? element.dataId : element.id) +
+							// TODO? We can give the welcome message a proper VM or get rid of the rest of the VMs
+							((isWelcomeVM(element) && !this.viewModel?.isInitialized) ? '_initializing' : '') +
 							// Ensure re-rendering an element once slash commands are loaded, so the colorization can be applied.
 							`${(isRequestVM(element) || isWelcomeVM(element)) && !!this.lastSlashCommands ? '_scLoaded' : ''}` +
 							// If a response is in the process of progressive rendering, we need to ensure that it will
@@ -183,6 +221,10 @@ export class ChatWidget extends Disposable implements IChatWidget {
 				}
 			});
 
+			if (this._dynamicMessageLayoutData) {
+				this.layoutDynamicChatTreeItemMode();
+			}
+
 			const lastItem = treeItems[treeItems.length - 1]?.element;
 			if (lastItem && isResponseVM(lastItem) && lastItem.isComplete) {
 				this.renderFollowups(lastItem.replyFollowups);
@@ -206,13 +248,13 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		this.renderer.setVisible(visible);
 
 		if (visible) {
-			setTimeout(() => {
+			this._register(disposableTimeout(() => {
 				// Progressive rendering paused while hidden, so start it up again.
 				// Do it after a timeout because the container is not visible yet (it should be but offsetHeight returns 0 here)
 				if (this.visible) {
 					this.onDidChangeItems();
 				}
-			}, 0);
+			}, 0));
 		}
 	}
 
@@ -228,6 +270,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 					command: 'clear',
 					sortText: 'z_clear',
 					detail: localize('clear', "Clear the session"),
+					executeImmediately: true
 				};
 				this.lastSlashCommands = [
 					...(commands ?? []),
@@ -247,24 +290,24 @@ export class ChatWidget extends Disposable implements IChatWidget {
 			getListLength: () => this.tree.getNode(null).visibleChildrenCount,
 			getSlashCommands: () => this.lastSlashCommands ?? [],
 		};
-		this.renderer = this._register(scopedInstantiationService.createInstance(InteractiveListItemRenderer, this.editorOptions, rendererDelegate));
+		this.renderer = this._register(scopedInstantiationService.createInstance(ChatListItemRenderer, this.editorOptions, rendererDelegate));
 		this._register(this.renderer.onDidClickFollowup(item => {
 			this.acceptInput(item);
 		}));
 
-		this.tree = <WorkbenchObjectTree<InteractiveTreeItem>>scopedInstantiationService.createInstance(
+		this.tree = <WorkbenchObjectTree<ChatTreeItem>>scopedInstantiationService.createInstance(
 			WorkbenchObjectTree,
 			'Chat',
 			listContainer,
 			delegate,
 			[this.renderer],
 			{
-				identityProvider: { getId: (e: InteractiveTreeItem) => e.id },
+				identityProvider: { getId: (e: ChatTreeItem) => e.id },
 				horizontalScrolling: false,
 				supportDynamicHeights: true,
 				hideTwistiesOfChildlessElements: true,
-				accessibilityProvider: new ChatAccessibilityProvider(),
-				keyboardNavigationLabelProvider: { getKeyboardNavigationLabel: (e: InteractiveTreeItem) => isRequestVM(e) ? e.message : isResponseVM(e) ? e.response.value : '' }, // TODO
+				accessibilityProvider: this._instantiationService.createInstance(ChatAccessibilityProvider),
+				keyboardNavigationLabelProvider: { getKeyboardNavigationLabel: (e: ChatTreeItem) => isRequestVM(e) ? e.message : isResponseVM(e) ? e.response.value : '' }, // TODO
 				setRowLineHeight: false,
 				overrideStyles: {
 					listFocusBackground: this.styles.listBackground,
@@ -295,7 +338,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		}));
 	}
 
-	private onContextMenu(e: ITreeContextMenuEvent<InteractiveTreeItem | null>): void {
+	private onContextMenu(e: ITreeContextMenuEvent<ChatTreeItem | null>): void {
 		e.browserEvent.preventDefault();
 		e.browserEvent.stopPropagation();
 
@@ -324,8 +367,8 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		this.previousTreeScrollHeight = this.tree.scrollHeight;
 	}
 
-	private createInput(container: HTMLElement): void {
-		this.inputPart = this.instantiationService.createInstance(ChatInputPart);
+	private createInput(container: HTMLElement, options?: { renderFollowups: boolean }): void {
+		this.inputPart = this._register(this.instantiationService.createInstance(ChatInputPart, { renderFollowups: options?.renderFollowups ?? true }));
 		this.inputPart.render(container, '', this);
 
 		this._register(this.inputPart.onDidFocus(() => this._onDidFocus.fire()));
@@ -345,10 +388,14 @@ export class ChatWidget extends Disposable implements IChatWidget {
 
 		this.container.setAttribute('data-session-id', model.sessionId);
 		this.viewModel = this.instantiationService.createInstance(ChatViewModel, model);
-		this.viewModelDisposables.add(this.viewModel.onDidChange(() => {
+		this.viewModelDisposables.add(this.viewModel.onDidChange(e => {
 			this.slashCommandsPromise = undefined;
 			this.requestInProgress.set(this.viewModel!.requestInProgress);
 			this.onDidChangeItems();
+			if (e?.kind === 'addRequest') {
+				revealLastElement(this.tree);
+				this.focusInput();
+			}
 		}));
 		this.viewModelDisposables.add(this.viewModel.onDidDisposeModel(() => {
 			// Disposes the viewmodel and listeners
@@ -363,27 +410,58 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		}
 	}
 
+	getFocus(): ChatTreeItem | undefined {
+		return withNullAsUndefined(this.tree.getFocus()[0]);
+	}
+
+	reveal(item: ChatTreeItem): void {
+		this.tree.reveal(item);
+	}
+
+	focus(item: ChatTreeItem): void {
+		const items = this.tree.getNode(null).children;
+		const node = items.find(i => i.element?.id === item.id);
+		if (!node) {
+			return;
+		}
+
+		this.tree.setFocus([node.element]);
+		this.tree.domFocus();
+	}
+
 	async acceptInput(query?: string | IChatReplyFollowup): Promise<void> {
 		if (this.viewModel) {
 			const editorValue = this.inputPart.inputEditor.getValue();
 
 			// Shortcut for /clear command
-			if (!query && editorValue.trim() === '/clear') {
+			if (!query && editorValue.trim() === '/clear' || typeof query === 'string' && query.trim() === '/clear') {
 				// Small hack, if this becomes a repeated pattern, we should have a real internal slash command provider system
 				this.instantiationService.invokeFunction(clearChatSession, this);
 				return;
 			}
-
+			this._chatAccessibilityService.acceptRequest();
 			const input = query ?? editorValue;
-			const result = await this.chatService.sendRequest(this.viewModel.sessionId, input);
+			const usedSlashCommand = this.lookupSlashCommand(typeof input === 'string' ? input : input.message);
+			const result = await this.chatService.sendRequest(this.viewModel.sessionId, input, usedSlashCommand);
+
 			if (result) {
-				revealLastElement(this.tree);
 				this.inputPart.acceptInput(query);
+				result.responseCompletePromise.then(async () => {
+					const responses = this.viewModel?.getItems().filter(isResponseVM);
+					const lastResponse = responses?.[responses.length - 1];
+					this._chatAccessibilityService.acceptResponse(lastResponse);
+				});
+			} else {
+				this._chatAccessibilityService.acceptResponse();
 			}
 		}
 	}
 
-	getCodeBlockInfosForResponse(response: IInteractiveResponseViewModel): IChatCodeBlockInfo[] {
+	private lookupSlashCommand(input: string): ISlashCommand | undefined {
+		return this.lastSlashCommands?.find(sc => input.startsWith(`/${sc.command}`));
+	}
+
+	getCodeBlockInfosForResponse(response: IChatResponseViewModel): IChatCodeBlockInfo[] {
 		return this.renderer.getCodeBlockInfosForResponse(response);
 	}
 
@@ -407,6 +485,7 @@ export class ChatWidget extends Disposable implements IChatWidget {
 	}
 
 	layout(height: number, width: number): void {
+		width = Math.min(width, 850);
 		this.bodyDimension = new dom.Dimension(width, height);
 
 		const inputPartHeight = this.inputPart.layout(height, width);
@@ -424,14 +503,52 @@ export class ChatWidget extends Disposable implements IChatWidget {
 		this.listContainer.style.height = `${height - inputPartHeight}px`;
 	}
 
+	private _dynamicMessageLayoutData?: { numOfMessages: number; maxHeight: number };
+
+	// An alternative to layout, this allows you to specify the number of ChatTreeItems
+	// you want to show, and the max height of the container. It will then layout the
+	// tree to show that many items.
+	setDynamicChatTreeItemLayout(numOfChatTreeItems: number, maxHeight: number) {
+		this._dynamicMessageLayoutData = { numOfMessages: numOfChatTreeItems, maxHeight };
+		this._register(this.renderer.onDidChangeItemHeight(() => this.layoutDynamicChatTreeItemMode()));
+	}
+
+	layoutDynamicChatTreeItemMode(allowRecurse = true): void {
+		if (!this.viewModel) {
+			return;
+		}
+		const inputHeight = this.inputPart.layout(this._dynamicMessageLayoutData!.maxHeight, this.container.offsetWidth);
+
+		const totalMessages = this.viewModel.getItems();
+		// grab the last N messages
+		const messages = totalMessages.slice(-this._dynamicMessageLayoutData!.numOfMessages);
+
+		const needsRerender = messages.some(m => m.currentRenderedHeight === undefined);
+		const listHeight = needsRerender
+			? this._dynamicMessageLayoutData!.maxHeight
+			: messages.reduce((acc, message) => acc + message.currentRenderedHeight!, 0);
+
+		this.layout(
+			Math.min(
+				// we add an additional 25px in order to show that there is scrollable content
+				inputHeight + listHeight + (totalMessages.length > 2 ? 25 : 0),
+				this._dynamicMessageLayoutData!.maxHeight
+			),
+			this.container.offsetWidth
+		);
+
+		if (needsRerender && allowRecurse) {
+			// TODO: figure out a better place to reveal the last element
+			revealLastElement(this.tree);
+			this.layoutDynamicChatTreeItemMode(false);
+		}
+	}
+
 	saveState(): void {
 		this.inputPart.saveState();
 	}
 
 	getViewState(): IViewState {
-		if (this.inputEditor.getOption(EditorOption.readOnly)) {
-			return { inputValue: undefined };
-		}
 		this.inputPart.saveState();
 		return { inputValue: this.inputPart.inputEditor.getValue() };
 	}
@@ -450,15 +567,19 @@ export class ChatWidgetService implements IChatWidgetService {
 
 	constructor(
 		@IViewsService private readonly viewsService: IViewsService,
-		@IChatContributionService private readonly interactiveSessionContributionService: IChatContributionService,
+		@IChatContributionService private readonly chatContributionService: IChatContributionService,
 	) { }
 
 	getWidgetByInputUri(uri: URI): ChatWidget | undefined {
 		return this._widgets.find(w => isEqual(w.inputUri, uri));
 	}
 
+	getWidgetBySessionId(sessionId: string): ChatWidget | undefined {
+		return this._widgets.find(w => w.viewModel?.sessionId === sessionId);
+	}
+
 	async revealViewForProvider(providerId: string): Promise<ChatWidget | undefined> {
-		const viewId = this.interactiveSessionContributionService.getViewIdForProvider(providerId);
+		const viewId = this.chatContributionService.getViewIdForProvider(providerId);
 		const view = await this.viewsService.openView<ChatViewPane>(viewId);
 
 		return view?.widget;
@@ -485,3 +606,4 @@ export class ChatWidgetService implements IChatWidgetService {
 		);
 	}
 }
+
