@@ -11,6 +11,7 @@ import { Iterable } from 'vs/base/common/iterator';
 import { Disposable, IDisposable, toDisposable } from 'vs/base/common/lifecycle';
 import { StopWatch } from 'vs/base/common/stopwatch';
 import { withNullAsUndefined } from 'vs/base/common/types';
+import { URI, UriComponents } from 'vs/base/common/uri';
 import { localize } from 'vs/nls';
 import { CommandsRegistry } from 'vs/platform/commands/common/commands';
 import { IContextKey, IContextKeyService } from 'vs/platform/contextkey/common/contextkey';
@@ -18,19 +19,30 @@ import { IInstantiationService } from 'vs/platform/instantiation/common/instanti
 import { ILogService } from 'vs/platform/log/common/log';
 import { IStorageService, StorageScope, StorageTarget } from 'vs/platform/storage/common/storage';
 import { ITelemetryService } from 'vs/platform/telemetry/common/telemetry';
+import { IWorkspaceContextService } from 'vs/platform/workspace/common/workspace';
 import { CONTEXT_PROVIDER_EXISTS } from 'vs/workbench/contrib/chat/common/chatContextKeys';
 import { ChatModel, ChatWelcomeMessageModel, IChatModel, ISerializableChatData, ISerializableChatsData } from 'vs/workbench/contrib/chat/common/chatModel';
-import { IChat, IChatCompleteResponse, IChatDetail, IChatDynamicRequest, IChatProgress, IChatProvider, IChatProviderInfo, IChatReplyFollowup, IChatService, IChatUserActionEvent, ISlashCommand, ISlashCommandProvider, InteractiveSessionCopyKind, InteractiveSessionVoteDirection } from 'vs/workbench/contrib/chat/common/chatService';
+import { IChat, IChatCompleteResponse, IChatDetail, IChatDynamicRequest, IChatProgress, IChatProvider, IChatProviderInfo, IChatReplyFollowup, IChatService, IChatTransferredSessionData, IChatUserActionEvent, ISlashCommand, ISlashCommandProvider, InteractiveSessionCopyKind, InteractiveSessionVoteDirection } from 'vs/workbench/contrib/chat/common/chatService';
 import { IExtensionService } from 'vs/workbench/services/extensions/common/extensions';
 
 const serializedChatKey = 'interactive.sessions';
 
+const globalChatKey = 'chat.workspaceTransfer';
+interface IChatTransfer {
+	toWorkspace: UriComponents;
+	timestampInMilliseconds: number;
+	chat: ISerializableChatData;
+	inputValue: string;
+}
+const SESSION_TRANSFER_EXPIRATION_IN_MILLISECONDS = 1000 * 60;
+
 type ChatProviderInvokedEvent = {
 	providerId: string;
 	timeToFirstProgress: number;
 	totalTime: number;
 	result: 'success' | 'error' | 'errorWithOutput' | 'cancelled' | 'filtered';
 	requestType: 'string' | 'followup' | 'slashCommand';
+	slashCommand: string | undefined;
 };
 
 type ChatProviderInvokedClassification = {
@@ -39,6 +51,7 @@ type ChatProviderInvokedClassification = {
 	totalTime: { classification: 'SystemMetaData'; purpose: 'PerformanceAndHealth'; isMeasurement: true; comment: 'The total time it took to run the provider\'s `provideResponseWithProgress`.' };
 	result: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'Whether invoking the ChatProvider resulted in an error.' };
 	requestType: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'The type of request that the user made.' };
+	slashCommand?: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'The type of slashCommand used.' };
 	owner: 'roblourens';
 	comment: 'Provides insight into the performance of Chat providers.';
 };
@@ -103,7 +116,7 @@ type ChatTerminalClassification = {
 	comment: 'Provides insight into the usage of Chat features.';
 };
 
-const maxPersistedSessions = 20;
+const maxPersistedSessions = 25;
 
 export class ChatService extends Disposable implements IChatService {
 	declare _serviceBrand: undefined;
@@ -115,9 +128,17 @@ export class ChatService extends Disposable implements IChatService {
 	private readonly _persistedSessions: ISerializableChatsData;
 	private readonly _hasProvider: IContextKey<boolean>;
 
+	private _transferredSessionData: IChatTransferredSessionData | undefined;
+	public get transferredSessionData(): IChatTransferredSessionData | undefined {
+		return this._transferredSessionData;
+	}
+
 	private readonly _onDidPerformUserAction = this._register(new Emitter<IChatUserActionEvent>());
 	public readonly onDidPerformUserAction: Event<IChatUserActionEvent> = this._onDidPerformUserAction.event;
 
+	private readonly _onDidSubmitSlashCommand = this._register(new Emitter<{ slashCommand: string; sessionId: string }>());
+	public readonly onDidSubmitSlashCommand = this._onDidSubmitSlashCommand.event;
+
 	constructor(
 		@IStorageService private readonly storageService: IStorageService,
 		@ILogService private readonly logService: ILogService,
@@ -125,6 +146,7 @@ export class ChatService extends Disposable implements IChatService {
 		@IInstantiationService private readonly instantiationService: IInstantiationService,
 		@ITelemetryService private readonly telemetryService: ITelemetryService,
 		@IContextKeyService private readonly contextKeyService: IContextKeyService,
+		@IWorkspaceContextService private readonly workspaceContextService: IWorkspaceContextService
 	) {
 		super();
 
@@ -134,10 +156,19 @@ export class ChatService extends Disposable implements IChatService {
 		if (sessionData) {
 			this._persistedSessions = this.deserializeChats(sessionData);
 			const countsForLog = Object.keys(this._persistedSessions).length;
-			this.trace('constructor', `Restored ${countsForLog} persisted sessions`);
+			if (countsForLog > 0) {
+				this.trace('constructor', `Restored ${countsForLog} persisted sessions`);
+			}
 		} else {
 			this._persistedSessions = {};
-			this.trace('constructor', 'No persisted sessions');
+		}
+
+		const transferredData = this.getTransferredSessionData();
+		const transferredChat = transferredData?.chat;
+		if (transferredChat) {
+			this.trace('constructor', `Transferred session ${transferredChat.sessionId}`);
+			this._persistedSessions[transferredChat.sessionId] = transferredChat;
+			this._transferredSessionData = { sessionId: transferredChat.sessionId, inputValue: transferredData.inputValue };
 		}
 
 		this._register(storageService.onWillSaveState(() => this.saveState()));
@@ -147,12 +178,21 @@ export class ChatService extends Disposable implements IChatService {
 		let allSessions: (ChatModel | ISerializableChatData)[] = Array.from(this._sessionModels.values())
 			.filter(session => session.getRequests().length > 0);
 		allSessions = allSessions.concat(
-			Object.values(this._persistedSessions).filter(session => session.requests.length));
+			Object.values(this._persistedSessions)
+				.filter(session => !this._sessionModels.has(session.sessionId))
+				.filter(session => session.requests.length));
 		allSessions.sort((a, b) => (b.creationDate ?? 0) - (a.creationDate ?? 0));
 		allSessions = allSessions.slice(0, maxPersistedSessions);
-		this.trace('onWillSaveState', `Persisting ${allSessions.length} sessions`);
+		if (allSessions.length) {
+			this.trace('onWillSaveState', `Persisting ${allSessions.length} sessions`);
+		}
 
 		const serialized = JSON.stringify(allSessions);
+
+		if (allSessions.length) {
+			this.trace('onWillSaveState', `Persisting ${serialized.length} chars`);
+		}
+
 		this.storageService.store(serializedChatKey, serialized, StorageScope.WORKSPACE, StorageTarget.MACHINE);
 	}
 
@@ -215,16 +255,41 @@ export class ChatService extends Disposable implements IChatService {
 		}
 	}
 
+	private getTransferredSessionData(): IChatTransfer | undefined {
+		const data: IChatTransfer[] = this.storageService.getObject(globalChatKey, StorageScope.PROFILE, []);
+		const workspaceUri = this.workspaceContextService.getWorkspace().folders[0]?.uri;
+		if (!workspaceUri) {
+			return;
+		}
+
+		const thisWorkspace = workspaceUri.toString();
+		const currentTime = Date.now();
+		// Only use transferred data if it was created recently
+		const transferred = data.find(item => URI.revive(item.toWorkspace).toString() === thisWorkspace && (currentTime - item.timestampInMilliseconds < SESSION_TRANSFER_EXPIRATION_IN_MILLISECONDS));
+		// Keep data that isn't for the current workspace and that hasn't expired yet
+		const filtered = data.filter(item => URI.revive(item.toWorkspace).toString() !== thisWorkspace && (currentTime - item.timestampInMilliseconds < SESSION_TRANSFER_EXPIRATION_IN_MILLISECONDS));
+		this.storageService.store(globalChatKey, JSON.stringify(filtered), StorageScope.PROFILE, StorageTarget.MACHINE);
+		return transferred;
+	}
+
 	getHistory(): IChatDetail[] {
-		const sessions = Object.values(this._persistedSessions);
+		const sessions = Object.values(this._persistedSessions)
+			.filter(session => session.requests.length > 0);
 		sessions.sort((a, b) => (b.creationDate ?? 0) - (a.creationDate ?? 0));
 
-		return sessions.map(item => {
-			return <IChatDetail>{
-				sessionId: item.sessionId,
-				title: item.requests[0]?.message || '',
-			};
-		});
+		return sessions
+			.filter(session => !this._sessionModels.has(session.sessionId))
+			.filter(session => !session.isImported)
+			.map(item => {
+				return <IChatDetail>{
+					sessionId: item.sessionId,
+					title: item.requests[0]?.message || '',
+				};
+			});
+	}
+
+	removeHistoryEntry(sessionId: string): void {
+		delete this._persistedSessions[sessionId];
 	}
 
 	startSession(providerId: string, token: CancellationToken): ChatModel {
@@ -235,13 +300,8 @@ export class ChatService extends Disposable implements IChatService {
 	private _startSession(providerId: string, someSessionHistory: ISerializableChatData | undefined, token: CancellationToken): ChatModel {
 		const model = this.instantiationService.createInstance(ChatModel, providerId, someSessionHistory);
 		this._sessionModels.set(model.sessionId, model);
-		const modelInitPromise = this.initializeSession(model, someSessionHistory, token);
-		modelInitPromise.then(resolvedModel => {
-			if (!resolvedModel) {
-				model.dispose();
-				this._sessionModels.delete(model.sessionId);
-			}
-		}).catch(err => {
+		const modelInitPromise = this.initializeSession(model, token);
+		modelInitPromise.catch(err => {
 			this.trace('startSession', `initializeSession failed: ${err}`);
 			model.setInitializationError(err);
 			model.dispose();
@@ -251,7 +311,22 @@ export class ChatService extends Disposable implements IChatService {
 		return model;
 	}
 
-	private async initializeSession(model: ChatModel, sessionHistory: ISerializableChatData | undefined, token: CancellationToken): Promise<ChatModel | undefined> {
+	private reinitializeModel(model: ChatModel): void {
+		model.startReinitialize();
+		this.startSessionInit(model, CancellationToken.None);
+	}
+
+	private startSessionInit(model: ChatModel, token: CancellationToken): void {
+		const modelInitPromise = this.initializeSession(model, token);
+		modelInitPromise.catch(err => {
+			this.trace('startSession', `initializeSession failed: ${err}`);
+			model.setInitializationError(err);
+			model.dispose();
+			this._sessionModels.delete(model.sessionId);
+		});
+	}
+
+	private async initializeSession(model: ChatModel, token: CancellationToken): Promise<void> {
 		await this.extensionService.activateByEvent(`onInteractiveSession:${model.providerId}`);
 
 		const provider = this._providers.get(model.providerId);
@@ -267,29 +342,26 @@ export class ChatService extends Disposable implements IChatService {
 		}
 
 		if (!session) {
-			if (sessionHistory) {
-				// sessionHistory was not used, so store it for later
-				this._persistedSessions[sessionHistory.sessionId] = sessionHistory;
-			}
-
-			this.trace('startSession', 'Provider returned no session');
-			return undefined;
+			throw new Error('Provider returned no session');
 		}
 
 		this.trace('startSession', `Provider returned session`);
 
-		const welcomeMessage = sessionHistory ? undefined : withNullAsUndefined(await provider.provideWelcomeMessage?.(token));
+		const welcomeMessage = model.welcomeMessage ? undefined : withNullAsUndefined(await provider.provideWelcomeMessage?.(token));
 		const welcomeModel = welcomeMessage && new ChatWelcomeMessageModel(
-			welcomeMessage.map(item => typeof item === 'string' ? new MarkdownString(item) : item as IChatReplyFollowup[]), session.responderUsername, session.responderAvatarIconUri);
+			model, welcomeMessage.map(item => typeof item === 'string' ? new MarkdownString(item) : item as IChatReplyFollowup[]));
 
 		model.initialize(session, welcomeModel);
-		return model;
 	}
 
 	getSession(sessionId: string): IChatModel | undefined {
 		return this._sessionModels.get(sessionId);
 	}
 
+	getSessionId(sessionProviderId: number): string | undefined {
+		return Iterable.find(this._sessionModels.values(), model => model.session?.id === sessionProviderId)?.sessionId;
+	}
+
 	getOrRestoreSession(sessionId: string): ChatModel | undefined {
 		const model = this._sessionModels.get(sessionId);
 		if (model) {
@@ -301,7 +373,10 @@ export class ChatService extends Disposable implements IChatService {
 			return undefined;
 		}
 
-		delete this._persistedSessions[sessionId];
+		if (sessionId === this.transferredSessionData?.sessionId) {
+			this._transferredSessionData = undefined;
+		}
+
 		return this._startSession(sessionData.providerId, sessionData, CancellationToken.None);
 	}
 
@@ -309,12 +384,12 @@ export class ChatService extends Disposable implements IChatService {
 		return this._startSession(data.providerId, data, CancellationToken.None);
 	}
 
-	async sendRequest(sessionId: string, request: string | IChatReplyFollowup): Promise<boolean> {
+	async sendRequest(sessionId: string, request: string | IChatReplyFollowup, usedSlashCommand?: ISlashCommand): Promise<{ responseCompletePromise: Promise<void> } | undefined> {
 		const messageText = typeof request === 'string' ? request : request.message;
 		this.trace('sendRequest', `sessionId: ${sessionId}, message: ${messageText.substring(0, 20)}${messageText.length > 20 ? '[...]' : ''}}`);
 		if (!messageText.trim()) {
 			this.trace('sendRequest', 'Rejected empty message');
-			return false;
+			return;
 		}
 
 		const model = this._sessionModels.get(sessionId);
@@ -330,15 +405,14 @@ export class ChatService extends Disposable implements IChatService {
 
 		if (this._pendingRequests.has(sessionId)) {
 			this.trace('sendRequest', `Session ${sessionId} already has a pending request`);
-			return false;
+			return;
 		}
 
 		// This method is only returning whether the request was accepted - don't block on the actual request
-		this._sendRequestAsync(model, provider, request);
-		return true;
+		return { responseCompletePromise: this._sendRequestAsync(model, provider, request, usedSlashCommand) };
 	}
 
-	private async _sendRequestAsync(model: ChatModel, provider: IChatProvider, message: string | IChatReplyFollowup): Promise<void> {
+	private async _sendRequestAsync(model: ChatModel, provider: IChatProvider, message: string | IChatReplyFollowup, usedSlashCommand?: ISlashCommand): Promise<void> {
 		const request = model.addRequest(message);
 
 		const resolvedCommand = typeof message === 'string' && message.startsWith('/') ? await this.handleSlashCommand(model.sessionId, message) : message;
@@ -357,8 +431,10 @@ export class ChatService extends Disposable implements IChatService {
 				gotProgress = true;
 				if ('content' in progress) {
 					this.trace('sendRequest', `Provider returned progress for session ${model.sessionId}, ${progress.content.length} chars`);
+				} else if ('placeholder' in progress) {
+					this.trace('sendRequest', `Provider returned placeholder for session ${model.sessionId}, ${progress.placeholder}`);
 				} else {
-					this.trace('sendRequest', `Provider returned id for session ${model.sessionId}, ${progress.responseId}`);
+					this.trace('sendRequest', `Provider returned id for session ${model.sessionId}, ${progress.requestId}`);
 				}
 
 				model.acceptResponseProgress(request, progress);
@@ -373,11 +449,15 @@ export class ChatService extends Disposable implements IChatService {
 					// Normally timings happen inside the EH around the actual provider. For cancellation we can measure how long the user waited before cancelling
 					totalTime: stopWatch.elapsed(),
 					result: 'cancelled',
-					requestType
+					requestType,
+					slashCommand: usedSlashCommand?.command
 				});
 
 				model.cancelRequest(request);
 			});
+			if (usedSlashCommand?.command) {
+				this._onDidSubmitSlashCommand.fire({ slashCommand: usedSlashCommand.command, sessionId: model.sessionId });
+			}
 			let rawResponse = await provider.provideReply({ session: model.session!, message: resolvedCommand }, progressCallback, token);
 			if (token.isCancellationRequested) {
 				return;
@@ -396,15 +476,20 @@ export class ChatService extends Disposable implements IChatService {
 					timeToFirstProgress: rawResponse.timings?.firstProgress ?? 0,
 					totalTime: rawResponse.timings?.totalElapsed ?? 0,
 					result,
-					requestType
+					requestType,
+					slashCommand: usedSlashCommand?.command
 				});
-				model.completeResponse(request, rawResponse);
+				model.setResponse(request, rawResponse);
 				this.trace('sendRequest', `Provider returned response for session ${model.sessionId}`);
 
+				// TODO refactor this or rethink the API https://github.com/microsoft/vscode-copilot/issues/593
 				if (provider.provideFollowups) {
 					Promise.resolve(provider.provideFollowups(model.session!, CancellationToken.None)).then(followups => {
 						model.setFollowups(request, withNullAsUndefined(followups));
+						model.completeResponse(request);
 					});
+				} else {
+					model.completeResponse(request);
 				}
 			}
 		});
@@ -412,6 +497,23 @@ export class ChatService extends Disposable implements IChatService {
 		rawResponsePromise.finally(() => {
 			this._pendingRequests.delete(model.sessionId);
 		});
+		return rawResponsePromise;
+	}
+
+	async removeRequest(sessionId: string, requestId: string): Promise<void> {
+		const model = this._sessionModels.get(sessionId);
+		if (!model) {
+			throw new Error(`Unknown session: ${sessionId}`);
+		}
+
+		await model.waitForInitialization();
+		const provider = this._providers.get(model.providerId);
+		if (!provider) {
+			throw new Error(`Unknown provider: ${model.providerId}`);
+		}
+
+		model.removeRequest(requestId);
+		provider.removeRequest?.(model.session!, requestId);
 	}
 
 	private async handleSlashCommand(sessionId: string, command: string): Promise<string> {
@@ -467,30 +569,30 @@ export class ChatService extends Disposable implements IChatService {
 		const model = Iterable.first(this._sessionModels.values());
 		if (!model) {
 			// If no session, create one- how and is the service the right place to decide this?
-			this.trace('addInteractiveRequest', 'No session available');
+			this.trace('addRequest', 'No session available');
 			return;
 		}
 
 		const provider = this._providers.get(model.providerId);
 		if (!provider || !provider.resolveRequest) {
-			this.trace('addInteractiveRequest', 'No provider available');
+			this.trace('addRequest', 'No provider available');
 			return undefined;
 		}
 
-		this.trace('addInteractiveRequest', `Calling resolveRequest for session ${model.sessionId}`);
+		this.trace('addRequest', `Calling resolveRequest for session ${model.sessionId}`);
 		const request = await provider.resolveRequest(model.session!, context, CancellationToken.None);
 		if (!request) {
-			this.trace('addInteractiveRequest', `Provider returned no request for session ${model.sessionId}`);
+			this.trace('addRequest', `Provider returned no request for session ${model.sessionId}`);
 			return;
 		}
 
 		// Maybe this API should queue a request after the current one?
-		this.trace('addInteractiveRequest', `Sending resolved request for session ${model.sessionId}`);
+		this.trace('addRequest', `Sending resolved request for session ${model.sessionId}`);
 		this.sendRequest(model.sessionId, request.message);
 	}
 
 	async sendRequestToProvider(sessionId: string, message: IChatDynamicRequest): Promise<void> {
-		this.trace('sendInteractiveRequestToProvider', `sessionId: ${sessionId}`);
+		this.trace('sendRequestToProvider', `sessionId: ${sessionId}`);
 		await this.sendRequest(sessionId, message.message);
 	}
 
@@ -510,8 +612,8 @@ export class ChatService extends Disposable implements IChatService {
 		const request = model.addRequest(message);
 		model.acceptResponseProgress(request, {
 			content: response.message,
-		});
-		model.completeResponse(request, {
+		}, true);
+		model.setResponse(request, {
 			session: model.session!,
 			errorDetails: response.errorDetails,
 		});
@@ -529,9 +631,7 @@ export class ChatService extends Disposable implements IChatService {
 			throw new Error(`Unknown session: ${sessionId}`);
 		}
 
-		if (model.getRequests().length && !model.isImported) {
-			this._persistedSessions[sessionId] = model.toJSON();
-		}
+		this._persistedSessions[sessionId] = model.toJSON();
 
 		model.dispose();
 		this._sessionModels.delete(sessionId);
@@ -548,6 +648,10 @@ export class ChatService extends Disposable implements IChatService {
 		this._providers.set(provider.id, provider);
 		this._hasProvider.set(true);
 
+		Array.from(this._sessionModels.values())
+			.filter(model => model.providerId === provider.id)
+			.forEach(model => this.reinitializeModel(model));
+
 		return toDisposable(() => {
 			this.trace('registerProvider', `Disposing chat provider`);
 			this._providers.delete(provider.id);
@@ -573,4 +677,22 @@ export class ChatService extends Disposable implements IChatService {
 			};
 		});
 	}
+
+	transferChatSession(transferredSessionData: IChatTransferredSessionData, toWorkspace: URI): void {
+		const model = Iterable.find(this._sessionModels.values(), model => model.sessionId === transferredSessionData.sessionId);
+		if (!model) {
+			throw new Error(`Failed to transfer session. Unknown session ID: ${transferredSessionData.sessionId}`);
+		}
+
+		const existingRaw: IChatTransfer[] = this.storageService.getObject(globalChatKey, StorageScope.PROFILE, []);
+		existingRaw.push({
+			chat: model.toJSON(),
+			timestampInMilliseconds: Date.now(),
+			toWorkspace: toWorkspace,
+			inputValue: transferredSessionData.inputValue,
+		});
+
+		this.storageService.store(globalChatKey, JSON.stringify(existingRaw), StorageScope.PROFILE, StorageTarget.MACHINE);
+		this.trace('transferChatSession', `Transferred session ${model.sessionId} to workspace ${toWorkspace.toString()}`);
+	}
 }
