@@ -155,12 +155,14 @@ export class ResultCodeEditorView extends CodeEditorView {
 
 			if (modifiedBaseRange) {
 				const blockClassNames = ['merge-editor-block'];
+				let blockPadding: [top: number, right: number, bottom: number, left: number] = [0, 0, 0, 0];
 				const isHandled = model.isHandled(modifiedBaseRange).read(reader);
 				if (isHandled) {
 					blockClassNames.push('handled');
 				}
 				if (modifiedBaseRange === activeModifiedBaseRange) {
 					blockClassNames.push('focused');
+					blockPadding = [0, 2, 0, 2];
 				}
 				if (modifiedBaseRange.isConflicting) {
 					blockClassNames.push('conflicting');
@@ -177,6 +179,7 @@ export class ResultCodeEditorView extends CodeEditorView {
 					options: {
 						showIfCollapsed: true,
 						blockClassName: blockClassNames.join(' '),
+						blockPadding,
 						blockIsAfterEnd: range.startLineNumber > textModel.getLineCount(),
 						description: 'Result Diff',
 						minimap: {
