@@ -283,10 +283,10 @@ class MergeModelInterface extends Disposable {
 
 		const diffComputer: IMergeDiffComputer = {
 			async computeDiff(textModel1: ITextModel, textModel2: ITextModel, reader: IReader): Promise<IMergeDiffComputerResult> {
-				const result = await linesDiffComputers.legacy.computeDiff(
+				const result = await linesDiffComputers.getLegacy().computeDiff(
 					textModel1.getLinesContent(),
 					textModel2.getLinesContent(),
-					{ ignoreTrimWhitespace: false, maxComputationTimeMs: 10000 }
+					{ ignoreTrimWhitespace: false, maxComputationTimeMs: 10000, computeMoves: false }
 				);
 				const changes = result.changes.map(c =>
 					new DetailedLineRangeMapping(
