@@ -111,7 +111,7 @@ export class FindModel extends Disposable {
 			this._registerModelListener(this._notebookEditor.textModel);
 		}
 
-		this._findMatchDecorationModel = new FindMatchDecorationModel(this._notebookEditor);
+		this._findMatchDecorationModel = new FindMatchDecorationModel(this._notebookEditor, this._notebookEditor.getId());
 	}
 
 	private _updateCellStates(e: FindReplaceStateChangedEvent) {
@@ -125,7 +125,7 @@ export class FindModel extends Disposable {
 
 		// we only update cell state if users are using the hybrid mode (both input and preview are enabled)
 		const updateEditingState = () => {
-			const viewModel = this._notebookEditor._getViewModel() as NotebookViewModel | undefined;
+			const viewModel = this._notebookEditor.getViewModel() as NotebookViewModel | undefined;
 			if (!viewModel) {
 				return;
 			}
@@ -162,6 +162,25 @@ export class FindModel extends Disposable {
 		};
 
 
+		if (e.isReplaceRevealed && !this._state.isReplaceRevealed) {
+			// replace is hidden, we need to switch all markdown cells to preview mode
+			const viewModel = this._notebookEditor.getViewModel() as NotebookViewModel | undefined;
+			if (!viewModel) {
+				return;
+			}
+
+			for (let i = 0; i < viewModel.length; i++) {
+				const cell = viewModel.cellAt(i);
+				if (cell && cell.cellKind === CellKind.Markup) {
+					if (cell.getEditState() === CellEditState.Editing && cell.editStateSource === 'find') {
+						cell.updateEditState(CellEditState.Preview, 'find');
+					}
+				}
+			}
+
+			return;
+		}
+
 		if (e.isReplaceRevealed) {
 			updateEditingState();
 		} else if ((e.filters || e.isRevealed || e.searchString || e.replaceString) && this._state.isRevealed && this._state.isReplaceRevealed) {
@@ -452,6 +471,9 @@ export class FindModel extends Disposable {
 	}
 
 	private async _compute(token: CancellationToken): Promise<CellFindMatchWithIndex[] | null> {
+		if (!this._notebookEditor.hasModel()) {
+			return null;
+		}
 		let ret: CellFindMatchWithIndex[] | null = null;
 		const val = this._state.searchString;
 		const wordSeparators = this._configurationService.inspect<string>('editor.wordSeparators').value;
@@ -466,13 +488,8 @@ export class FindModel extends Disposable {
 			includeMarkupPreview: !!this._state.filters?.markupPreview,
 			includeOutput: !!this._state.filters?.codeOutput
 		};
-		if (!val) {
-			ret = null;
-		} else if (!this._notebookEditor.hasModel()) {
-			ret = null;
-		} else {
-			ret = await this._notebookEditor.find(val, options, token);
-		}
+
+		ret = await this._notebookEditor.find(val, options, token);
 
 		if (token.isCancellationRequested) {
 			return null;
