@@ -106,6 +106,7 @@ class OutputContribution extends Disposable implements IWorkbenchContribution {
 
 	private registerActions(): void {
 		this.registerSwitchOutputAction();
+		this.registerShowOutputChannelsAction();
 		this.registerClearOutputAction();
 		this.registerToggleAutoScrollAction();
 		this.registerOpenActiveLogOutputFileAction();
@@ -174,6 +175,45 @@ class OutputContribution extends Disposable implements IWorkbenchContribution {
 		}));
 	}
 
+	private registerShowOutputChannelsAction(): void {
+		this._register(registerAction2(class extends Action2 {
+			constructor() {
+				super({
+					id: 'workbench.action.showOutputChannels',
+					title: { value: nls.localize('showOutputChannels', "Show Output Channels..."), original: 'Show Output Channels...' },
+					category: { value: nls.localize('output', "Output"), original: 'Output' },
+					f1: true
+				});
+			}
+			async run(accessor: ServicesAccessor): Promise<void> {
+				const outputService = accessor.get(IOutputService);
+				const quickInputService = accessor.get(IQuickInputService);
+				const extensionChannels = [], coreChannels = [];
+				for (const channel of outputService.getChannelDescriptors()) {
+					if (channel.extensionId) {
+						extensionChannels.push(channel);
+					} else {
+						coreChannels.push(channel);
+					}
+				}
+				const entries: ({ id: string; label: string } | IQuickPickSeparator)[] = [];
+				for (const { id, label } of extensionChannels) {
+					entries.push({ id, label });
+				}
+				if (extensionChannels.length && coreChannels.length) {
+					entries.push({ type: 'separator' });
+				}
+				for (const { id, label } of coreChannels) {
+					entries.push({ id, label });
+				}
+				const entry = await quickInputService.pick(entries, { placeHolder: nls.localize('selectOutput', "Select Output Channel") });
+				if (entry) {
+					return outputService.showChannel(entry.id);
+				}
+			}
+		}));
+	}
+
 	private registerClearOutputAction(): void {
 		this._register(registerAction2(class extends Action2 {
 			constructor() {
@@ -328,9 +368,19 @@ class OutputContribution extends Disposable implements IWorkbenchContribution {
 					menu: {
 						id: MenuId.CommandPalette,
 					},
+					description: {
+						description: 'workbench.action.openLogFile',
+						args: [{
+							name: 'logFile',
+							schema: {
+								markdownDescription: nls.localize('logFile', "The id of the log file to open, for example `\"window\"`. Currently the best way to get this is to get the ID by checking the `workbench.action.output.show.<id>` commands"),
+								type: 'string'
+							}
+						}]
+					},
 				});
 			}
-			async run(accessor: ServicesAccessor): Promise<void> {
+			async run(accessor: ServicesAccessor, args?: unknown): Promise<void> {
 				const outputService = accessor.get(IOutputService);
 				const quickInputService = accessor.get(IQuickInputService);
 				const instantiationService = accessor.get(IInstantiationService);
@@ -339,7 +389,14 @@ class OutputContribution extends Disposable implements IWorkbenchContribution {
 				const entries: IOutputChannelQuickPickItem[] = outputService.getChannelDescriptors().filter(c => c.file && c.log)
 					.map(channel => (<IOutputChannelQuickPickItem>{ id: channel.id, label: channel.label, channel }));
 
-				const entry = await quickInputService.pick(entries, { placeHolder: nls.localize('selectlogFile', "Select Log file") });
+				const argName = args && typeof args === 'string' ? args : undefined;
+				let entry: IOutputChannelQuickPickItem | undefined;
+				if (argName) {
+					entry = entries.find(e => e.id === argName);
+				}
+				if (!entry) {
+					entry = await quickInputService.pick(entries, { placeHolder: nls.localize('selectlogFile', "Select Log File") });
+				}
 				if (entry) {
 					assertIsDefined(entry.channel.file);
 					await editorService.openEditor(instantiationService.createInstance(LogViewerInput, (entry.channel as IFileOutputChannelDescriptor)), { pinned: true });
