@@ -59,15 +59,15 @@ export class KeybindingsSearchWidget extends SearchWidget {
 		@IKeybindingService keybindingService: IKeybindingService,
 	) {
 		super(parent, options, contextViewService, instantiationService, contextKeyService, keybindingService);
+
 		this._register(toDisposable(() => this.stopRecordingKeys()));
+
 		this._chords = null;
 		this._inputValue = '';
-
-		this._reset();
 	}
 
 	override clear(): void {
-		this._reset();
+		this._chords = null;
 		super.clear();
 	}
 
@@ -81,7 +81,7 @@ export class KeybindingsSearchWidget extends SearchWidget {
 	}
 
 	stopRecordingKeys(): void {
-		this._reset();
+		this._chords = null;
 		this.recordDisposables.clear();
 	}
 
@@ -90,10 +90,6 @@ export class KeybindingsSearchWidget extends SearchWidget {
 		this.inputBox.value = this._inputValue;
 	}
 
-	private _reset() {
-		this._chords = null;
-	}
-
 	private _onKeyDown(keyboardEvent: IKeyboardEvent): void {
 		keyboardEvent.preventDefault();
 		keyboardEvent.stopPropagation();
@@ -103,10 +99,6 @@ export class KeybindingsSearchWidget extends SearchWidget {
 			return;
 		}
 		if (keyboardEvent.equals(KeyCode.Escape)) {
-			if (this._chords !== null) {
-				this.clear();
-			}
-
 			this._onEscape.fire();
 			return;
 		}
@@ -185,8 +177,8 @@ export class DefineKeybindingWidget extends Widget {
 		this._keybindingInputWidget.startRecordingKeys();
 		this._register(this._keybindingInputWidget.onKeybinding(keybinding => this.onKeybinding(keybinding)));
 		this._register(this._keybindingInputWidget.onEnter(() => this.hide()));
-		this._register(this._keybindingInputWidget.onEscape(() => this.onEscape()));
-		this._register(this._keybindingInputWidget.onBlur(() => this.onBlur()));
+		this._register(this._keybindingInputWidget.onEscape(() => this.clearOrHide()));
+		this._register(this._keybindingInputWidget.onBlur(() => this.onCancel()));
 
 		this._outputNode = dom.append(this._domNode.domNode, dom.$('.output'));
 		this._showExistingKeybindingsNode = dom.append(this._domNode.domNode, dom.$('.existing'));
@@ -251,19 +243,15 @@ export class DefineKeybindingWidget extends Widget {
 		dom.clearNode(this._outputNode);
 		dom.clearNode(this._showExistingKeybindingsNode);
 
-
-
 		const firstLabel = new KeybindingLabel(this._outputNode, OS, defaultKeybindingLabelStyles);
 		firstLabel.set(withNullAsUndefined(this._chords?.[0]));
 
-
 		if (this._chords) {
 			for (let i = 1; i < this._chords.length; i++) {
 				this._outputNode.appendChild(document.createTextNode(nls.localize('defineKeybinding.chordsTo', "chord to")));
 				const chordLabel = new KeybindingLabel(this._outputNode, OS, defaultKeybindingLabelStyles);
 				chordLabel.set(this._chords[i]);
 			}
-
 		}
 
 		const label = this.getUserSettingsLabel();
@@ -280,19 +268,20 @@ export class DefineKeybindingWidget extends Widget {
 		return label;
 	}
 
-	private onBlur(): void {
+	private onCancel(): void {
 		this._chords = null;
 		this.hide();
 	}
 
-	private onEscape(): void {
-		if (this._chords !== null) {
+	private clearOrHide(): void {
+		if (this._chords === null) {
+			this.hide();
+		} else {
 			this._chords = null;
+			this._keybindingInputWidget.clear();
 			dom.clearNode(this._outputNode);
 			dom.clearNode(this._showExistingKeybindingsNode);
-			return;
 		}
-		this.hide();
 	}
 
 	private hide(): void {
