@@ -6,12 +6,12 @@
 import * as nls from 'vs/nls';
 import { URI } from 'vs/base/common/uri';
 import * as network from 'vs/base/common/network';
-import { Disposable } from 'vs/base/common/lifecycle';
-import { IReplaceService } from 'vs/workbench/contrib/search/common/replace';
+import { Disposable, IReference } from 'vs/base/common/lifecycle';
+import { IReplaceService } from 'vs/workbench/contrib/search/browser/replace';
 import { IEditorService } from 'vs/workbench/services/editor/common/editorService';
 import { IModelService } from 'vs/editor/common/services/model';
 import { ILanguageService } from 'vs/editor/common/languages/language';
-import { Match, FileMatch, FileMatchOrMatch, ISearchWorkbenchService } from 'vs/workbench/contrib/search/common/searchModel';
+import { Match, FileMatch, FileMatchOrMatch, ISearchWorkbenchService, MatchInNotebook } from 'vs/workbench/contrib/search/browser/searchModel';
 import { IProgress, IProgressStep } from 'vs/platform/progress/common/progress';
 import { ITextModelService, ITextModelContentProvider } from 'vs/editor/common/services/resolverService';
 import { IWorkbenchContribution } from 'vs/workbench/common/contributions';
@@ -27,6 +27,8 @@ import { ILabelService } from 'vs/platform/label/common/label';
 import { dirname } from 'vs/base/common/resources';
 import { Promises } from 'vs/base/common/async';
 import { SaveSourceRegistry } from 'vs/workbench/common/editor';
+import { CellUri, IResolvedNotebookEditorModel } from 'vs/workbench/contrib/notebook/common/notebookCommon';
+import { INotebookEditorModelResolverService } from 'vs/workbench/contrib/notebook/common/notebookEditorModelResolverService';
 
 const REPLACE_PREVIEW = 'replacePreview';
 
@@ -99,7 +101,8 @@ export class ReplaceService implements IReplaceService {
 		@IEditorService private readonly editorService: IEditorService,
 		@ITextModelService private readonly textModelResolverService: ITextModelService,
 		@IBulkEditService private readonly bulkEditorService: IBulkEditService,
-		@ILabelService private readonly labelService: ILabelService
+		@ILabelService private readonly labelService: ILabelService,
+		@INotebookEditorModelResolverService private readonly notebookEditorModelResolverService: INotebookEditorModelResolverService
 	) { }
 
 	replace(match: Match): Promise<any>;
@@ -109,7 +112,25 @@ export class ReplaceService implements IReplaceService {
 		const edits = this.createEdits(arg, resource);
 		await this.bulkEditorService.apply(edits, { progress });
 
-		return Promises.settled(edits.map(async e => this.textFileService.files.get(e.resource)?.save({ source: ReplaceService.REPLACE_SAVE_SOURCE })));
+		const rawTextPromises = edits.map(async e => {
+			if (e.resource.scheme === network.Schemas.vscodeNotebookCell) {
+				const notebookResource = CellUri.parse(e.resource)?.notebook;
+				if (notebookResource) {
+					let ref: IReference<IResolvedNotebookEditorModel> | undefined;
+					try {
+						ref = await this.notebookEditorModelResolverService.resolve(notebookResource);
+						await ref.object.save({ source: ReplaceService.REPLACE_SAVE_SOURCE });
+					} finally {
+						ref?.dispose();
+					}
+				}
+				return;
+			} else {
+				return this.textFileService.files.get(e.resource)?.save({ source: ReplaceService.REPLACE_SAVE_SOURCE });
+			}
+		});
+
+		return Promises.settled(rawTextPromises);
 	}
 
 	async openReplacePreview(element: FileMatchOrMatch, preserveFocus?: boolean, sideBySide?: boolean, pinned?: boolean): Promise<any> {
@@ -177,8 +198,16 @@ export class ReplaceService implements IReplaceService {
 		const edits: ResourceTextEdit[] = [];
 
 		if (arg instanceof Match) {
-			const match = <Match>arg;
-			edits.push(this.createEdit(match, match.replaceString, resource));
+			if (arg instanceof MatchInNotebook) {
+				if (!arg.isWebviewMatch()) {
+					// only apply edits if it's not a webview match, since webview matches are read-only
+					const match = <MatchInNotebook>arg;
+					edits.push(this.createEdit(match, match.replaceString, match.cell.uri));
+				}
+			} else {
+				const match = <Match>arg;
+				edits.push(this.createEdit(match, match.replaceString, resource));
+			}
 		}
 
 		if (arg instanceof FileMatch) {
@@ -189,11 +218,12 @@ export class ReplaceService implements IReplaceService {
 			arg.forEach(element => {
 				const fileMatch = <FileMatch>element;
 				if (fileMatch.count() > 0) {
-					edits.push(...fileMatch.matches().map(match => this.createEdit(match, match.replaceString, resource)));
+					edits.push(...fileMatch.matches().flatMap(
+						match => this.createEdits(match, resource)
+					));
 				}
 			});
 		}
-
 		return edits;
 	}
 
