@@ -36,6 +36,10 @@ suite('Search Actions', () => {
 		counter = 0;
 	});
 
+	teardown(() => {
+		instantiationService.dispose();
+	});
+
 	test('get next element to focus after removing a match when it has next sibling file', function () {
 		const fileMatch1 = aFileMatch();
 		const fileMatch2 = aFileMatch();
@@ -116,7 +120,7 @@ suite('Search Actions', () => {
 		}, searchModel.searchResult, searchModel, null);
 		return instantiationService.createInstance(FileMatch, {
 			pattern: ''
-		}, undefined, undefined, folderMatch, rawMatch, null);
+		}, undefined, undefined, folderMatch, rawMatch, null, '');
 	}
 
 	function aMatch(fileMatch: FileMatch): Match {
