@@ -45,6 +45,10 @@ suite('Search - Viewlet', () => {
 		instantiation.stub(ILogService, new NullLogService());
 	});
 
+	teardown(() => {
+		instantiation.dispose();
+	});
+
 	test('Data Source', function () {
 		const result: SearchResult = aSearchResult();
 		result.query = {
@@ -74,7 +78,7 @@ suite('Search - Viewlet', () => {
 					endColumn: 1
 				}
 			}]
-		}]);
+		}], '');
 
 		const fileMatch = result.matches()[0];
 		const lineMatch = fileMatch.matches()[0];
@@ -177,7 +181,7 @@ suite('Search - Viewlet', () => {
 		};
 		return instantiation.createInstance(FileMatch, {
 			pattern: ''
-		}, undefined, undefined, parentFolder ?? aFolderMatch('', 0), rawMatch, null);
+		}, undefined, undefined, parentFolder ?? aFolderMatch('', 0), rawMatch, null, '');
 	}
 
 	function aFolderMatch(path: string, index: number, parent?: SearchResult): FolderMatch {
