@@ -4,15 +4,14 @@
  *--------------------------------------------------------------------------------------------*/
 
 import { StandardMouseEvent } from 'vs/base/browser/mouseEvent';
-import { Action, IAction } from 'vs/base/common/actions';
+import { IAction } from 'vs/base/common/actions';
 import { createAndFillInContextMenuActions } from 'vs/platform/actions/browser/menuEntryActionViewItem';
 import { IMenu } from 'vs/platform/actions/common/actions';
 import { IContextMenuService } from 'vs/platform/contextview/browser/contextView';
 
-export function openContextMenu(event: MouseEvent, parent: HTMLElement, menu: IMenu, contextMenuService: IContextMenuService, extraActions?: Action[]): void {
+export function openContextMenu(event: MouseEvent, parent: HTMLElement, menu: IMenu, contextMenuService: IContextMenuService, extraActions?: IAction[]): void {
 	const standardEvent = new StandardMouseEvent(event);
 
-	const anchor: { x: number; y: number } = { x: standardEvent.posx, y: standardEvent.posy };
 	const actions: IAction[] = [];
 
 	createAndFillInContextMenuActions(menu, undefined, actions);
@@ -22,7 +21,7 @@ export function openContextMenu(event: MouseEvent, parent: HTMLElement, menu: IM
 	}
 
 	contextMenuService.showContextMenu({
-		getAnchor: () => anchor,
+		getAnchor: () => standardEvent,
 		getActions: () => actions,
 		getActionsContext: () => parent,
 	});
