@@ -7,46 +7,35 @@ import { Schemas } from 'vs/base/common/network';
 import { env } from 'vs/base/common/process';
 import { withNullAsUndefined } from 'vs/base/common/types';
 import { IConfigurationService } from 'vs/platform/configuration/common/configuration';
-import { ILogService } from 'vs/platform/log/common/log';
 import { IWorkspaceContextService, IWorkspaceFolder } from 'vs/platform/workspace/common/workspace';
 import { IConfigurationResolverService } from 'vs/workbench/services/configurationResolver/common/configurationResolver';
 import { IHistoryService } from 'vs/workbench/services/history/common/history';
 import { IProcessEnvironment, OperatingSystem, OS } from 'vs/base/common/platform';
-import { IShellLaunchConfig, ITerminalProfile, ITerminalProfileObject, TerminalIcon, TerminalSettingId, TerminalSettingPrefix } from 'vs/platform/terminal/common/terminal';
+import { IShellLaunchConfig, ITerminalLogService, ITerminalProfile, TerminalIcon, TerminalSettingId } from 'vs/platform/terminal/common/terminal';
 import { IShellLaunchConfigResolveOptions, ITerminalProfileResolverService, ITerminalProfileService } from 'vs/workbench/contrib/terminal/common/terminal';
 import * as path from 'vs/base/common/path';
 import { Codicon } from 'vs/base/common/codicons';
 import { getIconRegistry, IIconRegistry } from 'vs/platform/theme/common/iconRegistry';
 import { IRemoteAgentService } from 'vs/workbench/services/remote/common/remoteAgentService';
 import { debounce } from 'vs/base/common/decorators';
-import { ThemeIcon } from 'vs/platform/theme/common/themeService';
+import { ThemeIcon } from 'vs/base/common/themables';
 import { URI } from 'vs/base/common/uri';
 import { equals } from 'vs/base/common/arrays';
-import { IStorageService, StorageScope } from 'vs/platform/storage/common/storage';
-import Severity from 'vs/base/common/severity';
-import { INotificationService, IPromptChoice, NeverShowAgainScope } from 'vs/platform/notification/common/notification';
-import { localize } from 'vs/nls';
 import { deepClone } from 'vs/base/common/objects';
 import { terminalProfileArgsMatch, isUriComponents } from 'vs/platform/terminal/common/terminalProfiles';
 import { ITerminalInstanceService } from 'vs/workbench/contrib/terminal/browser/terminal';
 
 export interface IProfileContextProvider {
-	getDefaultSystemShell: (remoteAuthority: string | undefined, os: OperatingSystem) => Promise<string>;
-	getEnvironment: (remoteAuthority: string | undefined) => Promise<IProcessEnvironment>;
+	getDefaultSystemShell(remoteAuthority: string | undefined, os: OperatingSystem): Promise<string>;
+	getEnvironment(remoteAuthority: string | undefined): Promise<IProcessEnvironment>;
 }
 
 const generatedProfileName = 'Generated';
 
 /*
-* Resolves terminal shell launch config and terminal
-* profiles for the given operating system,
-* environment, and user configuration
-*/
-
-const SHOULD_PROMPT_FOR_PROFILE_MIGRATION_KEY = 'terminals.integrated.profile-migration';
-
-let migrationMessageShown = false;
-
+ * Resolves terminal shell launch config and terminal profiles for the given operating system,
+ * environment, and user configuration.
+ */
 export abstract class BaseTerminalProfileResolverService implements ITerminalProfileResolverService {
 	declare _serviceBrand: undefined;
 
@@ -62,12 +51,10 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 		private readonly _configurationService: IConfigurationService,
 		private readonly _configurationResolverService: IConfigurationResolverService,
 		private readonly _historyService: IHistoryService,
-		private readonly _logService: ILogService,
+		private readonly _logService: ITerminalLogService,
 		private readonly _terminalProfileService: ITerminalProfileService,
 		private readonly _workspaceContextService: IWorkspaceContextService,
-		private readonly _remoteAgentService: IRemoteAgentService,
-		private readonly _storageService: IStorageService,
-		private readonly _notificationService: INotificationService
+		private readonly _remoteAgentService: IRemoteAgentService
 	) {
 		if (this._remoteAgentService.getConnection()) {
 			this._remoteAgentService.getEnvironment().then(env => this._primaryBackendOs = env?.os || OS);
@@ -82,7 +69,6 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 			}
 		});
 		this._terminalProfileService.onDidChangeAvailableProfiles(() => this._refreshDefaultProfileName());
-		this.showProfileMigrationNotification();
 	}
 
 	@debounce(200)
@@ -213,13 +199,6 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 			}
 		}
 
-		// If either shell or shellArgs are specified, they will take priority for now until we
-		// allow users to migrate, see https://github.com/microsoft/vscode/issues/123171
-		const shellSettingProfile = await this._getUnresolvedShellSettingDefaultProfile(options);
-		if (shellSettingProfile) {
-			return this._setIconForAutomation(options, shellSettingProfile);
-		}
-
 		// Return the real default profile if it exists and is valid, wait for profiles to be ready
 		// if the window just opened
 		await this._terminalProfileService.profilesReady;
@@ -243,65 +222,24 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 	}
 
 	private _getUnresolvedRealDefaultProfile(os: OperatingSystem): ITerminalProfile | undefined {
-		const defaultProfileName = this._configurationService.getValue(`${TerminalSettingPrefix.DefaultProfile}${this._getOsKey(os)}`);
-		if (defaultProfileName && typeof defaultProfileName === 'string') {
-			return this._terminalProfileService.availableProfiles.find(e => e.profileName === defaultProfileName);
-		}
-
-		return undefined;
-	}
-
-	private async _getUnresolvedShellSettingDefaultProfile(options: IShellLaunchConfigResolveOptions): Promise<ITerminalProfile | undefined> {
-		let executable = this._configurationService.getValue<string>(`${TerminalSettingPrefix.Shell}${this._getOsKey(options.os)}`);
-		if (!this._isValidShell(executable)) {
-			const shellArgs = this._configurationService.inspect(`${TerminalSettingPrefix.ShellArgs}${this._getOsKey(options.os)}`);
-			//  && !this.getSafeConfigValue('shellArgs', options.os, false)) {
-			if (!shellArgs.userValue && !shellArgs.workspaceValue) {
-				return undefined;
-			}
-		}
-
-		if (!executable || !this._isValidShell(executable)) {
-			executable = await this._context.getDefaultSystemShell(options.remoteAuthority, options.os);
-		}
-
-		let args: string | string[] | undefined;
-		const shellArgsSetting = this._configurationService.getValue(`${TerminalSettingPrefix.ShellArgs}${this._getOsKey(options.os)}`);
-		if (this._isValidShellArgs(shellArgsSetting, options.os)) {
-			args = shellArgsSetting;
-		}
-		if (args === undefined) {
-			if (options.os === OperatingSystem.Macintosh && args === undefined && path.parse(executable).name.match(/(zsh|bash|fish)/)) {
-				// macOS should launch a login shell by default
-				args = ['--login'];
-			} else {
-				// Resolve undefined to []
-				args = [];
-			}
-		}
-
-		const icon = this._guessProfileIcon(executable);
-
-		return {
-			profileName: generatedProfileName,
-			path: executable,
-			args,
-			icon,
-			isDefault: false
-		};
+		return this._terminalProfileService.getDefaultProfile(os);
 	}
 
 	private async _getUnresolvedFallbackDefaultProfile(options: IShellLaunchConfigResolveOptions): Promise<ITerminalProfile> {
 		const executable = await this._context.getDefaultSystemShell(options.remoteAuthority, options.os);
 
-		// Try select an existing profile to fallback to, based on the default system shell
-		let existingProfile = this._terminalProfileService.availableProfiles.find(e => path.parse(e.path).name === path.parse(executable).name);
-		if (existingProfile) {
-			if (options.allowAutomationShell) {
-				existingProfile = deepClone(existingProfile);
-				existingProfile.icon = Codicon.tools;
+		// Try select an existing profile to fallback to, based on the default system shell, only do
+		// this when it is NOT a local terminal in a remote window where the front and back end OS
+		// differs (eg. Windows -> WSL, Mac -> Linux)
+		if (options.os === OS) {
+			let existingProfile = this._terminalProfileService.availableProfiles.find(e => path.parse(e.path).name === path.parse(executable).name);
+			if (existingProfile) {
+				if (options.allowAutomationShell) {
+					existingProfile = deepClone(existingProfile);
+					existingProfile.icon = Codicon.tools;
+				}
+				return existingProfile;
 			}
-			return existingProfile;
 		}
 
 		// Finally fallback to a generated profile
@@ -326,17 +264,6 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 	}
 
 	private _getUnresolvedAutomationShellProfile(options: IShellLaunchConfigResolveOptions): ITerminalProfile | undefined {
-		const automationShell = this._configurationService.getValue(`terminal.integrated.automationShell.${this._getOsKey(options.os)}`);
-		if (automationShell && typeof automationShell === 'string') {
-			return {
-				path: automationShell,
-				profileName: generatedProfileName,
-				isDefault: false,
-				icon: Codicon.tools
-			};
-		}
-
-		// Use automationProfile second
 		const automationProfile = this._configurationService.getValue(`terminal.integrated.automationProfile.${this._getOsKey(options.os)}`);
 		if (this._isValidAutomationProfile(automationProfile, options.os)) {
 			automationProfile.icon = this._getCustomIcon(automationProfile.icon) || Codicon.tools;
@@ -347,11 +274,12 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 	}
 
 	private async _resolveProfile(profile: ITerminalProfile, options: IShellLaunchConfigResolveOptions): Promise<ITerminalProfile> {
+		const env = await this._context.getEnvironment(options.remoteAuthority);
+
 		if (options.os === OperatingSystem.Windows) {
 			// Change Sysnative to System32 if the OS is Windows but NOT WoW64. It's
 			// safe to assume that this was used by accident as Sysnative does not
 			// exist and will break the terminal in non-WoW64 environments.
-			const env = await this._context.getEnvironment(options.remoteAuthority);
 			const isWoW64 = !!env.hasOwnProperty('PROCESSOR_ARCHITEW6432');
 			const windir = env.windir;
 			if (!isWoW64 && windir) {
@@ -368,7 +296,6 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 		}
 
 		// Resolve path variables
-		const env = await this._context.getEnvironment(options.remoteAuthority);
 		const activeWorkspaceRootUri = this._historyService.getLastActiveWorkspaceRoot(options.remoteAuthority ? Schemas.vscodeRemote : Schemas.file);
 		const lastActiveWorkspace = activeWorkspaceRootUri ? withNullAsUndefined(this._workspaceContextService.getWorkspaceFolder(activeWorkspaceRootUri)) : undefined;
 		profile.path = await this._resolveVariables(profile.path, env, lastActiveWorkspace);
@@ -419,13 +346,6 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 		}
 	}
 
-	private _isValidShell(shell: unknown): shell is string {
-		if (!shell) {
-			return false;
-		}
-		return typeof shell === 'string';
-	}
-
 	private _isValidShellArgs(shellArgs: unknown, os: OperatingSystem): shellArgs is string | string[] | undefined {
 		if (shellArgs === undefined) {
 			return true;
@@ -477,48 +397,6 @@ export abstract class BaseTerminalProfileResolverService implements ITerminalPro
 		}
 		return false;
 	}
-
-	async showProfileMigrationNotification(): Promise<void> {
-		const shouldMigrateToProfile = (!!this._configurationService.getValue(TerminalSettingPrefix.Shell + this._primaryBackendOs) ||
-			!!this._configurationService.inspect(TerminalSettingPrefix.ShellArgs + this._primaryBackendOs).userValue) &&
-			!!this._configurationService.getValue(TerminalSettingPrefix.DefaultProfile + this._primaryBackendOs);
-		if (shouldMigrateToProfile && this._storageService.getBoolean(SHOULD_PROMPT_FOR_PROFILE_MIGRATION_KEY, StorageScope.WORKSPACE, true) && !migrationMessageShown) {
-			this._notificationService.prompt(
-				Severity.Info,
-				localize('terminalProfileMigration', "The terminal is using deprecated shell/shellArgs settings, do you want to migrate it to a profile?"),
-				[
-					{
-						label: localize('migrateToProfile', "Migrate"),
-						run: async () => {
-							const shell = this._configurationService.getValue(TerminalSettingPrefix.Shell + this._primaryBackendOs);
-							const shellArgs = this._configurationService.getValue(TerminalSettingPrefix.ShellArgs + this._primaryBackendOs);
-							const profile = await this.createProfileFromShellAndShellArgs(shell, shellArgs);
-							if (typeof profile === 'string') {
-								await this._configurationService.updateValue(TerminalSettingPrefix.DefaultProfile + this._primaryBackendOs, profile);
-								this._logService.trace(`migrated from shell/shellArgs, using existing profile ${profile}`);
-							} else {
-								const profiles = { ...this._configurationService.inspect<Readonly<{ [key: string]: ITerminalProfileObject }>>(TerminalSettingPrefix.Profiles + this._primaryBackendOs).userValue };
-								const profileConfig: ITerminalProfileObject = { path: profile.path };
-								if (profile.args) {
-									profileConfig.args = profile.args;
-								}
-								profiles[profile.profileName] = profileConfig;
-								await this._configurationService.updateValue(TerminalSettingPrefix.Profiles + this._primaryBackendOs, profiles);
-								await this._configurationService.updateValue(TerminalSettingPrefix.DefaultProfile + this._primaryBackendOs, profile.profileName);
-								this._logService.trace(`migrated from shell/shellArgs, ${shell} ${shellArgs} to profile ${JSON.stringify(profile)}`);
-							}
-							await this._configurationService.updateValue(TerminalSettingPrefix.Shell + this._primaryBackendOs, undefined);
-							await this._configurationService.updateValue(TerminalSettingPrefix.ShellArgs + this._primaryBackendOs, undefined);
-						}
-					} as IPromptChoice,
-				],
-				{
-					neverShowAgain: { id: SHOULD_PROMPT_FOR_PROFILE_MIGRATION_KEY, scope: NeverShowAgainScope.WORKSPACE }
-				}
-			);
-			migrationMessageShown = true;
-		}
-	}
 }
 
 export class BrowserTerminalProfileResolverService extends BaseTerminalProfileResolverService {
@@ -527,13 +405,11 @@ export class BrowserTerminalProfileResolverService extends BaseTerminalProfileRe
 		@IConfigurationResolverService configurationResolverService: IConfigurationResolverService,
 		@IConfigurationService configurationService: IConfigurationService,
 		@IHistoryService historyService: IHistoryService,
-		@ILogService logService: ILogService,
+		@ITerminalLogService logService: ITerminalLogService,
 		@ITerminalInstanceService terminalInstanceService: ITerminalInstanceService,
 		@ITerminalProfileService terminalProfileService: ITerminalProfileService,
 		@IWorkspaceContextService workspaceContextService: IWorkspaceContextService,
-		@IRemoteAgentService remoteAgentService: IRemoteAgentService,
-		@IStorageService storageService: IStorageService,
-		@INotificationService notificationService: INotificationService
+		@IRemoteAgentService remoteAgentService: IRemoteAgentService
 	) {
 		super(
 			{
@@ -559,9 +435,7 @@ export class BrowserTerminalProfileResolverService extends BaseTerminalProfileRe
 			logService,
 			terminalProfileService,
 			workspaceContextService,
-			remoteAgentService,
-			storageService,
-			notificationService
+			remoteAgentService
 		);
 	}
 }
