@@ -9,15 +9,15 @@
 
 import * as path from 'vs/base/common/path';
 import { URI } from 'vs/base/common/uri';
-import { IWorkspaceFolder } from 'vs/platform/workspace/common/workspace';
+import { IWorkspaceContextService, IWorkspaceFolder } from 'vs/platform/workspace/common/workspace';
 import { IConfigurationResolverService } from 'vs/workbench/services/configurationResolver/common/configurationResolver';
 import { sanitizeProcessEnvironment } from 'vs/base/common/processes';
-import { ILogService } from 'vs/platform/log/common/log';
-import { IShellLaunchConfig, ITerminalEnvironment, TerminalSettingId, TerminalSettingPrefix, TerminalShellType, WindowsShellType } from 'vs/platform/terminal/common/terminal';
-import { IProcessEnvironment, isWindows, language, OperatingSystem, platform, Platform } from 'vs/base/common/platform';
+import { IShellLaunchConfig, ITerminalBackend, ITerminalEnvironment, TerminalShellType, WindowsShellType } from 'vs/platform/terminal/common/terminal';
+import { IProcessEnvironment, isWindows, language, OperatingSystem } from 'vs/base/common/platform';
 import { escapeNonWindowsPath, sanitizeCwd } from 'vs/platform/terminal/common/terminalEnvironment';
-import { isString } from 'vs/base/common/types';
-import { ITerminalBackend } from 'vs/workbench/contrib/terminal/common/terminal';
+import { isString, withNullAsUndefined } from 'vs/base/common/types';
+import { IHistoryService } from 'vs/workbench/services/history/common/history';
+import { ILogService } from 'vs/platform/log/common/log';
 
 export function mergeEnvironments(parent: IProcessEnvironment, other: ITerminalEnvironment | undefined): void {
 	if (!other) {
@@ -231,21 +231,6 @@ async function _resolveCwd(cwd: string, variableResolver: VariableResolver | und
 	return cwd;
 }
 
-export type TerminalShellSetting = (
-	TerminalSettingId.AutomationShellWindows
-	| TerminalSettingId.AutomationShellMacOs
-	| TerminalSettingId.AutomationShellLinux
-	| TerminalSettingId.ShellWindows
-	| TerminalSettingId.ShellMacOs
-	| TerminalSettingId.ShellLinux
-);
-
-export type TerminalShellArgsSetting = (
-	TerminalSettingId.ShellArgsWindows
-	| TerminalSettingId.ShellArgsMacOs
-	| TerminalSettingId.ShellArgsLinux
-);
-
 export type VariableResolver = (str: string) => Promise<string>;
 
 export function createVariableResolver(lastActiveWorkspace: IWorkspaceFolder | undefined, env: IProcessEnvironment, configurationResolverService: IConfigurationResolverService | undefined): VariableResolver | undefined {
@@ -255,103 +240,6 @@ export function createVariableResolver(lastActiveWorkspace: IWorkspaceFolder | u
 	return (str) => configurationResolverService.resolveWithEnvironment(env, lastActiveWorkspace, str);
 }
 
-/**
- * @deprecated Use ITerminalProfileResolverService
- */
-export async function getDefaultShell(
-	fetchSetting: (key: TerminalShellSetting) => string | undefined,
-	defaultShell: string,
-	isWoW64: boolean,
-	windir: string | undefined,
-	variableResolver: VariableResolver | undefined,
-	logService: ILogService,
-	useAutomationShell: boolean,
-	platformOverride: Platform = platform
-): Promise<string> {
-	let maybeExecutable: string | undefined;
-	if (useAutomationShell) {
-		// If automationShell is specified, this should override the normal setting
-		maybeExecutable = getShellSetting(fetchSetting, 'automationShell', platformOverride) as string | undefined;
-	}
-	if (!maybeExecutable) {
-		maybeExecutable = getShellSetting(fetchSetting, 'shell', platformOverride) as string | undefined;
-	}
-	let executable: string = maybeExecutable || defaultShell;
-
-	// Change Sysnative to System32 if the OS is Windows but NOT WoW64. It's
-	// safe to assume that this was used by accident as Sysnative does not
-	// exist and will break the terminal in non-WoW64 environments.
-	if ((platformOverride === Platform.Windows) && !isWoW64 && windir) {
-		const sysnativePath = path.join(windir, 'Sysnative').replace(/\//g, '\\').toLowerCase();
-		if (executable && executable.toLowerCase().indexOf(sysnativePath) === 0) {
-			executable = path.join(windir, 'System32', executable.substr(sysnativePath.length + 1));
-		}
-	}
-
-	// Convert / to \ on Windows for convenience
-	if (executable && platformOverride === Platform.Windows) {
-		executable = executable.replace(/\//g, '\\');
-	}
-
-	if (variableResolver) {
-		try {
-			executable = await variableResolver(executable);
-		} catch (e) {
-			logService.error(`Could not resolve shell`, e);
-		}
-	}
-
-	return executable;
-}
-
-/**
- * @deprecated Use ITerminalProfileResolverService
- */
-export async function getDefaultShellArgs(
-	fetchSetting: (key: TerminalShellSetting | TerminalShellArgsSetting) => string | string[] | undefined,
-	useAutomationShell: boolean,
-	variableResolver: VariableResolver | undefined,
-	logService: ILogService,
-	platformOverride: Platform = platform,
-): Promise<string | string[]> {
-	if (useAutomationShell) {
-		if (!!getShellSetting(fetchSetting, 'automationShell', platformOverride)) {
-			return [];
-		}
-	}
-
-	const platformKey = platformOverride === Platform.Windows ? 'windows' : platformOverride === Platform.Mac ? 'osx' : 'linux';
-	let args = fetchSetting(<TerminalShellArgsSetting>`${TerminalSettingPrefix.ShellArgs}${platformKey}`);
-	if (!args) {
-		return [];
-	}
-	if (typeof args === 'string' && platformOverride === Platform.Windows) {
-		return variableResolver ? await variableResolver(args) : args;
-	}
-	if (variableResolver) {
-		const resolvedArgs: string[] = [];
-		for (const arg of args) {
-			try {
-				resolvedArgs.push(await variableResolver(arg));
-			} catch (e) {
-				logService.error(`Could not resolve ${TerminalSettingPrefix.ShellArgs}${platformKey}`, e);
-				resolvedArgs.push(arg);
-			}
-		}
-		args = resolvedArgs;
-	}
-	return args;
-}
-
-function getShellSetting(
-	fetchSetting: (key: TerminalShellSetting) => string | string[] | undefined,
-	type: 'automationShell' | 'shell',
-	platformOverride: Platform = platform,
-): string | string[] | undefined {
-	const platformKey = platformOverride === Platform.Windows ? 'windows' : platformOverride === Platform.Mac ? 'osx' : 'linux';
-	return fetchSetting(<TerminalShellSetting>`terminal.integrated.${type}.${platformKey}`);
-}
-
 export async function createTerminalEnvironment(
 	shellLaunchConfig: IShellLaunchConfig,
 	envFromConfig: ITerminalEnvironment | undefined,
@@ -470,3 +358,15 @@ export async function preparePathForShell(resource: string | URI, executable: st
 
 	return escapeNonWindowsPath(originalPath);
 }
+
+export function getWorkspaceForTerminal(cwd: URI | string | undefined, workspaceContextService: IWorkspaceContextService, historyService: IHistoryService): IWorkspaceFolder | undefined {
+	const cwdUri = typeof cwd === 'string' ? URI.parse(cwd) : cwd;
+	let workspaceFolder = cwdUri ? withNullAsUndefined(workspaceContextService.getWorkspaceFolder(cwdUri)) : undefined;
+	if (!workspaceFolder) {
+		// fallback to last active workspace if cwd is not available or it is not in workspace
+		// TOOD: last active workspace is known to be unreliable, we should remove this fallback eventually
+		const activeWorkspaceRootUri = historyService.getLastActiveWorkspaceRoot();
+		workspaceFolder = activeWorkspaceRootUri ? withNullAsUndefined(workspaceContextService.getWorkspaceFolder(activeWorkspaceRootUri)) : undefined;
+	}
+	return workspaceFolder;
+}
