@@ -10,16 +10,14 @@ import { URI } from 'vs/base/common/uri';
 import { IConfigurationService } from 'vs/platform/configuration/common/configuration';
 import { IInstantiationService } from 'vs/platform/instantiation/common/instantiation';
 import { ILabelService } from 'vs/platform/label/common/label';
-import { ILogService } from 'vs/platform/log/common/log';
-import { INotificationService } from 'vs/platform/notification/common/notification';
 import { Registry } from 'vs/platform/registry/common/platform';
 import { IStorageService, StorageScope, StorageTarget } from 'vs/platform/storage/common/storage';
-import { ILocalPtyService, IProcessPropertyMap, IPtyService, IShellLaunchConfig, ITerminalChildProcess, ITerminalEnvironment, ITerminalProcessOptions, ITerminalsLayoutInfo, ITerminalsLayoutInfoById, ProcessPropertyType, TerminalIpcChannels, TerminalSettingId, TitleEventSource } from 'vs/platform/terminal/common/terminal';
+import { ILocalPtyService, IProcessPropertyMap, IPtyHostLatencyMeasurement, IPtyService, IShellLaunchConfig, ITerminalBackend, ITerminalBackendRegistry, ITerminalChildProcess, ITerminalEnvironment, ITerminalLogService, ITerminalProcessOptions, ITerminalsLayoutInfo, ITerminalsLayoutInfoById, ProcessPropertyType, TerminalExtensions, TerminalIpcChannels, TerminalSettingId, TitleEventSource } from 'vs/platform/terminal/common/terminal';
 import { IGetTerminalLayoutInfoArgs, IProcessDetails, ISetTerminalLayoutInfoArgs } from 'vs/platform/terminal/common/terminalProcess';
 import { IWorkspaceContextService } from 'vs/platform/workspace/common/workspace';
 import { IWorkbenchContribution } from 'vs/workbench/common/contributions';
 import { ITerminalInstanceService } from 'vs/workbench/contrib/terminal/browser/terminal';
-import { ITerminalBackend, ITerminalBackendRegistry, ITerminalConfiguration, ITerminalProfileResolverService, TerminalExtensions, TERMINAL_CONFIG_SECTION } from 'vs/workbench/contrib/terminal/common/terminal';
+import { ITerminalConfiguration, ITerminalProfileResolverService, TERMINAL_CONFIG_SECTION } from 'vs/workbench/contrib/terminal/common/terminal';
 import { TerminalStorageKeys } from 'vs/workbench/contrib/terminal/common/terminalStorageKeys';
 import { LocalPty } from 'vs/workbench/contrib/terminal/electron-sandbox/localPty';
 import { IConfigurationResolverService } from 'vs/workbench/services/configurationResolver/common/configurationResolver';
@@ -29,14 +27,17 @@ import * as terminalEnvironment from 'vs/workbench/contrib/terminal/common/termi
 import { IProductService } from 'vs/platform/product/common/productService';
 import { IEnvironmentVariableService } from 'vs/workbench/contrib/terminal/common/environmentVariable';
 import { BaseTerminalBackend } from 'vs/workbench/contrib/terminal/browser/baseTerminalBackend';
-import { getWorkspaceForTerminal } from 'vs/workbench/services/configurationResolver/common/terminalResolver';
 import { INativeWorkbenchEnvironmentService } from 'vs/workbench/services/environment/electron-sandbox/environmentService';
 import { Client as MessagePortClient } from 'vs/base/parts/ipc/common/ipc.mp';
 import { acquirePort } from 'vs/base/parts/ipc/electron-sandbox/ipc.mp';
 import { getDelayedChannel, ProxyChannel } from 'vs/base/parts/ipc/common/ipc';
-import { mark } from 'vs/base/common/performance';
+import { mark, PerformanceMark } from 'vs/base/common/performance';
 import { ILifecycleService, LifecyclePhase } from 'vs/workbench/services/lifecycle/common/lifecycle';
 import { DeferredPromise } from 'vs/base/common/async';
+import { IStatusbarService } from 'vs/workbench/services/statusbar/browser/statusbar';
+import { memoize } from 'vs/base/common/decorators';
+import { StopWatch } from 'vs/base/common/stopwatch';
+import { IRemoteAgentService } from 'vs/workbench/services/remote/common/remoteAgentService';
 
 export class LocalTerminalBackendContribution implements IWorkbenchContribution {
 	constructor(
@@ -52,19 +53,28 @@ export class LocalTerminalBackendContribution implements IWorkbenchContribution
 class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBackend {
 	readonly remoteAuthority = undefined;
 
-	private readonly _proxy: IPtyService;
-	private readonly _clientEventually: DeferredPromise<MessagePortClient> = new DeferredPromise();
-
 	private readonly _ptys: Map<number, LocalPty> = new Map();
 
+	private _directProxyClientEventually: DeferredPromise<MessagePortClient> | undefined;
+	private _directProxy: IPtyService | undefined;
+	/**
+	 * Communicate to the direct proxy (renderer<->ptyhost) if it's available, otherwise use the
+	 * indirect proxy (renderer<->main<->ptyhost). The latter may not need to actually launch the
+	 * pty host, for example when detecting profiles.
+	 */
+	private get _proxy(): IPtyService { return this._directProxy || this._localPtyService; }
+
+	private readonly _whenReady = new DeferredPromise<void>();
+	get whenReady(): Promise<void> { return this._whenReady.p; }
+	setReady(): void { this._whenReady.complete(); }
+
 	private readonly _onDidRequestDetach = this._register(new Emitter<{ requestId: number; workspaceId: string; instanceId: number }>());
 	readonly onDidRequestDetach = this._onDidRequestDetach.event;
 
 	constructor(
-		@IInstantiationService private readonly _instantiationService: IInstantiationService,
 		@IWorkspaceContextService workspaceContextService: IWorkspaceContextService,
 		@ILifecycleService private readonly _lifecycleService: ILifecycleService,
-		@ILogService logService: ILogService,
+		@ITerminalLogService logService: ITerminalLogService,
 		@ILocalPtyService private readonly _localPtyService: ILocalPtyService,
 		@ILabelService private readonly _labelService: ILabelService,
 		@IShellEnvironmentService private readonly _shellEnvironmentService: IShellEnvironmentService,
@@ -75,47 +85,65 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 		@IHistoryService private readonly _historyService: IHistoryService,
 		@ITerminalProfileResolverService private readonly _terminalProfileResolverService: ITerminalProfileResolverService,
 		@IEnvironmentVariableService private readonly _environmentVariableService: IEnvironmentVariableService,
-		@INotificationService notificationService: INotificationService,
 		@IHistoryService historyService: IHistoryService,
-		@INativeWorkbenchEnvironmentService private readonly _environmentService: INativeWorkbenchEnvironmentService
+		@INativeWorkbenchEnvironmentService private readonly _environmentService: INativeWorkbenchEnvironmentService,
+		@IStatusbarService statusBarService: IStatusbarService,
+		@IRemoteAgentService private readonly _remoteAgentService: IRemoteAgentService,
 	) {
-		super(_localPtyService, logService, notificationService, historyService, _configurationResolverService, workspaceContextService);
+		super(_localPtyService, logService, historyService, _configurationResolverService, statusBarService, workspaceContextService);
 
-		this._proxy = ProxyChannel.toService<IPtyService>(getDelayedChannel(this._clientEventually.p.then(client => client.getChannel(TerminalIpcChannels.PtyHostWindow))));
-
-		this._connectToDirectProxy();
+		this.onPtyHostRestart(() => this._connectToDirectProxy());
 	}
 
+	/**
+	 * Request a direct connection to the pty host, this will launch the pty host process if necessary.
+	 */
 	private async _connectToDirectProxy(): Promise<void> {
-		// The pty host should not get launched until the first window restored phase
-		await this._lifecycleService.when(LifecyclePhase.Restored);
+		// Check if connecting is in progress
+		if (this._directProxyClientEventually) {
+			await this._directProxyClientEventually.p;
+			return;
+		}
 
-		mark('code/willConnectPtyHost');
+		this._logService.debug('Starting pty host');
+		const directProxyClientEventually = new DeferredPromise<MessagePortClient>();
+		this._directProxyClientEventually = directProxyClientEventually;
+		const directProxy = ProxyChannel.toService<IPtyService>(getDelayedChannel(this._directProxyClientEventually.p.then(client => client.getChannel(TerminalIpcChannels.PtyHostWindow))));
+		this._directProxy = directProxy;
+
+		// The pty host should not get launched until at least the window restored phase
+		// if remote auth exists, don't await
+		if (!this._remoteAgentService.getConnection()?.remoteAuthority) {
+			await this._lifecycleService.when(LifecyclePhase.Restored);
+		}
+
+		mark('code/terminal/willConnectPtyHost');
 		this._logService.trace('Renderer->PtyHost#connect: before acquirePort');
 		acquirePort('vscode:createPtyHostMessageChannel', 'vscode:createPtyHostMessageChannelResult').then(port => {
-			mark('code/didConnectPtyHost');
+			mark('code/terminal/didConnectPtyHost');
 			this._logService.trace('Renderer->PtyHost#connect: connection established');
 			// There are two connections to the pty host; one to the regular shared process
 			// _localPtyService, and one directly via message port _ptyHostDirectProxy. The former is
 			// used for pty host management messages, it would make sense in the future to use a
 			// separate interface/service for this one.
 			const client = new MessagePortClient(port, `window:${this._environmentService.window.id}`);
-			this._clientEventually.complete(client);
+			directProxyClientEventually.complete(client);
+			this._onPtyHostConnected.fire();
 
 			// Attach process listeners
-			this._proxy.onProcessData(e => this._ptys.get(e.id)?.handleData(e.event));
-			this._proxy.onDidChangeProperty(e => this._ptys.get(e.id)?.handleDidChangeProperty(e.property));
-			this._proxy.onProcessExit(e => {
+			directProxy.onProcessData(e => this._ptys.get(e.id)?.handleData(e.event));
+			directProxy.onDidChangeProperty(e => this._ptys.get(e.id)?.handleDidChangeProperty(e.property));
+			directProxy.onProcessExit(e => {
 				const pty = this._ptys.get(e.id);
 				if (pty) {
 					pty.handleExit(e.event);
 					this._ptys.delete(e.id);
 				}
 			});
-			this._proxy.onProcessReady(e => this._ptys.get(e.id)?.handleReady(e.event));
-			this._proxy.onProcessReplay(e => this._ptys.get(e.id)?.handleReplay(e.event));
-			this._proxy.onProcessOrphanQuestion(e => this._ptys.get(e.id)?.handleOrphanQuestion());
-			this._proxy.onDidRequestDetach(e => this._onDidRequestDetach.fire(e));
+			directProxy.onProcessReady(e => this._ptys.get(e.id)?.handleReady(e.event));
+			directProxy.onProcessReplay(e => this._ptys.get(e.id)?.handleReplay(e.event));
+			directProxy.onProcessOrphanQuestion(e => this._ptys.get(e.id)?.handleOrphanQuestion());
+			directProxy.onDidRequestDetach(e => this._onDidRequestDetach.fire(e));
 
 			// Listen for config changes
 			const initialConfig = this._configurationService.getValue<ITerminalConfiguration>(TERMINAL_CONFIG_SECTION);
@@ -123,13 +151,13 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 				// Ensure the reply is value
 				const reply = initialConfig.autoReplies[match] as string | null;
 				if (reply) {
-					this._proxy.installAutoReply(match, reply);
+					directProxy.installAutoReply(match, reply);
 				}
 			}
 			// TODO: Could simplify update to a single call
 			this._register(this._configurationService.onDidChangeConfiguration(async e => {
 				if (e.affectsConfiguration(TerminalSettingId.AutoReplies)) {
-					this._proxy.uninstallAllAutoReplies();
+					directProxy.uninstallAllAutoReplies();
 					const config = this._configurationService.getValue<ITerminalConfiguration>(TERMINAL_CONFIG_SECTION);
 					for (const match of Object.keys(config.autoReplies)) {
 						// Ensure the reply is value
@@ -140,6 +168,9 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 					}
 				}
 			}));
+
+			// Eagerly fetch the backend's environment for memoization
+			this.getEnvironment();
 		});
 	}
 
@@ -183,17 +214,19 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 		options: ITerminalProcessOptions,
 		shouldPersist: boolean
 	): Promise<ITerminalChildProcess> {
+		await this._connectToDirectProxy();
 		const executableEnv = await this._shellEnvironmentService.getShellEnv();
 		const id = await this._proxy.createProcess(shellLaunchConfig, cwd, cols, rows, unicodeVersion, env, executableEnv, options, shouldPersist, this._getWorkspaceId(), this._getWorkspaceName());
-		const pty = this._instantiationService.createInstance(LocalPty, id, shouldPersist);
+		const pty = new LocalPty(id, shouldPersist, this._proxy);
 		this._ptys.set(id, pty);
 		return pty;
 	}
 
 	async attachToProcess(id: number): Promise<ITerminalChildProcess | undefined> {
+		await this._connectToDirectProxy();
 		try {
 			await this._proxy.attachToProcess(id);
-			const pty = this._instantiationService.createInstance(LocalPty, id, true);
+			const pty = new LocalPty(id, true, this._proxy);
 			this._ptys.set(id, pty);
 			return pty;
 		} catch (e) {
@@ -203,8 +236,9 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 	}
 
 	async attachToRevivedProcess(id: number): Promise<ITerminalChildProcess | undefined> {
+		await this._connectToDirectProxy();
 		try {
-			const newId = await this._proxy.getRevivedPtyNewId(id) ?? id;
+			const newId = await this._proxy.getRevivedPtyNewId(this._getWorkspaceId(), id) ?? id;
 			return await this.attachToProcess(newId);
 		} catch (e) {
 			this._logService.warn(`Couldn't attach to process ${e.message}`);
@@ -213,9 +247,38 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 	}
 
 	async listProcesses(): Promise<IProcessDetails[]> {
+		await this._connectToDirectProxy();
 		return this._proxy.listProcesses();
 	}
 
+	async getLatency(): Promise<IPtyHostLatencyMeasurement[]> {
+		const measurements: IPtyHostLatencyMeasurement[] = [];
+		const sw = new StopWatch();
+		if (this._directProxy) {
+			await this._directProxy.getLatency();
+			sw.stop();
+			measurements.push({
+				label: 'window<->ptyhost (message port)',
+				latency: sw.elapsed()
+			});
+			sw.reset();
+		}
+		const results = await this._localPtyService.getLatency();
+		sw.stop();
+		measurements.push({
+			label: 'window<->ptyhostservice<->ptyhost',
+			latency: sw.elapsed()
+		});
+		return [
+			...measurements,
+			...results
+		];
+	}
+
+	async getPerformanceMarks(): Promise<PerformanceMark[]> {
+		return this._proxy.getPerformanceMarks();
+	}
+
 	async reduceConnectionGraceTime(): Promise<void> {
 		this._proxy.reduceConnectionGraceTime();
 	}
@@ -225,14 +288,15 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 	}
 
 	async getProfiles(profiles: unknown, defaultProfile: unknown, includeDetectedProfiles?: boolean) {
-		// TODO: Differentiate interfaces of direct to pty host and pty host service (or just move them all to pty host)
-		return this._localPtyService.getProfiles?.(this._workspaceContextService.getWorkspace().id, profiles, defaultProfile, includeDetectedProfiles) || [];
+		return this._localPtyService.getProfiles(this._workspaceContextService.getWorkspace().id, profiles, defaultProfile, includeDetectedProfiles) || [];
 	}
 
+	@memoize
 	async getEnvironment(): Promise<IProcessEnvironment> {
 		return this._proxy.getEnvironment();
 	}
 
+	@memoize
 	async getShellEnvironment(): Promise<IProcessEnvironment> {
 		return this._shellEnvironmentService.getShellEnv();
 	}
@@ -253,14 +317,13 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 	}
 
 	async getTerminalLayoutInfo(): Promise<ITerminalsLayoutInfo | undefined> {
-		const layoutArgs: IGetTerminalLayoutInfoArgs = {
-			workspaceId: this._getWorkspaceId()
-		};
+		const workspaceId = this._getWorkspaceId();
+		const layoutArgs: IGetTerminalLayoutInfoArgs = { workspaceId };
 
 		// Revive processes if needed
 		const serializedState = this._storageService.get(TerminalStorageKeys.TerminalBufferState, StorageScope.WORKSPACE);
-		const parsed = this._deserializeTerminalState(serializedState);
-		if (parsed) {
+		const reviveBufferState = this._deserializeTerminalState(serializedState);
+		if (reviveBufferState && reviveBufferState.length > 0) {
 			try {
 				// Create variable resolver
 				const activeWorkspaceRootUri = this._historyService.getLastActiveWorkspaceRoot();
@@ -269,26 +332,34 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 
 				// Re-resolve the environments and replace it on the state so local terminals use a fresh
 				// environment
-				for (const state of parsed) {
-					const freshEnv = await this._resolveEnvironmentForRevive(variableResolver, state.shellLaunchConfig);
-					state.processLaunchConfig.env = freshEnv;
-				}
+				mark('code/terminal/willGetReviveEnvironments');
+				await Promise.all(reviveBufferState.map(state => new Promise<void>(r => {
+					this._resolveEnvironmentForRevive(variableResolver, state.shellLaunchConfig).then(freshEnv => {
+						state.processLaunchConfig.env = freshEnv;
+						r();
+					});
+				})));
+				mark('code/terminal/didGetReviveEnvironments');
 
-				await this._localPtyService.reviveTerminalProcesses(parsed, Intl.DateTimeFormat().resolvedOptions().locale);
+				mark('code/terminal/willReviveTerminalProcesses');
+				await this._proxy.reviveTerminalProcesses(workspaceId, reviveBufferState, Intl.DateTimeFormat().resolvedOptions().locale);
+				mark('code/terminal/didReviveTerminalProcesses');
 				this._storageService.remove(TerminalStorageKeys.TerminalBufferState, StorageScope.WORKSPACE);
 				// If reviving processes, send the terminal layout info back to the pty host as it
 				// will not have been persisted on application exit
 				const layoutInfo = this._storageService.get(TerminalStorageKeys.TerminalLayoutInfo, StorageScope.WORKSPACE);
 				if (layoutInfo) {
-					await this._localPtyService.setTerminalLayoutInfo(JSON.parse(layoutInfo));
+					mark('code/terminal/willSetTerminalLayoutInfo');
+					await this._proxy.setTerminalLayoutInfo(JSON.parse(layoutInfo));
+					mark('code/terminal/didSetTerminalLayoutInfo');
 					this._storageService.remove(TerminalStorageKeys.TerminalLayoutInfo, StorageScope.WORKSPACE);
 				}
-			} catch {
-				// no-op
+			} catch (e: unknown) {
+				this._logService.warn('LocalTerminalBackend#getTerminalLayoutInfo Error', e && typeof e === 'object' && 'message' in e ? e.message : e);
 			}
 		}
 
-		return this._localPtyService.getTerminalLayoutInfo(layoutArgs);
+		return this._proxy.getTerminalLayoutInfo(layoutArgs);
 	}
 
 	private async _resolveEnvironmentForRevive(variableResolver: terminalEnvironment.VariableResolver | undefined, shellLaunchConfig: IShellLaunchConfig): Promise<IProcessEnvironment> {
@@ -297,7 +368,7 @@ class LocalTerminalBackend extends BaseTerminalBackend implements ITerminalBacke
 		const baseEnv = await (shellLaunchConfig.useShellEnvironment ? this.getShellEnvironment() : this.getEnvironment());
 		const env = await terminalEnvironment.createTerminalEnvironment(shellLaunchConfig, envFromConfigValue, variableResolver, this._productService.version, this._configurationService.getValue(TerminalSettingId.DetectLocale), baseEnv);
 		if (!shellLaunchConfig.strictEnv && !shellLaunchConfig.hideFromUser) {
-			const workspaceFolder = getWorkspaceForTerminal(shellLaunchConfig.cwd, this._workspaceContextService, this._historyService);
+			const workspaceFolder = terminalEnvironment.getWorkspaceForTerminal(shellLaunchConfig.cwd, this._workspaceContextService, this._historyService);
 			await this._environmentVariableService.mergedCollection.applyToProcessEnvironment(env, { workspaceFolder }, variableResolver);
 		}
 		return env;
