@@ -53,7 +53,11 @@ suite('Terminal history', () => {
 			instantiationService.set(IConfigurationService, configurationService);
 			instantiationService.set(IStorageService, storageService);
 
-			history = instantiationService.createInstance(TerminalPersistedHistory, 'test');
+			history = instantiationService.createInstance(TerminalPersistedHistory<number>, 'test');
+		});
+
+		teardown(() => {
+			instantiationService.dispose();
 		});
 
 		test('should support adding items to the cache and respect LRU', () => {
@@ -151,6 +155,10 @@ suite('Terminal history', () => {
 			} as Pick<IRemoteAgentService, 'getConnection' | 'getEnvironment'>);
 		});
 
+		teardown(() => {
+			instantiationService.dispose();
+		});
+
 		if (!isWindows) {
 			suite('local', async () => {
 				let originalEnvValues: { HOME: string | undefined };
@@ -239,6 +247,10 @@ suite('Terminal history', () => {
 			} as Pick<IRemoteAgentService, 'getConnection' | 'getEnvironment'>);
 		});
 
+		teardown(() => {
+			instantiationService.dispose();
+		});
+
 		if (!isWindows) {
 			suite('local', () => {
 				let originalEnvValues: { HOME: string | undefined };
@@ -328,6 +340,10 @@ suite('Terminal history', () => {
 			} as Pick<IRemoteAgentService, 'getConnection' | 'getEnvironment'>);
 		});
 
+		teardown(() => {
+			instantiationService.dispose();
+		});
+
 		suite('local', async () => {
 			let originalEnvValues: { HOME: string | undefined; APPDATA: string | undefined };
 			setup(() => {
@@ -433,6 +449,10 @@ suite('Terminal history', () => {
 			} as Pick<IRemoteAgentService, 'getConnection' | 'getEnvironment'>);
 		});
 
+		teardown(() => {
+			instantiationService.dispose();
+		});
+
 		if (!isWindows) {
 			suite('local', () => {
 				let originalEnvValues: { HOME: string | undefined };
