@@ -4,27 +4,29 @@
  *--------------------------------------------------------------------------------------------*/
 
 import { KeyCode, KeyMod } from 'vs/base/common/keyCodes';
-import { DisposableStore } from 'vs/base/common/lifecycle';
-import { ServicesAccessor } from 'vs/editor/browser/editorExtensions';
+import { Disposable, DisposableStore } from 'vs/base/common/lifecycle';
 import { localize } from 'vs/nls';
 import { CONTEXT_ACCESSIBILITY_MODE_ENABLED } from 'vs/platform/accessibility/common/accessibility';
 import { ContextKeyExpr } from 'vs/platform/contextkey/common/contextkey';
 import { IInstantiationService } from 'vs/platform/instantiation/common/instantiation';
 import { KeybindingWeight } from 'vs/platform/keybinding/common/keybindingsRegistry';
 import { IQuickPick, IQuickPickItem } from 'vs/platform/quickinput/common/quickInput';
-import { terminalTabFocusContextKey } from 'vs/platform/terminal/common/terminal';
-import { ITerminalContribution, ITerminalInstance, IXtermTerminal } from 'vs/workbench/contrib/terminal/browser/terminal';
+import { terminalTabFocusModeContextKey } from 'vs/platform/terminal/common/terminal';
+import { AccessibilityHelpAction } from 'vs/workbench/contrib/accessibility/browser/accessibilityContribution';
+import { IAccessibleViewService } from 'vs/workbench/contrib/accessibility/browser/accessibleView';
+import { ITerminalContribution, ITerminalInstance, ITerminalService, IXtermTerminal } from 'vs/workbench/contrib/terminal/browser/terminal';
 import { registerTerminalAction } from 'vs/workbench/contrib/terminal/browser/terminalActions';
 import { registerTerminalContribution } from 'vs/workbench/contrib/terminal/browser/terminalExtensions';
 import { TerminalWidgetManager } from 'vs/workbench/contrib/terminal/browser/widgets/widgetManager';
 import { ITerminalProcessManager, TerminalCommandId } from 'vs/workbench/contrib/terminal/common/terminal';
 import { TerminalContextKeys } from 'vs/workbench/contrib/terminal/common/terminalContextKey';
-import { AccessibilityHelpWidget } from 'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp';
+import { TerminalAccessibleContentProvider } from 'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibilityHelp';
 import { AccessibleBufferWidget, NavigationType } from 'vs/workbench/contrib/terminalContrib/accessibility/browser/terminalAccessibleBuffer';
-import { Terminal } from 'xterm';
+import type { Terminal } from 'xterm';
 
 class AccessibleBufferContribution extends DisposableStore implements ITerminalContribution {
-	static readonly ID: 'terminal.accessible-buffer';
+	static readonly ID = 'terminal.accessible-buffer';
+	private _xterm: IXtermTerminal & { raw: Terminal } | undefined;
 	static get(instance: ITerminalInstance): AccessibleBufferContribution | null {
 		return instance.getContribution<AccessibleBufferContribution>(AccessibleBufferContribution.ID);
 	}
@@ -39,12 +41,16 @@ class AccessibleBufferContribution extends DisposableStore implements ITerminalC
 		super();
 	}
 	layout(xterm: IXtermTerminal & { raw: Terminal }): void {
-		if (!this._accessibleBufferWidget) {
-			this._accessibleBufferWidget = this.add(this._instantiationService.createInstance(AccessibleBufferWidget, this._instance, xterm));
-		}
+		this._xterm = xterm;
 	}
 	async show(): Promise<void> {
-		await this._accessibleBufferWidget?.show();
+		if (!this._xterm) {
+			return;
+		}
+		if (!this._accessibleBufferWidget) {
+			this._accessibleBufferWidget = this.add(this._instantiationService.createInstance(AccessibleBufferWidget, this._instance, this._xterm));
+		}
+		await this._accessibleBufferWidget.show();
 	}
 
 	async createCommandQuickPick(): Promise<IQuickPick<IQuickPickItem> | undefined> {
@@ -57,29 +63,26 @@ class AccessibleBufferContribution extends DisposableStore implements ITerminalC
 }
 registerTerminalContribution(AccessibleBufferContribution.ID, AccessibleBufferContribution);
 
-registerTerminalAction({
-	id: TerminalCommandId.ShowTerminalAccessibilityHelp,
-	title: { value: localize('workbench.action.terminal.showAccessibilityHelp', "Show Terminal Accessibility Help"), original: 'Show Terminal Accessibility Help' },
-	keybinding: {
-		primary: KeyMod.Alt | KeyCode.F1,
-		weight: KeybindingWeight.WorkbenchContrib,
-		linux: {
-			primary: KeyMod.Alt | KeyMod.Shift | KeyCode.F1,
-			secondary: [KeyMod.Alt | KeyCode.F1]
-		},
-		when: TerminalContextKeys.focus
-	},
-	run: async (c, accessor: ServicesAccessor) => {
-		const instantiationService = accessor.get(IInstantiationService);
-		const instance = await c.service.getActiveOrCreateInstance();
-		await c.service.revealActiveTerminal();
-		const terminal = instance?.xterm;
-		if (!terminal) {
-			return;
-		}
-		await instantiationService.createInstance(AccessibilityHelpWidget, instance, terminal).show();
+export class TerminalAccessibilityHelpContribution extends Disposable {
+	static ID: 'terminalAccessibilityHelpContribution';
+	constructor() {
+		super();
+
+		this._register(AccessibilityHelpAction.addImplementation(105, 'terminal', async accessor => {
+			const instantiationService = accessor.get(IInstantiationService);
+			const terminalService = accessor.get(ITerminalService);
+			const accessibleViewService = accessor.get(IAccessibleViewService);
+			const instance = await terminalService.getActiveOrCreateInstance();
+			await terminalService.revealActiveTerminal();
+			const terminal = instance?.xterm;
+			if (!terminal) {
+				return;
+			}
+			accessibleViewService.show(instantiationService.createInstance(TerminalAccessibleContentProvider, instance, terminal));
+		}, TerminalContextKeys.focus));
 	}
-});
+}
+registerTerminalContribution(TerminalAccessibilityHelpContribution.ID, TerminalAccessibilityHelpContribution);
 
 registerTerminalAction({
 	id: TerminalCommandId.FocusAccessibleBuffer,
@@ -89,7 +92,7 @@ registerTerminalAction({
 		{
 			primary: KeyMod.Shift | KeyCode.Tab,
 			weight: KeybindingWeight.WorkbenchContrib,
-			when: ContextKeyExpr.and(CONTEXT_ACCESSIBILITY_MODE_ENABLED, terminalTabFocusContextKey, TerminalContextKeys.accessibleBufferFocus.negate())
+			when: ContextKeyExpr.and(CONTEXT_ACCESSIBILITY_MODE_ENABLED, TerminalContextKeys.focus, ContextKeyExpr.or(terminalTabFocusModeContextKey, TerminalContextKeys.accessibleBufferFocus.negate()))
 		}
 	],
 	run: async (c) => {
@@ -108,7 +111,7 @@ registerTerminalAction({
 	precondition: ContextKeyExpr.or(TerminalContextKeys.processSupported, TerminalContextKeys.terminalHasBeenCreated),
 	keybinding: [
 		{
-			primary: KeyMod.CtrlCmd | KeyMod.Shift | KeyCode.KeyO,
+			primary: KeyMod.CtrlCmd | KeyMod.Shift | KeyCode.KeyG,
 			weight: KeybindingWeight.WorkbenchContrib + 2,
 			when: TerminalContextKeys.accessibleBufferFocus
 		}
@@ -131,8 +134,14 @@ registerTerminalAction({
 	keybinding: [
 		{
 			primary: KeyMod.CtrlCmd | KeyCode.DownArrow,
-			weight: KeybindingWeight.WorkbenchContrib + 2,
-			when: TerminalContextKeys.accessibleBufferFocus
+			when: ContextKeyExpr.and(TerminalContextKeys.accessibleBufferFocus, CONTEXT_ACCESSIBILITY_MODE_ENABLED.negate()),
+			weight: KeybindingWeight.WorkbenchContrib + 2
+		},
+		{
+			primary: KeyMod.CtrlCmd | KeyCode.DownArrow,
+			mac: { primary: KeyMod.Alt | KeyCode.DownArrow },
+			when: ContextKeyExpr.and(TerminalContextKeys.accessibleBufferFocus, CONTEXT_ACCESSIBILITY_MODE_ENABLED),
+			weight: KeybindingWeight.WorkbenchContrib + 2
 		}
 	],
 	run: async (c) => {
@@ -153,8 +162,14 @@ registerTerminalAction({
 	keybinding: [
 		{
 			primary: KeyMod.CtrlCmd | KeyCode.UpArrow,
-			weight: KeybindingWeight.WorkbenchContrib + 2,
-			when: TerminalContextKeys.accessibleBufferFocus
+			when: ContextKeyExpr.and(TerminalContextKeys.accessibleBufferFocus, CONTEXT_ACCESSIBILITY_MODE_ENABLED.negate()),
+			weight: KeybindingWeight.WorkbenchContrib + 2
+		},
+		{
+			primary: KeyMod.CtrlCmd | KeyCode.UpArrow,
+			mac: { primary: KeyMod.Alt | KeyCode.UpArrow },
+			when: ContextKeyExpr.and(TerminalContextKeys.accessibleBufferFocus, CONTEXT_ACCESSIBILITY_MODE_ENABLED),
+			weight: KeybindingWeight.WorkbenchContrib + 2
 		}
 	],
 	run: async (c) => {
