@@ -20,7 +20,7 @@ const element = document.createElement('div');
 element.style.height = '1000px';
 element.style.width = '200px';
 
-export class TestObjectTree<T> extends ObjectTree<T, any> {
+class TestObjectTree<T> extends ObjectTree<T, any> {
 	constructor(serializer: (node: T) => string, sorter?: ITreeSorter<T>) {
 		super(
 			'test',
@@ -89,7 +89,7 @@ class ByLabelTreeSorter implements ITreeSorter<TestExplorerTreeElement> {
 			}
 		}
 
-		return (a.sortText || a.label).localeCompare(b.sortText || b.label);
+		return (a.test.item.sortText || a.test.item.label).localeCompare(b.test.item.sortText || b.test.item.label);
 	}
 }
 
@@ -120,7 +120,7 @@ export class TestTreeTestHarness<T extends ITestTreeProjection = ITestTreeProjec
 			onDidProcessDiff: this.onDiff.event,
 		} as any));
 		const sorter = new ByLabelTreeSorter();
-		this.tree = this._register(new TestObjectTree(t => 'label' in t ? t.label : t.message.toString(), sorter));
+		this.tree = this._register(new TestObjectTree(t => 'test' in t ? t.test.item.label : t.message.toString(), sorter));
 		this._register(this.tree.onDidChangeCollapseState(evt => {
 			if (evt.node.element instanceof TestItemTreeElement) {
 				this.projection.expandElement(evt.node.element, evt.deep ? Infinity : 0);
