@@ -5,18 +5,15 @@
 
 import { Disposable, DisposableStore, IDisposable, MutableDisposable } from 'vs/base/common/lifecycle';
 import { isWeb } from 'vs/base/common/platform';
-import { Event } from 'vs/base/common/event';
 import { ServicesAccessor } from 'vs/editor/browser/editorExtensions';
 import { localize } from 'vs/nls';
 import { Action2, IMenuService, ISubmenuItem, MenuId, MenuRegistry, registerAction2 } from 'vs/platform/actions/common/actions';
-import { ICommandService } from 'vs/platform/commands/common/commands';
 import { ContextKeyExpr, IContextKey, IContextKeyService } from 'vs/platform/contextkey/common/contextkey';
 import { IUserDataProfile, IUserDataProfilesService } from 'vs/platform/userDataProfile/common/userDataProfile';
 import { IWorkbenchContribution } from 'vs/workbench/common/contributions';
-import { RenameProfileAction } from 'vs/workbench/contrib/userDataProfile/browser/userDataProfileActions';
 import { ILifecycleService, LifecyclePhase } from 'vs/workbench/services/lifecycle/common/lifecycle';
-import { CURRENT_PROFILE_CONTEXT, HAS_PROFILES_CONTEXT, IS_CURRENT_PROFILE_TRANSIENT_CONTEXT, IS_PROFILE_IMPORT_IN_PROGRESS_CONTEXT, IUserDataProfileImportExportService, IUserDataProfileManagementService, IUserDataProfileService, PROFILES_CATEGORY, PROFILE_FILTER, IS_PROFILE_EXPORT_IN_PROGRESS_CONTEXT, ProfilesMenu, PROFILES_ENABLEMENT_CONTEXT, PROFILES_TITLE } from 'vs/workbench/services/userDataProfile/common/userDataProfile';
-import { IQuickInputService, IQuickPickItem, QuickPickItem } from 'vs/platform/quickinput/common/quickInput';
+import { CURRENT_PROFILE_CONTEXT, HAS_PROFILES_CONTEXT, IS_CURRENT_PROFILE_TRANSIENT_CONTEXT, IS_PROFILE_IMPORT_IN_PROGRESS_CONTEXT, IUserDataProfileImportExportService, IUserDataProfileManagementService, IUserDataProfileService, PROFILES_CATEGORY, PROFILE_FILTER, IS_PROFILE_EXPORT_IN_PROGRESS_CONTEXT, ProfilesMenu, PROFILES_ENABLEMENT_CONTEXT, PROFILES_TITLE, IProfileTemplateInfo } from 'vs/workbench/services/userDataProfile/common/userDataProfile';
+import { IQuickInputService, IQuickPickItem, IQuickPickSeparator } from 'vs/platform/quickinput/common/quickInput';
 import { INotificationService } from 'vs/platform/notification/common/notification';
 import { IFileDialogService } from 'vs/platform/dialogs/common/dialogs';
 import { URI } from 'vs/base/common/uri';
@@ -26,27 +23,6 @@ import { IWorkspaceTagsService } from 'vs/workbench/contrib/tags/common/workspac
 import { getErrorMessage } from 'vs/base/common/errors';
 import { Categories } from 'vs/platform/action/common/actionCommonCategories';
 import { IOpenerService } from 'vs/platform/opener/common/opener';
-import { IProductService } from 'vs/platform/product/common/productService';
-import { IRequestService, asJson } from 'vs/platform/request/common/request';
-import { CancellationToken } from 'vs/base/common/cancellation';
-import { ILogService } from 'vs/platform/log/common/log';
-
-const CREATE_EMPTY_PROFILE_ACTION_ID = 'workbench.profiles.actions.createEmptyProfile';
-const CREATE_EMPTY_PROFILE_ACTION_TITLE = {
-	value: localize('create empty profile', "Create an Empty Profile..."),
-	original: 'Create an Empty Profile...'
-};
-
-const CREATE_FROM_CURRENT_PROFILE_ACTION_ID = 'workbench.profiles.actions.createFromCurrentProfile';
-const CREATE_FROM_CURRENT_PROFILE_ACTION_TITLE = {
-	value: localize('save profile as', "Create from Current Profile..."),
-	original: 'Create from Current Profile...'
-};
-
-interface IProfileTemplateInfo {
-	readonly name: string;
-	readonly url: string;
-}
 
 type IProfileTemplateQuickPickItem = IQuickPickItem & IProfileTemplateInfo;
 
@@ -65,12 +41,7 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 		@IWorkspaceContextService private readonly workspaceContextService: IWorkspaceContextService,
 		@IWorkspaceTagsService private readonly workspaceTagsService: IWorkspaceTagsService,
 		@IContextKeyService contextKeyService: IContextKeyService,
-		@IQuickInputService private readonly quickInputService: IQuickInputService,
-		@INotificationService private readonly notificationService: INotificationService,
 		@ILifecycleService private readonly lifecycleService: ILifecycleService,
-		@IProductService private readonly productService: IProductService,
-		@IRequestService private readonly requestService: IRequestService,
-		@ILogService private readonly logService: ILogService,
 	) {
 		super();
 
@@ -106,13 +77,11 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 		this._register(this.userDataProfilesService.onDidChangeProfiles(() => this.registerProfilesActions()));
 
 		this.registerCurrentProfilesActions();
-		this._register(Event.any(this.userDataProfileService.onDidChangeCurrentProfile, this.userDataProfileService.onDidUpdateCurrentProfile)(() => this.registerCurrentProfilesActions()));
+		this._register(this.userDataProfileService.onDidChangeCurrentProfile(() => this.registerCurrentProfilesActions()));
 
-		this.registerCreateEmptyProfileAction();
 		this.registerCreateFromCurrentProfileAction();
 		this.registerCreateProfileAction();
 		this.registerDeleteProfileAction();
-		this.registerCreateProfileFromTemplatesAction();
 
 		this.registerHelpAction();
 	}
@@ -207,22 +176,23 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 	private readonly currentprofileActionsDisposable = this._register(new MutableDisposable<DisposableStore>());
 	private registerCurrentProfilesActions(): void {
 		this.currentprofileActionsDisposable.value = new DisposableStore();
-		this.currentprofileActionsDisposable.value.add(this.registerRenameCurrentProfileAction());
+		this.currentprofileActionsDisposable.value.add(this.registerEditCurrentProfileAction());
 		this.currentprofileActionsDisposable.value.add(this.registerShowCurrentProfileContentsAction());
 		this.currentprofileActionsDisposable.value.add(this.registerExportCurrentProfileAction());
 		this.currentprofileActionsDisposable.value.add(this.registerImportProfileAction());
 	}
 
-	private registerRenameCurrentProfileAction(): IDisposable {
+	private registerEditCurrentProfileAction(): IDisposable {
 		const that = this;
 		return registerAction2(class RenameCurrentProfileAction extends Action2 {
 			constructor() {
 				super({
-					id: `workbench.profiles.actions.renameCurrentProfile`,
+					id: `workbench.profiles.actions.editCurrentProfile`,
 					title: {
-						value: localize('rename profile', "Rename..."),
-						original: `Rename...`
+						value: localize('edit profile', "Edit Profile..."),
+						original: `Edit Profile...`
 					},
+					f1: true,
 					menu: [
 						{
 							id: ProfilesMenu,
@@ -233,8 +203,8 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 					]
 				});
 			}
-			async run(accessor: ServicesAccessor) {
-				accessor.get(ICommandService).executeCommand(RenameProfileAction.ID, that.userDataProfileService.currentProfile);
+			run() {
+				return that.userDataProfileImportExportService.editProfile(that.userDataProfileService.currentProfile);
 			}
 		});
 	}
@@ -246,8 +216,8 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 				super({
 					id,
 					title: {
-						value: localize('show profile contents', "Show Contents..."),
-						original: `ShowContents...`
+						value: localize('show profile contents', "Show Profile Contents"),
+						original: `Show Profile Contents`
 					},
 					category: PROFILES_CATEGORY,
 					menu: [
@@ -316,6 +286,7 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 	private registerImportProfileAction(): IDisposable {
 		const disposables = new DisposableStore();
 		const id = 'workbench.profiles.actions.importProfile';
+		const that = this;
 		disposables.add(registerAction2(class ImportProfileAction extends Action2 {
 			constructor() {
 				super({
@@ -348,21 +319,41 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 
 				const disposables = new DisposableStore();
 				const quickPick = disposables.add(quickInputService.createQuickPick());
+				const profileTemplateQuickPickItems = await that.getProfileTemplatesQuickPickItems();
+
 				const updateQuickPickItems = (value?: string) => {
-					const selectFromFileItem: IQuickPickItem = { label: localize('import from file', "Create from profile template file") };
-					quickPick.items = value ? [{ label: localize('import from url', "Create from profile template URL"), description: quickPick.value }, selectFromFileItem] : [selectFromFileItem];
+					const quickPickItems: (IQuickPickItem | IQuickPickSeparator)[] = [];
+					if (value) {
+						quickPickItems.push({ label: quickPick.value, description: localize('import from url', "Import from URL") });
+					}
+					quickPickItems.push({ label: localize('import from file', "Select File...") });
+					if (profileTemplateQuickPickItems.length) {
+						quickPickItems.push({
+							type: 'separator',
+							label: localize('templates', "Profile Templates")
+						}, ...profileTemplateQuickPickItems);
+					}
+					quickPick.items = quickPickItems;
 				};
-				quickPick.title = localize('import profile quick pick title', "Create Profile from Profile Template...");
-				quickPick.placeholder = localize('import profile placeholder', "Provide profile template URL or select profile template file");
+
+				quickPick.title = localize('import profile quick pick title', "Import from Profile Template...");
+				quickPick.placeholder = localize('import profile placeholder', "Provide Profile Template URL");
 				quickPick.ignoreFocusOut = true;
 				disposables.add(quickPick.onDidChangeValue(updateQuickPickItems));
 				updateQuickPickItems();
 				quickPick.matchOnLabel = false;
 				quickPick.matchOnDescription = false;
 				disposables.add(quickPick.onDidAccept(async () => {
+					quickPick.hide();
+					const selectedItem = quickPick.selectedItems[0];
+					if (!selectedItem) {
+						return;
+					}
 					try {
-						quickPick.hide();
-						const profile = quickPick.selectedItems[0].description ? URI.parse(quickPick.value) : await this.getProfileUriFromFileSystem(fileDialogService);
+						if ((<IProfileTemplateQuickPickItem>selectedItem).url) {
+							return await that.userDataProfileImportExportService.createProfile(URI.parse((<IProfileTemplateQuickPickItem>selectedItem).url));
+						}
+						const profile = selectedItem.label === quickPick.value ? URI.parse(quickPick.value) : await this.getProfileUriFromFileSystem(fileDialogService);
 						if (profile) {
 							await userDataProfileImportExportService.importProfile(profile);
 						}
@@ -406,8 +397,11 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 		this._register(registerAction2(class CreateFromCurrentProfileAction extends Action2 {
 			constructor() {
 				super({
-					id: CREATE_FROM_CURRENT_PROFILE_ACTION_ID,
-					title: CREATE_FROM_CURRENT_PROFILE_ACTION_TITLE,
+					id: 'workbench.profiles.actions.createFromCurrentProfile',
+					title: {
+						value: localize('save profile as', "Save Current Profile As..."),
+						original: 'Save Current Profile As...'
+					},
 					category: PROFILES_CATEGORY,
 					f1: true,
 					precondition: PROFILES_ENABLEMENT_CONTEXT
@@ -415,55 +409,11 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 			}
 
 			run(accessor: ServicesAccessor) {
-				return that.createProfile(true);
+				return that.userDataProfileImportExportService.createProfile(that.userDataProfileService.currentProfile);
 			}
 		}));
 	}
 
-	private registerCreateEmptyProfileAction(): void {
-		const that = this;
-		this._register(registerAction2(class CreateEmptyProfileAction extends Action2 {
-			constructor() {
-				super({
-					id: CREATE_EMPTY_PROFILE_ACTION_ID,
-					title: CREATE_EMPTY_PROFILE_ACTION_TITLE,
-					category: PROFILES_CATEGORY,
-					f1: true,
-					precondition: PROFILES_ENABLEMENT_CONTEXT
-				});
-			}
-
-			run(accessor: ServicesAccessor) {
-				return that.createProfile(false);
-			}
-		}));
-	}
-
-	private async createProfile(fromExisting: boolean): Promise<void> {
-		const name = await this.quickInputService.input({
-			placeHolder: localize('name', "Profile name"),
-			title: fromExisting ? localize('create from current profle', "Create from Current Profile...") : localize('create empty profile', "Create an Empty Profile..."),
-			validateInput: async (value: string) => {
-				if (this.userDataProfilesService.profiles.some(p => p.name === value)) {
-					return localize('profileExists', "Profile with name {0} already exists.", value);
-				}
-				return undefined;
-			}
-		});
-		if (!name) {
-			return;
-		}
-		try {
-			if (fromExisting) {
-				await this.userDataProfileImportExportService.createFromCurrentProfile(name);
-			} else {
-				await this.userDataProfileManagementService.createAndEnterProfile(name, undefined, false);
-			}
-		} catch (error) {
-			this.notificationService.error(error);
-		}
-	}
-
 	private registerCreateProfileAction(): void {
 		const that = this;
 		this._register(registerAction2(class CreateProfileAction extends Action2 {
@@ -476,6 +426,7 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 					},
 					category: PROFILES_CATEGORY,
 					precondition: PROFILES_ENABLEMENT_CONTEXT,
+					f1: true,
 					menu: [
 						{
 							id: ProfilesMenu,
@@ -488,47 +439,7 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 			}
 
 			async run(accessor: ServicesAccessor) {
-				const quickInputService = accessor.get(IQuickInputService);
-				const commandService = accessor.get(ICommandService);
-				const userDataProfileImportExportService = accessor.get(IUserDataProfileImportExportService);
-				const quickPickItems: QuickPickItem[] = [{
-					id: CREATE_EMPTY_PROFILE_ACTION_ID,
-					label: localize('empty', "Empty Profile"),
-				}, {
-					id: CREATE_FROM_CURRENT_PROFILE_ACTION_ID,
-					label: localize('using current', "Using Current Profile"),
-				}];
-				const profileTemplateQuickPickItems = await that.getProfileTemplatesQuickPickItems();
-				if (profileTemplateQuickPickItems.length) {
-					quickPickItems.push({
-						type: 'separator',
-						label: localize('templates', "Profile Templates")
-					}, ...profileTemplateQuickPickItems);
-				}
-				const pick = await quickInputService.pick(quickPickItems,
-					{
-						hideInput: true,
-						canPickMany: false,
-						title: localize('create profile title', "Create Profile...")
-					});
-				if (pick) {
-					if (pick.id) {
-						return commandService.executeCommand(pick.id);
-					}
-					if ((<IProfileTemplateQuickPickItem>pick).url) {
-						type ProfileCreationFromTemplateActionClassification = {
-							owner: 'sandy081';
-							comment: 'Report profile creation from template action';
-							profileName: { classification: 'SystemMetaData'; purpose: 'FeatureInsight'; comment: 'Name of the profile created from template' };
-						};
-						type ProfileCreationFromTemplateActionEvent = {
-							profileName: string;
-						};
-						that.telemetryService.publicLog2<ProfileCreationFromTemplateActionEvent, ProfileCreationFromTemplateActionClassification>('profileCreationAction:builtinTemplate', { profileName: (<IProfileTemplateQuickPickItem>pick).name });
-						const uri = URI.parse((<IProfileTemplateQuickPickItem>pick).url);
-						return userDataProfileImportExportService.importProfile(uri);
-					}
-				}
+				return that.userDataProfileImportExportService.createProfile();
 			}
 		}));
 	}
@@ -588,44 +499,6 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 		});
 	}
 
-	private registerCreateProfileFromTemplatesAction(): void {
-		const that = this;
-		this._register(registerAction2(class CreateProfileFromTemplatesAction extends Action2 {
-			constructor() {
-				super({
-					id: 'workbench.profiles.actions.createProfileFromTemplates',
-					title: {
-						value: localize('create profile from templates', "Create Profile from Templates..."),
-						original: 'Create Profile from Templates...'
-					},
-					category: PROFILES_CATEGORY,
-					precondition: PROFILES_ENABLEMENT_CONTEXT,
-				});
-			}
-
-			async run(accessor: ServicesAccessor) {
-				const quickInputService = accessor.get(IQuickInputService);
-				const userDataProfileImportExportService = accessor.get(IUserDataProfileImportExportService);
-				const notificationService = accessor.get(INotificationService);
-				const profileTemplateQuickPickItems = await that.getProfileTemplatesQuickPickItems();
-				if (profileTemplateQuickPickItems.length) {
-					const pick = await quickInputService.pick(profileTemplateQuickPickItems,
-						{
-							hideInput: true,
-							canPickMany: false,
-							title: localize('create profile from template title', "{0}: Create...", PROFILES_CATEGORY.value)
-						});
-					if ((<IProfileTemplateQuickPickItem>pick)?.url) {
-						const uri = URI.parse((<IProfileTemplateQuickPickItem>pick).url);
-						return userDataProfileImportExportService.importProfile(uri);
-					}
-				} else {
-					notificationService.info(localize('no templates', "There are no templates to create from"));
-				}
-			}
-		}));
-	}
-
 	private registerHelpAction(): void {
 		this._register(registerAction2(class HelpAction extends Action2 {
 			constructor() {
@@ -646,7 +519,7 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 
 	private async getProfileTemplatesQuickPickItems(): Promise<IProfileTemplateQuickPickItem[]> {
 		const quickPickItems: IProfileTemplateQuickPickItem[] = [];
-		const profileTemplates = await this.getProfileTemplatesFromProduct();
+		const profileTemplates = await this.userDataProfileManagementService.getBuiltinProfileTemplates();
 		for (const template of profileTemplates) {
 			quickPickItems.push({
 				label: template.name,
@@ -656,22 +529,6 @@ export class UserDataProfilesWorkbenchContribution extends Disposable implements
 		return quickPickItems;
 	}
 
-	private async getProfileTemplatesFromProduct(): Promise<IProfileTemplateInfo[]> {
-		if (this.productService.profileTemplatesUrl) {
-			try {
-				const context = await this.requestService.request({ type: 'GET', url: this.productService.profileTemplatesUrl }, CancellationToken.None);
-				if (context.res.statusCode === 200) {
-					return (await asJson<IProfileTemplateInfo[]>(context)) || [];
-				} else {
-					this.logService.error('Could not get profile templates.', context.res.statusCode);
-				}
-			} catch (error) {
-				this.logService.error(error);
-			}
-		}
-		return [];
-	}
-
 	private async reportWorkspaceProfileInfo(): Promise<void> {
 		await this.lifecycleService.when(LifecyclePhase.Eventually);
 		const workspaceId = await this.workspaceTagsService.getTelemetryWorkspaceId(this.workspaceContextService.getWorkspace(), this.workspaceContextService.getWorkbenchState());
