@@ -3,13 +3,14 @@
  *  Licensed under the MIT License. See License.txt in the project root for license information.
  *--------------------------------------------------------------------------------------------*/
 
+import { localize } from 'vs/nls';
 import { createDecorator } from 'vs/platform/instantiation/common/instantiation';
 import { InstantiationType, registerSingleton } from 'vs/platform/instantiation/common/extensions';
 import { Event, Emitter } from 'vs/base/common/event';
 import { Disposable } from 'vs/base/common/lifecycle';
 import { RawContextKey, IContextKey, IContextKeyService } from 'vs/platform/contextkey/common/contextkey';
 import { IConfigurationService } from 'vs/platform/configuration/common/configuration';
-import { IFilesConfiguration, AutoSaveConfiguration, HotExitConfiguration, FILES_READONLY_INCLUDE_CONFIG, FILES_READONLY_EXCLUDE_CONFIG, IFileStatWithMetadata, IFileService, FileSystemProviderCapabilities, IBaseFileStat } from 'vs/platform/files/common/files';
+import { IFilesConfiguration, AutoSaveConfiguration, HotExitConfiguration, FILES_READONLY_INCLUDE_CONFIG, FILES_READONLY_EXCLUDE_CONFIG, IFileStatWithMetadata, IFileService, IBaseFileStat, hasReadonlyCapability } from 'vs/platform/files/common/files';
 import { equals } from 'vs/base/common/objects';
 import { URI } from 'vs/base/common/uri';
 import { isWeb } from 'vs/base/common/platform';
@@ -20,13 +21,14 @@ import { IUriIdentityService } from 'vs/platform/uriIdentity/common/uriIdentity'
 import { IEnvironmentService } from 'vs/platform/environment/common/environment';
 import { ResourceMap } from 'vs/base/common/map';
 import { withNullAsUndefined } from 'vs/base/common/types';
+import { IMarkdownString } from 'vs/base/common/htmlContent';
 
 export const AutoSaveAfterShortDelayContext = new RawContextKey<boolean>('autoSaveAfterShortDelayContext', false, true);
 
 export interface IAutoSaveConfiguration {
-	autoSaveDelay?: number;
-	autoSaveFocusChange: boolean;
-	autoSaveApplicationChange: boolean;
+	readonly autoSaveDelay?: number;
+	readonly autoSaveFocusChange: boolean;
+	readonly autoSaveApplicationChange: boolean;
 }
 
 export const enum AutoSaveMode {
@@ -59,7 +61,7 @@ export interface IFilesConfigurationService {
 
 	readonly onReadonlyChange: Event<void>;
 
-	isReadonly(resource: URI, stat?: IBaseFileStat): boolean;
+	isReadonly(resource: URI, stat?: IBaseFileStat): boolean | IMarkdownString;
 
 	updateReadonly(resource: URI, readonly: true | false | 'toggle' | 'reset'): Promise<void>;
 
@@ -78,7 +80,15 @@ export class FilesConfigurationService extends Disposable implements IFilesConfi
 
 	declare readonly _serviceBrand: undefined;
 
-	private static DEFAULT_AUTO_SAVE_MODE = isWeb ? AutoSaveConfiguration.AFTER_DELAY : AutoSaveConfiguration.OFF;
+	private static readonly DEFAULT_AUTO_SAVE_MODE = isWeb ? AutoSaveConfiguration.AFTER_DELAY : AutoSaveConfiguration.OFF;
+
+	private static readonly READONLY_MESSAGES = {
+		providerReadonly: { value: localize('providerReadonly', "Editor is read-only because the file system of the file is read-only."), isTrusted: true },
+		sessionReadonly: { value: localize({ key: 'sessionReadonly', comment: ['Please do not translate the word "command", it is part of our internal syntax which must not change', '{Locked="](command:{0})"}'] }, "Editor is read-only because the file was set read-only in this session. [Click here](command:{0}) to set writeable.", 'workbench.action.files.setActiveEditorWriteableInSession'), isTrusted: true },
+		configuredReadonly: { value: localize({ key: 'configuredReadonly', comment: ['Please do not translate the word "command", it is part of our internal syntax which must not change', '{Locked="](command:{0})"}'] }, "Editor is read-only because the file was set read-only via settings. [Click here](command:{0}) to configure.", `workbench.action.openSettings?${encodeURIComponent('["files.readonly"]')}`), isTrusted: true },
+		fileLocked: { value: localize({ key: 'fileLocked', comment: ['Please do not translate the word "command", it is part of our internal syntax which must not change', '{Locked="](command:{0})"}'] }, "Editor is read-only because of file permissions. [Click here](command:{0}) to set writeable anyway.", 'workbench.action.files.setActiveEditorWriteableInSession'), isTrusted: true },
+		fileReadonly: { value: localize('fileReadonly', "Editor is read-only because the file is read-only."), isTrusted: true }
+	};
 
 	private readonly _onAutoSaveConfigurationChange = this._register(new Emitter<IAutoSaveConfiguration>());
 	readonly onAutoSaveConfigurationChange = this._onAutoSaveConfigurationChange.event;
@@ -140,19 +150,20 @@ export class FilesConfigurationService extends Disposable implements IFilesConfi
 		return matcher;
 	}
 
-	isReadonly(resource: URI, stat?: IBaseFileStat): boolean {
+	isReadonly(resource: URI, stat?: IBaseFileStat): boolean | IMarkdownString {
 
 		// if the entire file system provider is readonly, we respect that
 		// and do not allow to change readonly. we take this as a hint that
 		// the provider has no capabilities of writing.
-		if (this.fileService.hasCapability(resource, FileSystemProviderCapabilities.Readonly)) {
-			return true;
+		const provider = this.fileService.getProvider(resource.scheme);
+		if (provider && hasReadonlyCapability(provider)) {
+			return provider.readOnlyMessage ?? FilesConfigurationService.READONLY_MESSAGES.providerReadonly;
 		}
 
 		// session override always wins over the others
 		const sessionReadonlyOverride = this.sessionReadonlyOverrides.get(resource);
 		if (typeof sessionReadonlyOverride === 'boolean') {
-			return sessionReadonlyOverride;
+			return sessionReadonlyOverride === true ? FilesConfigurationService.READONLY_MESSAGES.sessionReadonly : false;
 		}
 
 		if (
@@ -164,11 +175,20 @@ export class FilesConfigurationService extends Disposable implements IFilesConfi
 
 		// configured glob patterns win over stat information
 		if (this.readonlyIncludeMatcher.value.matches(resource)) {
-			return !this.readonlyExcludeMatcher.value.matches(resource);
+			return !this.readonlyExcludeMatcher.value.matches(resource) ? FilesConfigurationService.READONLY_MESSAGES.configuredReadonly : false;
 		}
 
-		// finally check for stat information
-		return (this.configuredReadonlyFromPermissions && stat?.locked) ?? stat?.readonly ?? false;
+		// check if file is locked and configured to treat as readonly
+		if (this.configuredReadonlyFromPermissions && stat?.locked) {
+			return FilesConfigurationService.READONLY_MESSAGES.fileLocked;
+		}
+
+		// check if file is marked readonly from the file system provider
+		if (stat?.readonly) {
+			return FilesConfigurationService.READONLY_MESSAGES.fileReadonly;
+		}
+
+		return false;
 	}
 
 	async updateReadonly(resource: URI, readonly: true | false | 'toggle' | 'reset'): Promise<void> {
