@@ -743,7 +743,7 @@ export class TunnelModel extends Disposable {
 	async close(host: string, port: number, reason: TunnelCloseReason): Promise<void> {
 		const key = makeAddress(host, port);
 		const oldTunnel = this.forwarded.get(key)!;
-		if (reason === TunnelCloseReason.AutoForwardEnd) {
+		if ((reason === TunnelCloseReason.AutoForwardEnd) && oldTunnel && (oldTunnel.source.source === TunnelSource.Auto)) {
 			this.sessionCachedProperties.set(key, {
 				local: oldTunnel.localPort,
 				name: oldTunnel.name,
@@ -894,13 +894,14 @@ export class TunnelModel extends Disposable {
 		const matchingCandidates: Map<number, CandidatePort> = new Map();
 		const pidToPortsMapping: Map<number | undefined, number[]> = new Map();
 		forwardedPorts.forEach(forwardedPort => {
-			const matchingCandidate = mapHasAddressLocalhostOrAllInterfaces<CandidatePort>(this._candidates ?? new Map(), LOCALHOST_ADDRESSES[0], forwardedPort.port);
+			const matchingCandidate = mapHasAddressLocalhostOrAllInterfaces<CandidatePort>(this._candidates ?? new Map(), LOCALHOST_ADDRESSES[0], forwardedPort.port) ?? forwardedPort;
 			if (matchingCandidate) {
 				matchingCandidates.set(forwardedPort.port, matchingCandidate);
-				if (!pidToPortsMapping.has(matchingCandidate.pid)) {
-					pidToPortsMapping.set(matchingCandidate.pid, []);
+				const pid = isCandidatePort(matchingCandidate) ? matchingCandidate.pid : undefined;
+				if (!pidToPortsMapping.has(pid)) {
+					pidToPortsMapping.set(pid, []);
 				}
-				pidToPortsMapping.get(matchingCandidate.pid)?.push(forwardedPort.port);
+				pidToPortsMapping.get(pid)?.push(forwardedPort.port);
 			}
 		});
 
@@ -964,6 +965,13 @@ export interface CandidatePort {
 	pid?: number;
 }
 
+export function isCandidatePort(candidate: any): candidate is CandidatePort {
+	return candidate && 'host' in candidate && typeof candidate.host === 'string'
+		&& 'port' in candidate && typeof candidate.port === 'number'
+		&& (!('detail' in candidate) || typeof candidate.detail === 'string')
+		&& (!('pid' in candidate) || typeof candidate.pid === 'string');
+}
+
 export interface IRemoteExplorerService {
 	readonly _serviceBrand: undefined;
 	onDidChangeTargetType: Event<string[]>;
