@@ -138,7 +138,7 @@ export abstract class AbstractTextFileService extends Disposable implements ITex
 						color: listErrorForeground,
 						letter: Codicon.lockSmall,
 						strikethrough: true,
-						tooltip: localize('readonlyAndDeleted', "Deleted, Read Only"),
+						tooltip: localize('readonlyAndDeleted', "Deleted, Read-only"),
 					};
 				}
 
@@ -146,7 +146,7 @@ export abstract class AbstractTextFileService extends Disposable implements ITex
 				else if (isReadonly) {
 					return {
 						letter: Codicon.lockSmall,
-						tooltip: localize('readonly', "Read Only"),
+						tooltip: localize('readonly', "Read-only"),
 					};
 				}
 
@@ -507,7 +507,7 @@ export abstract class AbstractTextFileService extends Disposable implements ITex
 		let sourceTextModel: ITextModel | undefined = undefined;
 		if (sourceModel instanceof BaseTextEditorModel) {
 			if (sourceModel.isResolved()) {
-				sourceTextModel = sourceModel.textEditorModel;
+				sourceTextModel = withNullAsUndefined(sourceModel.textEditorModel);
 			}
 		} else {
 			sourceTextModel = sourceModel as ITextModel;
